/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.basis;

public class SKT {

    /** Die Steigerungsspalte A*. */
	public static final int Astern = 0;
    /** Die Steigerungsspalte A. */
    public static final int A      = 1;
    /** Die Steigerungsspalte B. */
    public static final int B      = 2;
    /** Die Steigerungsspalte C. */
    public static final int C      = 3;
    /** Die Steigerungsspalte D. */
    public static final int D      = 4;
    /** Die Steigerungsspalte E. */
    public static final int E      = 5;
    /** Die Steigerungsspalte F. */
    public static final int F      = 6;
    /** Die Steigerungsspalte G. */
    public static final int G      = 7;
    /** Die Steigerungsspalte H. */
    public static final int H      = 8;

    /** Die Aktivierungskosten, die bei der Erschaffung zu zahlen sind. */
    private static final int[] STEIGERUNGSFAKTOREN =
            //A* A  B  C  D  E  F   G   H
            { 1, 1, 2, 3, 4, 5, 8, 10, 20 };

    /**
     * Die Text-Repraesentation Die Steigerungsspalten.
     * Der Text fuer Spalte {@link SKT#A} findet sich an
     * {@code SPALTEN_STRING[ SKT.A ]}.
     * */
    private static final String[] SPALTEN_STRING =
            { "(A*)", "(A)", "(B)", "(C)", "(D)", "(E)", "(F)", "(G)", "(H)" };

    private static final int[][] KOSTEN = {
    /*Talentwert:AKT,  1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,   31*/
    /* A* */    {  5,  1,   1,   1,   2,   4,   5,   6,   8,   9,  11,  12,  14,  15,  17,  19,  20,  22,  24,  25,  27,  29,  31,  32,  34,  36,  38,  40,  42,  43,  45,   48},
    /* A  */    {  5,  1,   2,   3,   4,   6,   7,   8,  10,  11,  13,  14,  16,  17,  19,  21,  22,  24,  26,  27,  29,  31,  33,  34,  36,  38,  40,  42,  44,  45,  47,   50},
    /* B  */    { 10,  2,   4,   6,   8,  11,  14,  17,  19,  22,  25,  28,  32,  35,  38,  41,  45,  48,  51,  55,  58,  62,  65,  69,  73,  76,  80,  84,  87,  91,  95,  100},
    /* C  */    { 15,  2,   6,   9,  13,  17,  21,  25,  29,  34,  38,  43,  47,  51,  55,  60,  65,  70,  75,  80,  85,  95, 100, 105, 110, 115, 120, 125, 130, 135, 140,  150},
    /* D  */    { 20,  3,   7,  12,  17,  22,  27,  33,  39,  45,  50,  55,  65,  70,  75,  85,  90,  95, 105, 110, 115, 125, 130, 140, 145, 150, 160, 165, 170, 180, 190,  200},
    /* E  */    { 25,  4,   9,  15,  21,  28,  34,  41,  48,  55,  65,  70,  80,  85,  95, 105, 110, 120, 130, 135, 145, 155, 165, 170, 180, 190, 200, 210, 220, 230, 240,  250},
    /* F  */    { 40,  6,  14,  22,  32,  41,  50,  60,  75,  85,  95, 105, 120, 130, 140, 155, 165, 180, 195, 210, 220, 230, 250, 260, 270, 290, 300, 310, 330, 340, 350,  375},
    /* G  */    { 50,  8,  18,  30,  42,  55,  70,  85,  95, 110, 125, 140, 160, 175, 190, 210, 220, 240, 260, 270, 290, 310, 330, 340, 360, 380, 400, 420, 440, 460, 480,  500},
    /* H  */    {100, 16,  35,  60,  85, 110, 140, 165, 195, 220, 250, 280, 320, 350, 380, 410, 450, 480, 510, 550, 580, 620, 650, 690, 720, 760, 800, 830, 870, 910, 950, 1000}
    };

    /**
     * Liefert einen Eintrag aus der Steigerungskostentabelle.
     *
     * @param steigerungsspalte Der Steigerungsfaktor und damit die Spalte der
     *      Tabelle. Ein Faktor jenseits von H/A* wird als H/A* behandelt.
     * @param stufe Zeile der Tabelle. Werte unter 1 liefern die
     *      Aktivierungskosten.
     */
    public static int getKosten( int stufe, int steigerungsspalte ) {

        if ( 0 > stufe ) {
            stufe = 0;
        } else if ( 31 < stufe ) {
            stufe = 31;
        }

        if ( Astern > steigerungsspalte ) {

            steigerungsspalte = Astern;

        } else if ( H < steigerungsspalte ) {

            steigerungsspalte = H;

        }

        return KOSTEN[ steigerungsspalte ][ stufe ];
    }

    /**
     * Liefert die Ap, die ein Held bezahlen muss um ein Talent einer
     * bestimmten Steigerungskategorie zu aktivieren.
     *
     * @param steigerungsspalte des zu aktivierenden Talents.
     * @return die Aktivierungskosten.
     */
    public static int getAktivierungskosten( int steigerungsspalte ) {

        if ( Astern > steigerungsspalte ) {

            steigerungsspalte = Astern;

        } else if ( H < steigerungsspalte ) {

            steigerungsspalte = H;

        }

        return KOSTEN[ steigerungsspalte ][ 0 ];
    }

    /**
     * Liefert die Ap, die ein Held <strong>bei der Erschaffung</strong>
     * bezahlen muss, um ein Talent einer bestimmten Steigerungskategorie
     * zu aktivieren.
     *
     * @param steigerungsspalte des zu aktivierenden Talents.
     * @return die Aktivierungskosten.
     */
    public static int getAktivierungskostenErschaffung( int steigerungsspalte ){

        if ( Astern > steigerungsspalte ) {

            steigerungsspalte = Astern;

        } else if ( H < steigerungsspalte ) {

            steigerungsspalte = H;

        }

        return STEIGERUNGSFAKTOREN[ steigerungsspalte ];
    }

    /**
     * @param steigerungsspalte eines Talents.
     * @return den Steigerungsfaktor einer Steigerungsspalte.
     */
    public static int getSteigerungsfaktor( int steigerungsspalte ){

        if ( Astern > steigerungsspalte ) {

            steigerungsspalte = Astern;

        } else if ( H < steigerungsspalte ) {

            steigerungsspalte = H;

        }

        return STEIGERUNGSFAKTOREN[ steigerungsspalte ];
    }

    /**
     * Liefert die String-Repraesentation eines Steigerungsfaktors.
     *
     * @param steigerungsspalte
     *            zu dem der String gesucht ist.
     * @return die String-Repraesentation des Steigerungsfaktors.
     */
    public static final String toString( int steigerungsspalte ) {

        if ( steigerungsspalte < Astern ) {

            steigerungsspalte = Astern;

        } else if ( steigerungsspalte > H ) {

            steigerungsspalte = H;

        }

        return SPALTEN_STRING[ steigerungsspalte ];

    }
}
