/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.basis.varianten;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.ElementMitName;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;

class VariantenBegabungTalent implements Varianten {

    /** Die Talente. */
    private final List< Variante > talente;

    VariantenBegabungTalent() {
        // Sicher gehen, dass die Talent auch initialisiert sind.
        final Talent talent = Talent.ABRICHTEN;
        talent.getName();

        talente = new ArrayList< Variante >();

        talente.addAll( VariantenUtil.konvertieren( Talentgruppe.GESELLSCHAFT ) );
        talente.addAll( VariantenUtil.konvertieren( Talentgruppe.HANDWERK ) );
        talente.addAll( VariantenUtil.konvertieren( Talentgruppe.KAMPF ) );
        talente.addAll( VariantenUtil.konvertieren( Talentgruppe.KOERPER ) );
        talente.addAll( VariantenUtil.konvertieren( Talentgruppe.NATUR ) );
        talente.addAll( VariantenUtil.konvertieren( Talentgruppe.WISSEN ) );
    }

    /**
     * @throws UnsupportedOperationException
     *             wenn die Methode aufgerufen wird.
     */
    public void einfuegen( final Variante variante ) {
        throw new UnsupportedOperationException(
                "Bei der VarianteBegabungTalent kann keine neue Variante " +
                "eingefuegt werden.");
    }

    /**
     * @return Die zum Objekt passende Variante.
     */
    public Variante getVariante( Object objekt ) {
        
        if ( objekt instanceof ElementMitName ) {
            
            return new Variante( (ElementMitName) objekt );
        
        } else {
            
            return (Variante) objekt;
        }
    }
    
    /**
     * @param held wird nicht benoetigt und darf {@code null} sein.
     * @return eine Liste von Talenten.
     */
    public List< Variante > getVarianten(final Held held) {

        Collections.sort( talente, Variante.NAME_COMPARATOR );

        return new ArrayList< Variante >( talente );
    }

    /** @return {@code true} */
    public boolean hatVarianten() {
        return true;
    }

    /** @return {@code false} */
    public boolean istFreieVariante() {
        return false;
    }

    /** Eine Aufzaehlung aller Varianten. */
    @Override
    public final String toString() {
        return getVarianten( null ).toString();
    }
}
