/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bernhardjung.dsaprogramm.basis.varianten;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.ElementMitNameImpl;

public class VariantenUtil {

    private VariantenUtil() {}

    /**
     * Prueft ob es in einer Liste von Varianten eine Variante mit einem 
     * bestimmten Namen gibt.
     * 
     * @param varianten Die Liste, die durchsucht wird.
     * @param variante Die Variante nach deren Name gesucht wird.
     * @return 
     */
    public static boolean enthaeltVarianteMitNamen(
            final List< Variante > varianten,
            final Variante variante ) {

        final String name = variante.getName();
        
        for ( final Variante existierendeVariante : varianten ) {
            
            if ( existierendeVariante.getName().equals( name ) ) {
                
                return true;
            }
        }
        
        return false;
    }

    /**
     * Sucht aus einer Liste von Varianten eine Variante mit einem bestimmten 
     * Namen heraus.
     * 
     * @param existierendeVarianten Die Liste aus der die Variante herausgesucht
     *          wird.
     * @param neueVariante Es wird eine Variante mit dem Namen dieser Variante
     *          in den existierenden Varianten gesucht.
     * @return Eine Variante aus den existierenden Varianten oder die
     *         {@code neueVariante}, wenn keine passende Variante gefunden wird.
     */
    public static Variante getVarianteMitNamen( 
            final List< Variante > existierendeVarianten,
            final Variante neueVariante ) {
        
        final String name = neueVariante.getName();
        
        for ( final Variante existierendeVariante : existierendeVarianten ) {
            
            if ( existierendeVariante.getName().equals( name ) ) {
                
                return existierendeVariante;
            }
        }
        
        return neueVariante;
    }
        
    /**
     * Verpackt die uebergebenen Strings als Varianten und liefert sie zurueck. 
     * 
     * @param strings Die als Varianten verpackt werden sollen.
     * @return Die Strings als Varianten.
     */
    public static List< Variante > konvertierenStrings( 
            final Collection< String > strings ) {
        
        final List< Variante > varianten = 
            new ArrayList< Variante >( strings.size() );
        
        for( final String string : strings ) {
         
            varianten.add( new Variante( string ) );
        }   
        
        return varianten;
    }
    
    /**
     * Verpackt die uebergebenen Element als Varianten.
     * 
     * @param <T>
     * @param elemente Die Element aus denen die Varianten bestehen sollen.
     * @return Die Elemente als Varianten verpackt.
     */
    public static < T extends ElementMitNameImpl> List< Variante > konvertieren(
            final Collection< T > elemente ) {
        
        final List< Variante > varianten = 
            new ArrayList< Variante >( elemente.size() );
        
        for ( final T element : elemente ) {
            
            varianten.add( new Variante( element ) );
        }
        
        return varianten;
    }
}
