/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.bausteine.util;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_EIGENSCHAFT;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_MODIFIKATION;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_EIGENSCHAFT_MOD;
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;

public class EigenschaftsModifikation {

    /** Die Eigenschaft die die Modifikation erhaelt. */
    private final Eigenschaft eigenschaft;

    /** Die Modifikation die vergeben wird. */
    private final int modifikation;

    /**
     * @param dieEigenschaft die die Modifikation erhaelt.
     * @param dieModifikation die auf die Eigenschaft vergeben wird.
     */
    public EigenschaftsModifikation(
            final Eigenschaft dieEigenschaft,
            final int dieModifikation ) {

        eigenschaft = dieEigenschaft;

        modifikation = dieModifikation;
    }

    /**
     * @return die GP-Kosten, fuer den Baustein, die durch diese Modifikation
     *         entstehen.
     */
    public double getGpKosten() {
        return modifikation * 2.0;
    }

    /**
     * @return die Eigenschaft auf die die Modifikation vergeben wird.
     */
    public final Eigenschaft getEigenschaft() {
        return eigenschaft;
    }

    /**
     * @return der Bonus auf die Eigenschaft.
     */
    public final int getModifikation() {
        return modifikation;
    }

    /**
     * @return etwas wie &quot;KL -1&quot; oder &quot;KK +2&quot;
     */
    @Override
    public final String toString() {
        return String.format(
                "%1$s %2$+d",
                eigenschaft.getKuerzel(),
                modifikation );
    }

    /**
     * @return ein XML-Element, das die Modifikation repraesentiert.
     */
    public final IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_EIGENSCHAFT_MOD );

        element.setAttribute( ATTR_EIGENSCHAFT, eigenschaft.getId() );

        element.setAttribute( ATTR_MODIFIKATION,
                              String.valueOf( modifikation ) );

        return element;
    }
}
