/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.daten;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Observable;

import de.bernhardjung.dsaprogramm.basis.Fernkampfwaffe;
import de.bernhardjung.dsaprogramm.basis.Nahkampfwaffe;
import de.bernhardjung.dsaprogramm.basis.Ruestung;
import de.bernhardjung.dsaprogramm.basis.Schild;
import de.bernhardjung.dsaprogramm.basis.namen.NameFactory;
import de.bernhardjung.dsaprogramm.basis.namen.Namenliste;
import de.bernhardjung.dsaprogramm.bausteine.Kultur;
import de.bernhardjung.dsaprogramm.bausteine.Profession;
import de.bernhardjung.dsaprogramm.bausteine.Rasse;
import de.bernhardjung.dsaprogramm.gui.DialogFehlermeldung;

public class Datenspeicher extends Observable {

    private static Datenspeicher DATENSPEICHER;

    /**
     * Wird an die Observer gesendet, wenn sich die Liste der Fernkampfwaffen
     * aendert.
     */
    public static final Object FERNKAMPFWAFFEN = new Object();
    
    /**
     * Wird an die Observer gesendet, wenn sich die Liste der Kulturen aendert.
     */
    public static final Object KULTUREN = new Object();

    /**
     * Wird an die Observer gesendet, wenn sich die Liste der Nahkampfwaffen
     * aendert.
     */
    public static final Object NAHKAMPFWAFFEN = new Object();
    
    /**
     * Wird an die Observer gesendet, wenn sich die Liste der NameFactorys
     * aendert.
     */
    public static final Object NAMEFACTORYS = new Object();

    /**
     * Wird an die Observer gesendet, wenn sich die Liste der Namenlisten 
     * aendert.
     */
    public static final Object NAMENLISTEN = new Object();
    
    /**
     * Wird an die Observer gesendet, wenn sich die Liste der Professionen
     * aendern.
     */
    public static final Object PROFESSIONEN = new Object();

    /**
     * Wird an die Observer gesendet, wenn sich die Liste der Rassen aendern.
     */
    public static final Object RASSEN = new Object();

    /**
     * Wird an die Observer gesendet, wenn sich die Liste der Ruestungen 
     * aendert.
     */
    public static final Object RUESTUNGEN = new Object();
    
    /**
     * Wird an die Observer gesendet, wenn sich die Liste der Schilde aendert.
     */
    public static final Object SCHILDE = new Object();
    
    public static final Datenspeicher getInstanz() {
        if ( null == DATENSPEICHER ) {
            try {

                DATENSPEICHER = new Datenspeicher();

            } catch ( final Exception e ) {

                DialogFehlermeldung.zeigeDialog(
                        "Der Datenspeicher konnte nicht initialisiert werden.",
                        e );
            }
        }

        return DATENSPEICHER;
    }

    /** Eine Liste aller Fernwaffen. */
    private final Map< String, Fernkampfwaffe > fernkampfwaffen;

    /** Eine Map in der jede Fernwaffe unter ihrer Id gespeichert ist. */
    private final Map< String, Fernkampfwaffe > fernkampfwaffenMap;

    /** Eine Liste aller Kulturen. */
    private final Map< String, Kultur > kulturen;

    /** Eine Map in der jede Kultur unter ihrer Id gespeichert ist. */
    private final Map< String, Kultur > kulturenMap;

    /** Eine Liste aller Nahkampfwaffen. */
    private final Map< String, Nahkampfwaffe > nahkampfwaffen;

    /** Eine Map in der jede Nahkampfwaffe unter ihrer Id gespeichert ist. */
    private final Map< String, Nahkampfwaffe > nahkampfwaffenMap;

    /** Eine Liste aller NameFactorys. */
    private final Map< String, NameFactory > nameFactorys;
    
    /** Eine Map in der jede NameFactory unter ihrer Id gespeichert ist */
    private final Map< String, NameFactory > nameFactorysMap;

    /** Eine Liste aller Namenliste. */
    private final Map< String, Namenliste > namenlisten;
    
    /** Eine Map in der jede Namenliste unter ihrer Id gespeichert ist */
    private final Map< String, Namenliste > namenlistenMap;
    
    /** Eine Liste aller initialisierten Rassen. */
    private final Map< String, Rasse > rassen;

    /**
     * Eine Map in der jede Rasse unter ihrer ID gespeichert ist.
     * Jede Rasse, die erzeugt wird kommt in diese Map.
     */
    private final Map< String, Rasse > rassenMap;

    /** Eine Liste aller initalisierten Professionen. */
    private final Map< String, Profession > professionen;

    /** Eine Map in der jede Profession unter ihrer Id gespeichert ist. */
    private final Map< String, Profession > professionenMap;

    /** Eine Liste aller initalisierten Ruestungen. */
    private final Map< String, Ruestung > ruestungen;

    /** Eine Map in der jede Ruestung unter ihrer Id gespeichert ist. */
    private final Map< String, Ruestung > ruestungenMap;

    /** Eine Liste aller initialisierten Schilde. */
    private final Map< String, Schild > schilde;

    /** Ein Map in der jeder Schild unter seiner Id gespeicher ist. */
    private final Map< String, Schild > schildeMap;

    private Datenspeicher() throws Exception {

        fernkampfwaffen = new HashMap< String, Fernkampfwaffe >();

        fernkampfwaffenMap = new HashMap< String, Fernkampfwaffe >();

        kulturen = new HashMap< String, Kultur >();

        kulturenMap = new HashMap< String, Kultur >();

        nahkampfwaffen = new HashMap< String, Nahkampfwaffe >();

        nahkampfwaffenMap = new HashMap< String, Nahkampfwaffe >();

        nameFactorys = new HashMap< String, NameFactory >();
        
        nameFactorysMap = new HashMap< String, NameFactory >();
        
        namenlisten = new HashMap< String, Namenliste >();
        
        namenlistenMap = new HashMap< String, Namenliste >();
        
        rassen = new HashMap< String, Rasse >( 6 );

        rassenMap = new HashMap< String, Rasse >();

        professionen = new HashMap< String, Profession >();

        professionenMap = new HashMap< String, Profession >();

        ruestungen = new HashMap< String, Ruestung >();

        ruestungenMap = new HashMap< String, Ruestung >();

        schilde = new HashMap< String, Schild >();

        schildeMap = new HashMap< String, Schild >();
    }

    /**
     * Fuegt der Liste der Fernwaffen eine neue Waffe hinzu.
     *
     * @param waffe
     *            die der Liste der Fernwaffen hinzugefuegt werden soll.
     *            Es kann keine Waffe mehrfach zur Liste hinzugefuegt werden.
     */
    public final void einfuegen( final Fernkampfwaffe waffe ) {

        fernkampfwaffen.put( waffe.getId(), waffe );
    }

    /**
     * Fuegt der Liste der Kulturen eine neue Kultur hinzu.
     *
     * @param kultur
     *            die der Liste der Kulturen hinzugefuegt werden soll. Es kann
     *            keine Kultur mehrfach zur Liste hinzugefuegt werden.
     */
    public final void einfuegen( final Kultur kultur ) {

        kulturen.put( kultur.getId(), kultur );
        
        setChanged();
        notifyObservers( KULTUREN );
    }

    /**
     * Fuegt der Liste der Nahkampfwaffen eine neue Waffe hinzu.
     *
     * @param waffe
     *            die der Liste der Nahkampfwaffen hinzugefuegt werden soll.
     *            Es kann keine Waffe mehrfach zur Liste hinzugefuegt werden.
     */
    public final void einfuegen( final Nahkampfwaffe waffe ) {

        nahkampfwaffen.put( waffe.getId(), waffe );
        
        setChanged();
        notifyObservers( NAHKAMPFWAFFEN );
    }

    /**
     * Fuegt der Liste der Name Factorys eine neue Factory hinzu.
     *
     * @param factory
     *            die der Liste hinzugefuegt werden soll.
     *            Es kann keine Factory mehrfach zur Liste hinzugefuegt werden.
     */
    public final void einfuegen( final NameFactory factory ) {

        nameFactorys.put( factory.getId(), factory );
        
        setChanged();
        notifyObservers( NAMEFACTORYS );
    }

    /**
     * Fuegt der Liste der Namenlisten eine neue Liste hinzu.
     *
     * @param neueListe
     *            die der Liste hinzugefuegt werden soll.
     *            Es kann keine Liste mehrfach zur Liste hinzugefuegt werden.
     */
    public final void einfuegen( final Namenliste neueListe ) {

        namenlisten.put( neueListe.getId(), neueListe);
            
        setChanged();
        notifyObservers( NAMENLISTEN );
    }

    /**
     * Fuegt der Liste der Professionen eine neue Profession hinzu.
     *
     * @param profession
     *            die der Liste der Professionen hinzugefuegt werden soll. Es kann
     *            keine Profession mehrfach zur Liste hinzugefuegt werden.
     */
    public final void einfuegen( final Profession profession ) {

        professionen.put( profession.getId(), profession );

        setChanged();
        notifyObservers( PROFESSIONEN );
    }
    
    /**
     * Fuegt der Liste der Rassen eine neue Rasse hinzu.
     *
     * @param rasse
     *            die der Liste der Rassen hinzugefuegt werden soll. Es kann
     *            keine Rasse mehrfach zur Liste hinzugefuegt werden.
     */
    public final void einfuegen( final Rasse rasse ) {

        rassen.put( rasse.getId(), rasse );

        setChanged();
        notifyObservers( RASSEN );
    }

    /**
     * Fuegt der Liste der Ruestungen eine neue Ruestung hinzu.
     *
     * @param ruestung
     *            die der Liste der Ruestungen hinzugefuegt werden soll. Es kann
     *            keine Ruestung mehrfach zur Liste hinzugefuegt werden.
     */
    public final void einfuegen( final Ruestung ruestung ) {

        ruestungen.put( ruestung.getId(), ruestung );
        
        setChanged();
        notifyObservers( RUESTUNGEN );
    }

    /**
     * Fuegt der Liste der Schilde einen neuen Schild hinzu.
     *
     * @param schild
     *            der der Liste der Schilde hinzugefuegt werden soll.
     *            Es kann kein Schild mehrfach zur Liste hinzugefuegt werden.
     */
    public final void einfuegen( final Schild schild ) {

        schilde.put( schild.getId(), schild );
        
        setChanged();
        notifyObservers( SCHILDE );
    }

    /**
     * Entfernt eine Fernwaffe aus der Liste der Waffen.
     *
     * @param waffe die entfernt werden soll.
     */
    public final void entfernen( final Fernkampfwaffe waffe ) {

        fernkampfwaffen.remove( waffe.getId() );
        
        setChanged();
        notifyObservers( FERNKAMPFWAFFEN );
    }

    /**
     * Entfernt eine Nahkampfwaffe aus der Liste der Waffen.
     *
     * @param waffe die entfernt werden soll.
     */
    public final void entfernen( final Nahkampfwaffe waffe ) {

        nahkampfwaffen.remove( waffe.getId() );
    }

    /**
     * Entfernt eine Factory aus der Liste der NameFactorys.
     * 
     * @param factory die entfernt werden soll.
     */
    public final void entfernen( final NameFactory factory ) {
        
        nameFactorys.remove( factory.getId() );
        
        setChanged();
        notifyObservers( NAMEFACTORYS );
    }

    /**
     * Entfernt eine Namenliste aus der Liste der Namenlisten.
     * 
     * @param liste die entfernt werden soll.
     */
    public final void entfernen( final Namenliste liste ) {
        
        namenlisten.remove( liste.getId() );
        
        setChanged();
        notifyObservers( NAMENLISTEN );
    }
    
    /**
     * Entfernt eine Ruestung aus der Liste der Ruestungen.
     *
     * @param ruestung die entfernt werden soll.
     */
    public final void entfernen( final Ruestung ruestung ) {

        ruestungen.remove( ruestung.getId() );
    }

    public final void entfernen( final Schild schild ) {

        schilde.remove( schild.getId() );
    }
    
    /**
     * Prueft ob es fuer den uebergebenen String eine Ruestung im Speicher
     * gibt.
     *  
     * @param id fuer die geprueft werden soll ob es eine passende Ruestung 
     *          gibt.
     * @return {@code true} wenn es eine Ruestung gibt.
     */
    public final boolean enthaeltRuestung( final String id ) {
        
        return ruestungenMap.containsKey( id );
    }

    /**
     * Liefert eine bestimmte Fernwaffe.
     *
     * @param id
     *            der gesuchten Fernwaffe.
     * @return eine Fernwaffe.
     */
    public final Fernkampfwaffe getFernkampfwaffe( final String id ) {

        if ( ! fernkampfwaffenMap.containsKey( id ) ) {

            fernkampfwaffenMap.put( id, new Fernkampfwaffe( id ) );
        }

        return fernkampfwaffenMap.get( id );
    }

    /**
     * @return eine sortierte Liste aller Fernwaffen.
     */
    public final List< Fernkampfwaffe > getFernkampfwaffen() {

        final List< Fernkampfwaffe > liste = 
            new ArrayList< Fernkampfwaffe >( fernkampfwaffen.values() );
        
        Collections.sort( liste, Fernkampfwaffe.NAME_COMPARATOR );
        
        return liste;
    }

    /**
     * @param id
     *            der gesuchte Kultur.
     * @return die Kultur mit der angegebenen id. Wenn die Kultur noch nicht
     *         existiert wird sie erzeugt.
     */
    public final Kultur getKultur( final String id ) {

        if ( ! kulturenMap.containsKey( id ) ) {

            kulturenMap.put( id, new Kultur( id ) );

        }

        return kulturenMap.get( id );
    }

    /**
     * @return eine sortierte Liste aller Kulturen.
     */
    public final List< Kultur > getKulturen() {
        
        final List< Kultur > liste = 
            new ArrayList< Kultur >( kulturen.values() );
        
        Collections.sort( liste, Kultur.NAME_COMPARATOR );
        
        return liste;
    }

    /**
     * Liefert eine bestimmte Nahkampfwaffe.
     *
     * @param id
     *            der gesuchten Waffe.
     * @return eine Nahkampfwaffe.
     */
    public final Nahkampfwaffe getNahkampfwaffe( final String id ) {

        if ( ! nahkampfwaffenMap.containsKey( id ) ) {

            nahkampfwaffenMap.put( id, new Nahkampfwaffe( id ) );
        }

        return nahkampfwaffenMap.get( id );
    }

    /** @return eine sortierte Liste aller Nahkampfwaffen. */
    public final List< Nahkampfwaffe > getNahkampfwaffen() {
        
        final List< Nahkampfwaffe > liste = 
            new ArrayList< Nahkampfwaffe >( nahkampfwaffen.values() );
        
        Collections.sort(  liste, Nahkampfwaffe.NAME_COMPARATOR );
        
        return liste;
    }
    
    /**
     * Liefert eine bestimmte NameFactory.
     * 
     * @param id der gesuchten Factory.
     * @return Die gesuchte Factory. Wenn sie nicht existiert wird sie erzeugt.
     */
    public final NameFactory getNameFactory( final String id ) {
        
        if ( ! nameFactorysMap.containsKey( id  ) ) {
            
            nameFactorysMap.put( id, new NameFactory( id ) );
        }
        
        return nameFactorysMap.get( id );
    }

    /**
     * @return eine sortierte Liste alle NameFactorys.
     */
    public final List< NameFactory > getNameFactorys() {
        
        final List< NameFactory > liste = 
            new ArrayList< NameFactory >( nameFactorys.values() );
        
        Collections.sort( liste, NameFactory.BEZEICHNUNG_COMPARATOR );
        
        return liste;
    }

    /**
     * Liefert eine bestimmte Namenliste.
     * 
     * @param id der gesuchten Namenliste.
     * @return Die gesuchte Liste. Wenn sie nicht existiert wird sie erzeugt.
     */
    public final Namenliste getNamenliste( final String id ) {
        
        if ( ! namenlistenMap.containsKey( id  ) ) {
            
            namenlistenMap.put( id, new Namenliste( id ) );
        }
        
        return namenlistenMap.get( id );
    }

    /**
     * @return eine sortierte Liste alle Namenlisten.
     */
    public final List< Namenliste > getNamenlisten() {
        
        final List< Namenliste > liste = 
            new ArrayList< Namenliste >( namenlisten.values() );
        
        Collections.sort( liste, Namenliste.BEZEICHNUNG_COMPARATOR );
        
        return liste;
    }
    
    /**
     * @param id
     *            der gesuchte Profession.
     * @return die Profession mit der angegebenen id. Wenn die Profession noch
     *         nicht existiert wird sie erzeugt.
     */
    public final Profession getProfession( final String id ) {

        if ( ! professionenMap.containsKey( id ) ) {

            professionenMap.put( id, new Profession( id ) );

        }

        return professionenMap.get( id );
    }

    /**
     * @return eine sortierte Liste aller Professionen.
     */
    public final List< Profession > getProfessionen() {
        
        final List< Profession > liste = 
            new ArrayList< Profession >( professionen.values() );
        
        Collections.sort( liste, Profession.NAME_COMPARATOR );
        
        return liste;
    }

    /**
     * Liefert eine Rasse mit einer bestimmten Id. Eine durch diese Methode
     * gelieferte Rasse steht im Programm noch nicht zur Verfuegung, d.h. sie
     * wird noch nicht von {@link #getRassen()} geliefert. Dazu muss sie erst
     * mit {@link #einfuegen(Rasse)} oder {@link #setRassen(Collection)} zur
     * Liste der Rassen hinzugefuegt werden.
     *
     * @param id
     *            der gesuchte Rasse.
     * @return die Rasse mit der angegebenen id. Wenn die Rasse noch nicht
     *         existiert wird sie erzeugt.
     */
    public final Rasse getRasse( final String id ) {

        if ( ! rassenMap.containsKey( id ) ) {

            rassenMap.put( id, new Rasse( id ) );

        }

        return rassenMap.get( id );
    }

    /**
     * @return eine sortierte Liste aller Rassen.
     */
    public final List< Rasse > getRassen() {
        
        final List< Rasse > liste = new ArrayList< Rasse >( rassen.values() );
        
        Collections.sort( liste, Rasse.NAME_COMPARATOR );
        
        return liste;
    }

    /**
     * Liefert eine bestimmte Ruestung.
     *
     * @param id
     *            der gesuchten Ruestung.
     * @return eine Ruestung.
     */
    public final Ruestung getRuestung( final String id ) {

        if ( ! ruestungenMap.containsKey( id ) ) {

            ruestungenMap.put( id, new Ruestung( id ) );
        }

        return ruestungenMap.get( id );
    }

    /** @return eine sortierte Liste aller Ruestungen. */
    public final List< Ruestung > getRuestungen() {
        
        final List< Ruestung > liste = 
            new ArrayList< Ruestung >( ruestungen.values() );
        
        Collections.sort( liste, Ruestung.NAME_COMPARATOR );
        
        return liste;
    }

    /**
     * Liefert einen bestimmten Schild.
     *
     * @param id
     *            des gesucht Schildes.
     * @return einen Schild.
     */
    public final Schild getSchild( final String id ) {

        if ( ! schildeMap.containsKey( id ) ) {

            schildeMap.put( id, new Schild( id ) );
        }

        return schildeMap.get( id );
    }

    /** @return eine sortierte Liste aller Schilde. */
    public final List< Schild > getSchilde() {
        
        final List< Schild > liste = new ArrayList< Schild >( schilde.values());
        
        Collections.sort( liste, Schild.NAME_COMPARATOR );
        
        return liste;
    }

    /**
     * @throws Exception
     *             wenn der XML-Parser zum Laden der Daten nicht initialisiert
     *             werden kann.
     */
    public final void ladeDaten() throws Exception {

        Datenlader.ladeDaten();
    }

    /**
     * Bestimmt die ueber den Datenspeicher verfuegbaren Fernwaffen.
     *
     * @param neueWaffen
     *           die im Datenspeicher gespeichert werden.
     */
    final void setFernwaffen( final Collection< Fernkampfwaffe > neueWaffen ) {

        fernkampfwaffen.clear();

        for ( final Fernkampfwaffe waffe : neueWaffen ) {
            
            fernkampfwaffen.put( waffe.getId(), waffe );
        }

        
        setChanged();
        notifyObservers( FERNKAMPFWAFFEN );
    }

    /**
     * Bestimmt die ueber den Datenspeicher verfuegbaren Kulturen.
     *
     * @param neueKulturen
     *            Die Kulturen, die im Datenspeicher gespeichert werden.
     */
    final void setKulturen( final Collection< Kultur > neueKulturen ) {

        kulturen.clear();

        for ( final Kultur kultur : neueKulturen ) {
            
            kulturen.put( kultur.getId(), kultur );
        }

        setChanged();
        notifyObservers( KULTUREN );
    }

    /**
     * @param neueNahkampfwaffen
     *            Die List der Nahkampfwaffen, die im Speicher gespeichert
     *            werden soll.
     */
    final void setNahkampfwaffen(
            final Collection< Nahkampfwaffe > neueNahkampfwaffen ) {

        nahkampfwaffen.clear();

        for ( final Nahkampfwaffe waffe : neueNahkampfwaffen ) {
            
            nahkampfwaffen.put( waffe.getId(), waffe );
        }
        
        setChanged();
        notifyObservers( NAHKAMPFWAFFEN );
    }

    /**
     * @param neueFactorys
     *          Die Liste der NameFactorys die zur Verfuegung stehen soll.
     */
    final void setNameFactorys(
            final Collection< NameFactory > neueFactorys ) {
        
        nameFactorys.clear();

        for ( final NameFactory factory: neueFactorys ) {
            
            nameFactorys.put( factory.getId(), factory );
        }
        
        setChanged();
        notifyObservers( NAMEFACTORYS );
    }

    /**
     * @param neueNamenlisten
     *          Die Liste der Namenlisten die zur Verfuegung stehen soll.
     */
    final void setNamenlisten(
            final Collection< Namenliste > neueNamenlisten ) {
        
        namenlisten.clear();

        for ( final Namenliste liste : neueNamenlisten ) {
            
            namenlisten.put( liste.getId(), liste );
        }
        
        setChanged();
        notifyObservers( NAMENLISTEN );
    }
    
    /**
     * Bestimmt die ueber den Datenspeicher verfuegbaren Rassen.
     *
     * @param neueProfessionen die im Datenspeicher gespeichert werden.
     */
    final void setProfessionen(
            final Collection< Profession > neueProfessionen ) {

        professionen.clear();

        for ( final Profession profession: neueProfessionen ) {
            
            professionen.put( profession.getId(), profession );
        }

        setChanged();
        notifyObservers( PROFESSIONEN );
    }

    /**
     * Bestimmt die ueber den Datenspeicher verfuegbaren Professionen.
     *
     * @param neueRassen die im Datenspeicher gespeichert werden.
     */
    final void setRassen( final Collection< Rasse > neueRassen ) {

        rassen.clear();

        for ( final Rasse rasse : neueRassen ) {
            
            rassen.put( rasse.getId(), rasse );
        }

        setChanged();
        notifyObservers( RASSEN );
    }

    /**
     * Bestimmt die ueber den Datenspeicher verfuegbaren Ruestungen.
     *
     * @param neueRuestungen die im Datenspeicher gespeichert werden.
     */
    final void setRuestungen( final Collection< Ruestung > neueRuestungen ) {
        
        ruestungen.clear();

        for ( final Ruestung ruestung : neueRuestungen ) {
            
            ruestungen.put( ruestung.getId(), ruestung );
        }
        
        setChanged();
        notifyObservers( RUESTUNGEN );
    }


    /**
     * Bestimmt die ueber den Datenspeicher verfuegbaren Schilde.
     *
     * @param neueSchilde die im Datenspeicher gespeichert werden.
     */
    final void setSchilde( final Collection< Schild > neueSchilde ) {

        schilde.clear();

        for ( final Schild schild: neueSchilde ) {
            
            schilde.put( schild.getId(), schild );
        }
        
        setChanged();
        notifyObservers( SCHILDE );
    }

    public final void speichereDaten( final File verzeichnis ) {

        Datenspeicherer.speichereDaten( verzeichnis );
    }
}
