/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.daten.factorys.bausteine;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_NAME;
import net.n3.nanoxml.IXMLElement;

public class FactoryNameIn {

    /**
     * Ermittelt den Namen des Heldenbausteins. Der Name wird in 3 Stufen
     * bestimmt. Als erstes wird geprueft ob ein Name in der XML-Datei angegeben
     * ist. Wird einer gefunden, wird er als Name verwendet. Wird kein Name in
     * der Datei gefunden wird ueber die Id ueberprueft ob in der
     * Lokalisierungsdatei ein Name angegeben ist. Ist auch kein Name in der
     * Lokalisierungsdatei vorhanden wird einfach die Id als Name verwendet.
     *
     * @param wurzelElement
     *            des XML-Dokuments.
     * @param id
     *            des Heldenbausteins.
     * @return den Namen des Heldenbausteins.
     */
    public final String erzeugen(
            final IXMLElement wurzelElement,
            final String id ) {

        final String name;

        final IXMLElement elementName = wurzelElement.getFirstChildNamed( TAG_NAME );

        if ( null != elementName ) {

            name = elementName.getContent();

        } else {

            name = null;
        }

        return name;
    }
}
