/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui;

import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JLabel;
import javax.swing.JPanel;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basiswerte.Basiswert;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

class PanelBasiswerte
extends JPanel 
implements Observer, Uebersetzbar {

    /**
     * Die Ueberschrift fuer die Spalte die den Startwert des Basiswerts
     * anzeigt.
     */
    private static final String STARTWERT = "PanelBasiswerte.Startwert";

    /**
     * Die Ueberschrift fuer Spalte die den Modifikator des Basiswerts anzeigt.
     */
    private static final String MOD = "PanelBasiswerte.Mod";

    /**
     * Die Ueberschrift fuer die Spalte die den Namen des Basiswerts anezeigt.
     */
    private static final String BASISWERT = "PanelBasiswerte.Basiswert";

    /**
     * Die Ueberschrift die den aktuellen Wert des Helden anzeigt.
     */
    private static final String AKTUELL = "PanelBasiswerte.Aktuell";

    /**
     * Die Label welche die Basisstufen des Basiswerts anzeigen.
     * Die Label in der Liste sind den Basiswerten ueber ihren Index
     * zugeordnet d.h. an Position i steht das Label fuer den Basiswert mit
     * index i.
     */
    private final List< JLabel > basisstufeLabel;

    /**
     * Die Label welche die Stufen der Basiswerte anzeigen.
     * Die Label in der Liste sind den Basiswerten ueber ihren Index
     * zugeordnet d.h. an Position i steht das Label fuer den Basiswert mit
     * index i.
     */
    private final List< JLabel > gesamtstufeLabel;

    /**
     * Die Label welche die Modifikationen der Basiswerte anzeigen.
     * Die Label in der Liste sind den Basiswerten ueber ihren Index
     * zugeordnet d.h. an Position i steht das Label fuer den Basiswert mit
     * index i.
     */
    private final List< JLabel > modifikationLabel;
    
    private final Uebersetzer uebersetzer;
    
    PanelBasiswerte() {

        uebersetzer = new Uebersetzer();
        
        basisstufeLabel = new ArrayList< JLabel >();

        gesamtstufeLabel = new ArrayList< JLabel >();

        modifikationLabel = new ArrayList< JLabel >();
        
        final List< JLabel > nameLabel = new ArrayList< JLabel >();
        
        final Basiswert[] basiswerte = Basiswert.getBasiswerte();
        
        for ( int i = 0, n = basiswerte.length; i < n ; ++i ) {
            
            basisstufeLabel.add( Util.erzeugeLabelRechtsbuendig() );
            
            gesamtstufeLabel.add( Util.erzeugeLabelRechtsbuendig() );
            
            modifikationLabel.add( Util.erzeugeLabelRechtsbuendig() );
            
            nameLabel.add( uebersetzer.erzeugeLabel( 
                    basiswerte[ i ].getLokalisierungsschluesselName() ) );
        }

        final JLabel ueberschriftAktuell = 
            uebersetzer.erzeugeUeberschrift( AKTUELL );
        
        final JLabel ueberschriftBasiswert = 
            uebersetzer.erzeugeUeberschrift( BASISWERT );
        
        final JLabel ueberschriftModifikator = 
            uebersetzer.erzeugeUeberschrift( MOD );
            
        final JLabel ueberschriftStartwert = 
            uebersetzer.erzeugeUeberschrift( STARTWERT );
        
        final FormLayout layout = new FormLayout(
                // Basiswert      Mod        +       Basis       =       Gesamt
                "7dlu, P:G, 4dlu, P:G, 4dlu, P, 4dlu, P:G, 4dlu, P, 4dlu, P:G, 7dlu",
                "7dlu, P,   4dlu, P:G, 4dlu, P:G, 4dlu, P:G, 4dlu, P:G, " +
                "4dlu, P:G, 4dlu, P:G, 4dlu, P:G, 4dlu, P:G, 4dlu, P:G, 7dlu" );

        setLayout( layout );

        final CellConstraints cc = new CellConstraints();

        add( ueberschriftBasiswert, cc.xy(  2, 2 ) );
        add( ueberschriftModifikator, cc.xy(  4, 2 ) );
        add( ueberschriftStartwert, cc.xy(  8, 2 ) );
        add( ueberschriftAktuell, cc.xy( 12, 2 ) );

        for ( int i = 0 ; i < basiswerte.length ; ++i ) {

            final int y = 2 * i + 4;
            int x = 0;

            add( nameLabel.get( i ), cc.xy( x+=2, y ) );
            add( modifikationLabel.get( i ),              cc.xy( x+=2, y ) );
            add( new JLabel( "+" ),                       cc.xy( x+=2, y ) );
            add( basisstufeLabel.get( i ),                cc.xy( x+=2, y ) );
            add( new JLabel( "=" ),                       cc.xy( x+=2, y ) );
            add( gesamtstufeLabel.get( i ),               cc.xy( x+=2, y ) );
        }
    }

    public void neuUebersetzen() {

        uebersetzer.neuUebersetzen();
    }
    
    /**
     * Aendert den Held dessen Werte im Panel dargestellt werden.
     * @param held
     */
    final void setHeld( final Held held ) {
        held.addObserver( this );

        update( held, null );
    }

    /**
     * Aktualisiert die angezeigten Werte.
     *
     * @param o der Held der ueberwacht wird.
     * @param arg ungenutzt
     */
    public void update( final Observable o, final Object arg ) {

        final Held held = (Held) o;

        for ( final Basiswert basiswert : Basiswert.getBasiswerte() ) {

            final int index = basiswert.getIndex();

            String text = String.valueOf( basiswert.getBasisstufe( held ) );
            basisstufeLabel.get( index ).setText( text );

            text = String.valueOf( held.getStufe( basiswert ) );
            gesamtstufeLabel.get( index ).setText( text );

            text = String.valueOf( basiswert.getModifikation( held ) );
            modifikationLabel.get( index ).setText( text );
        }
    }

}
