/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Augenfarbe;
import de.bernhardjung.dsaprogramm.basis.Haarfarbe;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.namen.NameFactory;
import de.bernhardjung.dsaprogramm.bausteine.Kultur;
import de.bernhardjung.dsaprogramm.bausteine.Rasse;
import de.bernhardjung.dsaprogramm.bausteine.util.Geschlecht;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

public class PanelHeldendaten
extends ResetbaresPanel 
implements Observer, Uebersetzbar {
    
    private static final String AUGENFARBE = "PanelHeldendaten.Augenfarbe";

    private static final String AUSSEHEN = "PanelHeldendaten.Aussehen";

    private static final String GEBURTSTAG = "PanelHeldendaten.Geburtstag";

    private static final String GESCHLECHT = "PanelHeldendaten.Geschlecht";

    private static final String GEWICHT = "PanelHeldendaten.Gewicht";

    private static final String GROESSE = "PanelHeldendaten.Groesse";

    private static final String HAARFARBE = "PanelHeldendaten.Haarfarbe";

    private static final String HALBFINGER = "PanelHeldendaten.Halbfinger";

    private static final String HINTERGRUND = "PanelHeldendaten.Hintergrund";

    private static final String JAHRE = "PanelHeldendaten.Jahre";

    private static final String KULTUR = "PanelHeldendaten.Kultur";

    private static final String NAME = "PanelHeldendaten.Name";

    private static final String PROFESSION = "PanelHeldendaten.Profession";

    private static final String RASSE = "PanelHeldendaten.Rasse";

    private static final String STAND = "PanelHeldendaten.Stand";

    private static final String STEIN = "PanelHeldendaten.Stein";

    private static final String TITEL = "PanelHeldendaten.Titel";

    private static final String ALTER = "PanelHeldendaten.Alter";

    /**
     * Die Beschriftung fuer die diversen Knopfe mit denen sich irgenwelche
     * Eigenschaften des Heldes auswuerfeln lassen.
     */
    private static final String AUSWUERFELN = "PanelHeldendaten.Auswuerfeln";

    /**
     * Ein Spinner mit dem man das Alter des Helden in Jahren einstellen kann.
     */
    private final JSpinner alter;

    /**
     * Ein Knopf mit dem man das Alter des Helden auswuerfeln lassen kann.
     */
    private final JButton alterAuswuerfeln;
    
    /** Eine ComboBox in der man die Augenfarbe waehlen kann. */
    private final JComboBox augenfarbe;

    /** Ein Knopf um die Augenfarbe auswuerfeln zu lassen */
    private final JButton augenfarbeAuswuerfeln;

    /**
     * Ein Textfeld in das eine kurze Beschreibung des Aussehens des Helden
     * eingetragen werden kann.
     */
    private final JTextField aussehen;

    /**
     * Ein Textfeld in das der Geburtstag des Helden eingetragen werden kann.
     */
    private final JTextField geburtstag;
    
    /**
     * Eine Combobox mit der das Geschlecht des Helden/der Heldin gewaehlt
     * werden kann.
     */
    private final JComboBox geschlecht;
    
    /**
     * Ein Spinner mit dem das Gewicht des Helden gewaehlt werden kann.
     */
    private final JSpinner gewicht;

    /**
     * Ein Knopf um das Gewicht des Helden auszwuerfeln.
     * Die Option ist nötig für den Fall, dass der Held fettleibig ist.
     */
    private final JButton gewichtAuswuerfeln;
    
    /** Ein Spinner um die Groesse des Helden zu waehlen. */
    private final JSpinner groesse;

    /** Ein Knopf um die groesse des Helden auswuerfen zu lassen */
    private final JButton groesseAuswuerfeln;
    
    /** Eine Combobox mit der die Haarfarbe des Helden gewaehlt werden kann. */
    private final JComboBox haarfarbe;

    /** Ein Knopf um die Haarfarbe des Helden auswuerfeln zu lassen. */
    private final JButton haarfarbeAuswuerfeln;
    
    /** Der Held der gerade erschaffen wird. */
    private Held held;
    
    /**
     * Ein Textfeld in das Informationen zu Familie, Herkunft und Hintergrund
     * des Helden eingegeben werden koennen.
     */
    private final JTextField hintergrund;
    
    /** Das Label das den Namen der Kultur des Helden anzeigt. */
    private final JLabel kultur;
    
    /** Ein Textfeld in das der Name des Helden eingegeben werden kann. */
    private final JTextField name;
    
    /** 
     * Die Combobox mit der man die Factory auswaehlt, um einen Namen fuer den
     * Helden zu erzeugen.
     */
    private final JComboBox nameFactorys;
    
    /** Ein Label das den Namen der Profession des Helden anzeigt. */
    private final JLabel profession;
    
    /** Ein Label das den Namen der Rasse des Helden anzeigt. */
    private final JLabel rasse;
    
    
    
    /** Ein Texfeld in das der Stand des Helden eingetragen werden kann. */
    private final JTextField stand;
    
    /** Ein Texfeld in das der Titel des Helden eingetragen werden kann. */
    private final JTextField titel;

    private Uebersetzer uebersetzer;
    
    public PanelHeldendaten() {

        Datenspeicher.getInstanz().addObserver( this );
        
        uebersetzer = new Uebersetzer();
    
        
        
        alter = Util.erzeugeSpinner( 1, 1, 999, 1 );
        alter.addChangeListener( new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                held.setAlter( (Integer) alter.getValue() );
            }
        } );

        alterAuswuerfeln = uebersetzer.erzeugeButton( AUSWUERFELN );
        alterAuswuerfeln.addActionListener( new ActionListener(){
            public void actionPerformed( final ActionEvent e ) {
                held.setAlter( held.getRasse().getAlter().auswuerfeln( held ) );
            }
        });
        
        augenfarbe = uebersetzer.erzeugeComboBox( 
                Augenfarbe.getAugenfarben(),
                Augenfarbe.NAME_COMPARATOR );
        augenfarbe.setEditable( true );
        augenfarbe.addKeyListener( new KeyAdapter() {
            @Override
            public void keyTyped( final KeyEvent e ) {
                
                held.setAugenfarbe( getAugenfarbe() );
            }
        } );
        augenfarbe.addActionListener( new ActionListener(){
            public void actionPerformed( final ActionEvent e ) {
                
                held.setAugenfarbe( getAugenfarbe() );
            }
        } );

        augenfarbeAuswuerfeln = uebersetzer.erzeugeButton( AUSWUERFELN );
        augenfarbeAuswuerfeln.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                
                final String id = held.getRasse().getAugenfarben().wuerfeln();
                
                held.setAugenfarbe( Augenfarbe.getAugenfarbe( id ) );
            }
        } );
        
        aussehen = new JTextField();
        aussehen.addFocusListener( new FocusAdapter() {
            @Override
            public void focusLost( final FocusEvent e ) {
                held.setAussehen( aussehen.getText() );
            }
        } );

        geburtstag = new JTextField();
        geburtstag.addFocusListener( new FocusAdapter() {
            @Override
            public void focusLost( final FocusEvent e ) {
                held.setGeburtstag( geburtstag.getText() );
            }
        } );
        
        geschlecht = uebersetzer.erzeugeComboBox( Geschlecht.values() );
        geschlecht.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                held.setGeschlecht(
                        (Geschlecht) geschlecht.getSelectedItem() );
            }
        } );
        
        gewicht = Util.erzeugeSpinner( 1, 1, Integer.MAX_VALUE, 1 );
        gewicht.addChangeListener( new ChangeListener() {
            public void stateChanged( final ChangeEvent e ) {
                held.setGewicht( (Integer) gewicht.getValue() );
            }
        } );

        gewichtAuswuerfeln = uebersetzer.erzeugeButton( AUSWUERFELN );
        gewichtAuswuerfeln.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                held.setGewicht( held.getRasse().getGewicht().berechnen( held));
            }
        } );
        
        groesse = Util.erzeugeSpinner( 1, 1, Integer.MAX_VALUE, 1 );
        groesse.addChangeListener( new ChangeListener() {
            public void stateChanged( final ChangeEvent e ) {
                held.setGroesse( (Integer) groesse.getValue() );
                final int dasGewicht;
                dasGewicht = held.getRasse().getGewicht().berechnen( held );
                held.setGewicht( dasGewicht );
            }
        } );

        groesseAuswuerfeln = uebersetzer.erzeugeButton( AUSWUERFELN );
        groesseAuswuerfeln.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                held.setGroesse( held.getRasse().getGroesse().wuerfeln() );
            }
        } );

        haarfarbe = uebersetzer.erzeugeComboBox( 
                Haarfarbe.getHaarfarben(),
                Haarfarbe.NAME_COMPARATOR );
        haarfarbe.setEditable( true );
        haarfarbe.addFocusListener( new FocusAdapter() {
            @Override
            public void focusLost( final FocusEvent e ) {
                
                held.setHaarfarbe( getHaarfarbe() );
            }
        } );
        haarfarbe.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                
                held.setHaarfarbe( getHaarfarbe() );
            }
        } );

        haarfarbeAuswuerfeln = uebersetzer.erzeugeButton( AUSWUERFELN );
        haarfarbeAuswuerfeln.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                
                final String id = held.getRasse().getHaarfarben().wuerfeln();
                
                held.setHaarfarbe( Haarfarbe.getHaarfarbe( id ) );
            }
        } );

        held = new Held();

        hintergrund = new JTextField();
        hintergrund.addFocusListener( new FocusAdapter() {
            @Override
            public void focusLost( final FocusEvent e ) {
                held.setHintergrund( hintergrund.getText() );
            }
        } );
        
        kultur = new JLabel();

        name = new JTextField();
        name.addFocusListener( new FocusAdapter() {
            @Override
            public void focusLost( final FocusEvent e ) {
                held.setName( name.getText() );
            }
        } );
        
        nameFactorys = new JComboBox();
        updateNameFactorys();
        
        profession = new JLabel();
        
        rasse = new JLabel();

        stand = new JTextField();
        stand.addFocusListener( new FocusAdapter() {
            @Override
            public void focusLost( final FocusEvent e ) {
                held.setStand( stand.getText() );
            }
        } );

        titel = new JTextField();
        titel.addFocusListener( new FocusAdapter() {
            @Override
            public void focusLost( final FocusEvent e ) {
                held.setTitel( titel.getText() );
            }
        } );

        aufbauenBenutzeroberflaeche();
    }

    /** Baut die Benutzeroberlfaeche auf. Ordnet alle Komponenten an. */
    private final void aufbauenBenutzeroberflaeche() {
        
        final FormLayout layout = new FormLayout(
              // 1    2        3          4   5    6   7
                "M, 4dlu, 30dlu:G(0.5), 2dlu, M, 4dlu, M, " + // 1. Hauptspalte
                "8dlu, " +
                "M, 4dlu, 30dlu:G(0.5), 2dlu, M, 4dlu, M"    // 2. Hauptspalte
              // 9   10       11         12  13   14  15
                ); // Zeilen werden vom Builder erzeugt.

        // Die 1. Spalte in der linken Hauptspalte soll die gleiche Breite
        // haben wie die 1. in der rechten Hauptspalte.
        // Gleiches gilt fuer alle anderen Spalten.
        layout.setColumnGroups( new int[][]{ { 1,  9 },
                                             { 3, 11 },
                                             { 5, 13 },
                                             { 7, 15 } } );

        final DefaultFormBuilder builder = new DefaultFormBuilder( layout );
        // ein schoener breiter Rahmen um alles
        builder.setDefaultDialogBorder();
        builder.setRowGroupingEnabled( true ); // alle Zeilen gleich hoch.

        // 1. Zeile
        builder.append( uebersetzer.erzeugeLabel( NAME ) );
        builder.append( erzeugePanelName(), 13 );

        // naechste Zeile
        builder.append( uebersetzer.erzeugeLabel( RASSE ) );
        builder.append( rasse, 13 );

        // naechste Zeile
        builder.append( uebersetzer.erzeugeLabel( KULTUR ) );
        builder.append( kultur, 13 );

        // naechste Zeile
        builder.append( uebersetzer.erzeugeLabel( PROFESSION ) );
        builder.append( profession, 13 );

        // naechste Zeile
        // linke Haelfte
        builder.append( uebersetzer.erzeugeLabel( GESCHLECHT ) );
        builder.append( geschlecht, 5 );
        // rechte Haelfte
        builder.append( uebersetzer.erzeugeLabel( GEBURTSTAG ) );
        builder.append( geburtstag, 5 );

        // naechste Zeile
        // linke Haelfte
        builder.append( new JLabel(), 7 );
        // rechte Haelfte
        builder.append( uebersetzer.erzeugeLabel( ALTER ) );
        builder.append( alter );
        builder.append( uebersetzer.erzeugeLabel( JAHRE ) );
        builder.append( alterAuswuerfeln );
        builder.nextLine();

        // naechste Zeile
        // linke Haelfte
        builder.append( uebersetzer.erzeugeLabel( HAARFARBE ) );
        builder.append( haarfarbe, 3 );
        builder.append( haarfarbeAuswuerfeln );
        // rechte Haelfte
        builder.append( uebersetzer.erzeugeLabel( GROESSE ) );
        builder.append( groesse );
        builder.append( uebersetzer.erzeugeLabel( HALBFINGER ) );
        builder.append( groesseAuswuerfeln );

        // naechste Zeile
        // linke Haelfte
        builder.append( uebersetzer.erzeugeLabel( AUGENFARBE ) );
        builder.append( augenfarbe, 3 );
        builder.append( augenfarbeAuswuerfeln );
        // rechte Haelfte
        builder.append( uebersetzer.erzeugeLabel( GEWICHT ) );
        builder.append( gewicht );
        builder.append( uebersetzer.erzeugeLabel( STEIN ) );
        builder.append( gewichtAuswuerfeln );

        // naechste Zeile
        builder.append( uebersetzer.erzeugeLabel( AUSSEHEN ) );
        builder.append( aussehen, 13 );

        // naechste Zeile
        builder.append( uebersetzer.erzeugeLabel( STAND ) );
        builder.append( stand, 13 );

        // naechste Zeile
        builder.append( uebersetzer.erzeugeLabel( TITEL ) );
        builder.append( titel, 13 );

        // naechste Zeile
        builder.append( uebersetzer.erzeugeLabel( HINTERGRUND ) );
        builder.append( hintergrund, 13 );

        final JScrollPane scrollPane;
        
        scrollPane = Util.erzeugeScrollPane( builder.getPanel() );
        resetter.einfuegen( scrollPane );
        
        setLayout( new BorderLayout() );
        
        add( scrollPane, BorderLayout.CENTER );
    }
    
    /**
     * \return Das Textfeld in das man den Namen des Charakters eingeben kann
     *         mit der Combobox und dem Knopf, zum automatischen Erzeugen eines
     *         Namens. 
     */
    private JComponent erzeugePanelName() {

        final JButton nameGenerieren = 
            new JButton( Util.ladeIcon( "generieren" ) );
        nameGenerieren.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                generiereName();
            }
        } );
        
        final FormLayout layout = 
            new FormLayout( "F:P:G, 2dlu, min(200px;P), 2dlu, P" );
        
        final DefaultFormBuilder builder = new DefaultFormBuilder( layout );
        
        builder.append( name );
        
        builder.append( nameFactorys );
        
        builder.append( nameGenerieren );
        
        return builder.getPanel();
    }
    
    /**
     * Erzeugt mit Hilfe der gewaehlten NameFactory einen Namen fuer den 
     * aktuellen Helden. 
     */
    private void generiereName() {
        
        if ( -1 == nameFactorys.getSelectedIndex() ) {
            
            return;
        }
        
        final NameFactory factory = 
            (NameFactory) nameFactorys.getSelectedItem();
        
        String name = factory.erzeugeName( held );
        
        held.setName( name );
    }

    /**
     * @return Die in der Combobox gewaehlte Augenfarbe.
     */
    private Augenfarbe getAugenfarbe() {

        final Object gewaehlt = augenfarbe.getSelectedItem();
        
        final Augenfarbe farbe;
        
        if ( gewaehlt instanceof String ) {
            
            farbe = Augenfarbe.getAugenfarbe( gewaehlt.toString() );
        
        } else {
            
            farbe = (Augenfarbe) gewaehlt;
        }
        
        return farbe;
    }

    /**
     * @return Die in der Combobox gewaehlte Haarfarbe.
     */
    private Haarfarbe getHaarfarbe() {

        final Object gewaehlt = haarfarbe.getSelectedItem();
        
        final Haarfarbe farbe;
        
        if ( gewaehlt instanceof String ) {
            
            farbe = Haarfarbe.getHaarfarbe( gewaehlt.toString() );
        
        } else {
            
            farbe = (Haarfarbe) gewaehlt;
        }
        
        return farbe;
    }
    
    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();

        alterAuswuerfeln.setToolTipText( 
                held.getRasse().getAlter().toString() );

        augenfarbeAuswuerfeln.setToolTipText( "<html>" +
                held.getRasse().getAugenfarben().toHtmlTabelleVertikal() );

        gewichtAuswuerfeln.setToolTipText( 
                held.getRasse().getGewicht().toString() );

        groesseAuswuerfeln.setToolTipText( 
                held.getRasse().getGroesse().toString() );

        haarfarbeAuswuerfeln.setToolTipText( "<html>" +
                held.getRasse().getHaarfarben().toHtmlTabelleVertikal() );

        kultur.setText( held.getKultur().getName() );
        
        profession.setText( held.getProfession().getName( held ) );

        rasse.setText( held.getRasse().getName() );
        
        updateNameFactorys();
    }
    
    /**
     * Bestimmt den Held der gerade erschaffen wird.
     * Die Werte des Helden werden angezeigt und koennen in Folge ueber das
     * Panel editiert werden.
     *
     * @param neuerHeld der gerade erschaffen wird.
     */
    public final void setHeld( final Held neuerHeld ) {

        held = neuerHeld;

        held.addObserver( this );

        final Rasse rasse = held.getRasse();

        alterAuswuerfeln.setToolTipText( rasse.getAlter().toString() );

        augenfarbeAuswuerfeln.setToolTipText( "<html>" +
                rasse.getAugenfarben().toHtmlTabelleVertikal() );

        gewichtAuswuerfeln.setToolTipText( rasse.getGewicht().toString() );

        groesseAuswuerfeln.setToolTipText( rasse.getGroesse().toString() );

        haarfarbeAuswuerfeln.setToolTipText( "<html>" +
                rasse.getHaarfarben().toHtmlTabelleVertikal() );

        Kultur k = held.getKultur();
        
        kultur.setText( k.getName() );

        if ( k.hatNameFactory() )
        {
            nameFactorys.setSelectedItem( k.getNameFactory() );
        }
        
        profession.setText( held.getProfession().getName( held ) );

        this.rasse.setText( rasse.getName() );

        update( null, null );
    }

    /**
     * Aktualisiert die Anzeige wenn Aenderungen am Helden stattgefunden haben.
     * @param o ungenutzt
     * @param arg ungenutzt
     */
    public void update( final Observable o, final Object arg ) {

        if ( Datenspeicher.NAMEFACTORYS == arg ) {
            
            updateNameFactorys();
        
        } else {
        
            KonfliktBeschreibung beschreibung = held.getRasse().pruefen( held );
    
            if ( beschreibung.enthaeltKonflikt() ) {
    
                rasse.setForeground( Util.FEHLERFARBE );
    
                rasse.setToolTipText( beschreibung.toHtml() );
    
            } else {
    
                rasse.setForeground( getForeground() );
    
                rasse.setToolTipText( null );
            }
    
            beschreibung = held.getKultur().pruefen( held );
    
            if ( beschreibung.enthaeltKonflikt() ) {
    
                kultur.setForeground( Util.FEHLERFARBE );
    
                kultur.setToolTipText( beschreibung.toHtml() );
    
            } else {
    
                kultur.setForeground( getForeground() );
    
                kultur.setToolTipText( null );
    
            }
    
            beschreibung = held.getProfession().pruefen( held );
    
            if ( beschreibung.enthaeltKonflikt() ) {
    
                profession.setForeground( Util.FEHLERFARBE );
    
                profession.setToolTipText( beschreibung.toHtml() );
    
            } else {
    
                profession.setForeground( getForeground() );
    
                profession.setToolTipText( null );
            }
    
            profession.setText( held.getProfession().getName( held ) );
    
            alter.setValue( held.getAlter() );
    
            augenfarbe.setSelectedItem( held.getAugenfarbe() );
    
            geburtstag.setText( held.getGeburtstag() );
    
            geschlecht.setSelectedItem( held.getGeschlecht() );
    
            gewicht.setValue( held.getGewicht() );
    
            gewichtAuswuerfeln.setEnabled( held.hat( VorNachteil.FETTLEIBIG ) );
    
            groesse.setValue( held.getGroesse() );
    
            haarfarbe.setSelectedItem( held.getHaarfarbe() );
    
            name.setText( held.getName() );
        }
    }

    /**
     * Aktualisiert den Inhalt der ComboBox mit den NameFactorys. 
     */
    public void updateNameFactorys() {
        
        final List< NameFactory > nameFactorysListe = 
            Datenspeicher.getInstanz().getNameFactorys();
        
        ComboBoxModel model = new DefaultComboBoxModel( 
                nameFactorysListe.toArray() );
        
        nameFactorys.setModel( model );
    }

}
