/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JTabbedPane;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif_lite.panel.SimpleInternalFrame;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.erschaffung.ErschaffungManager;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeitgruppe;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;

public class PanelSonderfertigkeiten
extends ResetbaresPanel 
implements Uebersetzbar {

    private static final String VERBILLIGTE_REITER = "PanelSonderfertigkeiten.Reiter.verbilligte";

    private static final String GEWAEHLTE_REITER = "PanelSonderfertigkeiten.Reiter.Gewaehlte";

    private static final String VERBILLIGTE = "PanelSonderfertigkeiten.verbilligte";

    private static final String GEWAEHLTE = "PanelSonderfertigkeiten.Gewaehlte";

    /** Der Text neben der Combobox mit der man die Lernmethode waehlt. */
    private static final String LERNMETHODE =
        "PanelSonderfertigkeiten.Lernmethode";

    /**
     * Die Liste der Sonderfertigkeiten die der Held besitzt, oder besitzen
     * sollte.
     */
    private final PanelSonderfertigkeitenListe gewaehlte;

    /** Der Held, dessen Werte im Panel angezeigt werden. */
    private Held held;
    
    /** Die Combobox mit der man die Lernmethode waehlt. */
    private final JComboBox lernmethode;

    
    
    /** 
     * Die Liste von Paneln mit den Sonderfertigkeiten aus den 
     * unterschiedlichen Gruppen. 
     */
    private final List< PanelSonderfertigkeitenListe > sonderfertigkeitGruppen;
    
    /** Uebersetzt verschiedenen Label und die Tabbedpane. */
    private final Uebersetzer uebersetzer;
    
    /** Die Liste mit den verbilligten Sonderfertigkeiten. */
    private final PanelSonderfertigkeitenListe verbilligte;
    
    /**
     * Erzeugt das Register mit den einzelnen Listen.
     *
     * @param zumSteigern {@code true}, wenn das Panel beim Steigern eines
     *                    Helden verwendet wird, {@code false}, wenn das Panel
     *                    bei der Heldenerschaffung verwendet wird.
     */
    public PanelSonderfertigkeiten( final boolean zumSteigern ) {

        uebersetzer = new Uebersetzer();
    
        
        
        final JLabel lernmethodeLabel = uebersetzer.erzeugeLabel( 
                LERNMETHODE );
        
        lernmethode = uebersetzer.erzeugeComboBox( 
                Lernmethode.LEHRMEISTER,
                Lernmethode.SPEZIELLE_ERFAHRUNG,
                Lernmethode.KEINE );
        resetter.einfuegen( lernmethode );
        
        final Lernmethoden lernmethoden = new Lernmethoden( lernmethode );

        gewaehlte = new PanelSonderfertigkeitenListe(
                true, zumSteigern, lernmethoden );
        resetter.einfuegen( gewaehlte );
        
        final SimpleInternalFrame gewaehlteFrame = uebersetzer.erzeugeFrame( 
                GEWAEHLTE,
                gewaehlte );

        verbilligte = new PanelSonderfertigkeitenListe(
                false, zumSteigern, lernmethoden );
        resetter.einfuegen( verbilligte );
        
        final SimpleInternalFrame verbilligteFrame = uebersetzer.erzeugeFrame(
                VERBILLIGTE,
                verbilligte );
        
        final JTabbedPane tabbedPane = new JTabbedPane( JTabbedPane.LEFT );
        resetter.einfuegen( tabbedPane );
        
        uebersetzer.erzeugeTab( tabbedPane, GEWAEHLTE_REITER, gewaehlteFrame );

        uebersetzer.erzeugeTab( tabbedPane,VERBILLIGTE_REITER,verbilligteFrame);

        final List< Sonderfertigkeitgruppe > gruppen =
            Sonderfertigkeitgruppe.getGruppen();

        sonderfertigkeitGruppen = 
            new ArrayList< PanelSonderfertigkeitenListe >( gruppen.size() );
        
        for ( final Sonderfertigkeitgruppe gruppe : gruppen ) {

            final List< SonderfertigkeitWert > werte = erzeugeWerte( gruppe );

            final PanelSonderfertigkeitenListe p =
                new PanelSonderfertigkeitenListe(
                        false, zumSteigern, lernmethoden );
            resetter.einfuegen(  p  );
            
            p.setWerte( werte );

            sonderfertigkeitGruppen.add( p );

            final SimpleInternalFrame frame = uebersetzer.erzeugeFrame( 
                    gruppe.getLokalisierungsschluesselName(),
                    p );
            
            uebersetzer.erzeugeTab( 
                    tabbedPane,
                    gruppe.getLokalisierungsschluesselKuerzel(), 
                    frame );
        }

        if ( zumSteigern  ) {

            setLayout( new FormLayout( "P:G, P, 2dlu, P, P:G", "2dlu, P, 2dlu, F:P:G" ) );

            final CellConstraints cc = new CellConstraints();

            add( lernmethodeLabel, cc.xy ( 2, 2 ) );

            add( lernmethode, cc.xy( 4, 2 ) );

            add( tabbedPane, cc.xyw( 1, 4, 5 ) );

        } else {

            setLayout( new BorderLayout() );

            add( tabbedPane, BorderLayout.CENTER );
        }
    }

    /**
     * Erzeugt eine Liste von Werten zu einer Sonderfertigkeit-Gruppe.
     *
     * @param gruppe von Sonderfertigkeiten.
     * @return eine Liste von Werten.
     */
    private final List< SonderfertigkeitWert > erzeugeWerte(
            final List< Sonderfertigkeit > gruppe ) {

        final List< SonderfertigkeitWert > werte =
            new ArrayList< SonderfertigkeitWert >( gruppe.size() );

        for ( final Sonderfertigkeit sonderfertigkeit : gruppe ) {

            if ( sonderfertigkeit.hatVarianten() ) {

                // die Variante bestimmten:
                final Variante variante;

                final List< Variante > varianten =
                    sonderfertigkeit.getVarianten( null );

                if ( 0 == varianten.size() ) {

                    variante = new Variante();

                } else {

                    variante = varianten.get( 0 );
                }

                werte.add( new SonderfertigkeitWert( 
                        sonderfertigkeit,
                        variante ) );

            } else {

                werte.add( new SonderfertigkeitWert( sonderfertigkeit ) );

            }
        }

        return werte;
    }

    public void neuUebersetzen() {

        uebersetzer.neuUebersetzen();
    }
    
    /**
     * Muss zum Steigern eines Helden aufgerufen werden. Bestimmt den Helden
     * der ueber das Panel editiert wird.
     *
     * @param derHeld
     *            der im Panel bearbeitet wird.
     */
    public final void setHeld( final Held held ) {
        
        this.held = held;
        
        gewaehlte.setHeld( held );

        // leitsonderfertigkeiten.setWerte( held.getLeitsonderfertigkeiten() );
        // leitsonderfertigkeiten.setHeld( held );

        final List< SonderfertigkeitWert > verbilligteWerte =
            erzeugeWerte( held.getVerbilligteSonderfertigkeiten() );
        verbilligteWerte.addAll( held.getVerbilligteVarianten() );

        verbilligte.setWerte( verbilligteWerte );

        verbilligte.setHeld( held );

        for ( final PanelSonderfertigkeitenListe p : sonderfertigkeitGruppen ) {

            p.setHeld( held );
        }
    }

    /**
     * Muss bei der Heldenerschaffung aufgerufen werden.
     * Bestimmt den Manager fuer die aktuell Erschaffung und damit
     * auch den Helden dessen Werte ueber die Schnittstelle manipuliert
     * werden.
     *
     * @param manager fuer die aktulle Erschaffung.
     */
    public final void setManager( final ErschaffungManager manager ) {

        held = manager.getHeld();

        gewaehlte.setManager( manager );

        // leitsonderfertigkeiten.setWerte( held.getLeitsonderfertigkeiten() );
        // leitsonderfertigkeiten.setManager( manager );

        for ( final PanelSonderfertigkeitenListe p : sonderfertigkeitGruppen ) {

            p.setManager( manager );
        }

        final List< SonderfertigkeitWert > verbilligteWerte =
            erzeugeWerte( held.getVerbilligteSonderfertigkeiten() );

        verbilligteWerte.addAll( held.getVerbilligteVarianten() );

        verbilligte.setWerte( verbilligteWerte );

        verbilligte.setManager( manager );
    }
}
