/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui;

import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JLabel;
import javax.swing.JTabbedPane;
import javax.swing.SwingConstants;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif_lite.panel.SimpleInternalFrame;

import de.bernhardjung.dsaprogramm.basis.GpNachteile;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.erschaffung.ErschaffungManager;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

class PanelVorNachteile
extends ResetbaresPanel 
implements Observer, Uebersetzbar {

    private static final String VERBINDUNGEN_REITER = "PanelVorNachteile.Reiter.Verbindungen";

    private static final String NACHTEILE_REITER = "PanelVorNachteile.Reiter.Nachteile";

    private static final String VORTEILE_REITER = "PanelVorNachteile.Reiter.Vorteile";

    private static final String GEWAEHLTE_REITER = "PanelVorNachteile.Reiter.Gewaehlte";

    private static final String VERBINDUNGEN = "PanelVorNachteile.Verbindungen";

    private static final String GEWAEHLTE = "PanelVorNachteile.Gewaehlte";

    /** Zeigt dem Benutzer an wie viele GP aus Nachteilen gewonnen wurden. */
    private static final String GP_NACHTEILE =
        "PanelVorNachteile.GpNachteile";

    /**
     * Erklaert dem Benutzer, dass nicht mehr als 50 Punkte aus Nachteilen
     * gewonnen werden duerfen.
     */
    public static final String GP_NACHTEILE_HINWEIS =
        "PanelVorNachteile.GpNachteileHinweis";

    /**
     * Zeigt dem Benutzer an wie viele GP aus Schlechten Eigenschaften gewonnen
     * wurden.
     */
    private static final String GP_SCHLECHTE_EIGENSCHAFTEN =
        "PanelVorNachteile.GpSchlechteEigenschaften";

    /**
     * Erklaert dem Benutzer, dass nicht mehr als 30 GP aus Schlechten
     * Eigenschaften gewonnen werden duerfen.
     */
    public static final String GP_SCHLECHTE_EIGENSCHAFTEN_HINWEIS =
        "PanelVorNachteile.GpSchlechteEigenschaftenHinweis";

    /** Die Maximale Anzahl an GP die man aus Nachteilen erhalten darf. */
    public static final int MAX_GP_NACHTEILE = 50;

    /**
     * Die Maximale Anzahl an GP die man aus schlechten Eigenschaften erhalten
     * darf.
     */
    public static final int MAX_GP_SCHLECHTE_EIGENSCHAFTEN = 30;

    private static final String NACHTEILE =
        "PanelVorNachteile.Nachteile";

    private static final String VOR_NACHTEILE =
        "PanelVorNachteile.VorNachteile";

    private static final String VORTEILE =
        "PanelVorNachteile.Vorteile";

    /** Die Liste der Vor-/Nachteile die der Held besitzt. */
    private final PanelVorNachteileListe gewaehlte;
    
    /** Zeigt an wie viele GP der Held aus Nachteilen erhalten hat. */
    private final JLabel gpNachteile;

    /**
     * Zeigt an wie viele GP der Held aus Schlechten Eigenschaften erhalten hat.
     */
    private final JLabel gpSchlechteEigenschaften;

    /**
     * Der Held dessen Werte gewaehlt und angezeigt werden.
     */
    private Held held;
    
    /** Die Liste der Nachteile. */
    private final PanelVorNachteileListe nachteile;
    
    
    
    /** Die tabbed Pane, die die einzelnen Listen enthaelt. */
    private final JTabbedPane tabbedPane;

    /** Uebersetzt die Benutzeroberflaeche. */
    public final Uebersetzer uebersetzer;
    
    /**
     * Das Panel auf dem die Punkte fuer Verbindungen verteilt werden konnen,
     * wenn der Held automatisch Punkte aus Rasse, Kultur oder Profession
     * erhalten hat.
     */
    private final PanelVerbindungenListe verbindungen;
    
    /** Die Liste der Vorteile. */
    private final PanelVorNachteileListe vorteile;

    PanelVorNachteile() {

        uebersetzer = new Uebersetzer();
    
        
        
        gewaehlte = new PanelVorNachteileListe( true, VOR_NACHTEILE );
        resetter.einfuegen( gewaehlte );
        
        final SimpleInternalFrame gewaehlteFrame = uebersetzer.erzeugeFrame( 
                GEWAEHLTE,
                gewaehlte );
        
        gpNachteile = new JLabel( "0 GP aus Nachteilen.");

        gpSchlechteEigenschaften =
            new JLabel( "0 GP aus Schlechten Eigenschaften." );
        gpSchlechteEigenschaften.setHorizontalAlignment( SwingConstants.RIGHT );

        nachteile = new PanelVorNachteileListe( false, NACHTEILE );
        resetter.einfuegen( nachteile );
        
        List< VorNachteilWert > werte =
            erzeugeWerte( VorNachteil.getNachteile() );

        nachteile.setWerte( werte );
        
        final SimpleInternalFrame nachteileFrame = uebersetzer.erzeugeFrame( 
                NACHTEILE, 
                nachteile );

        vorteile = new PanelVorNachteileListe( false, VORTEILE );
        resetter.einfuegen( vorteile );
        
        verbindungen = new PanelVerbindungenListe();
        
        // TODO gewaehlte.setPanelVerbindungen( verbindungen );

        final SimpleInternalFrame verbindungenFrame = uebersetzer.erzeugeFrame( 
                VERBINDUNGEN,
                verbindungen );
        
        werte = erzeugeWerte( VorNachteil.getVorteile() );

        vorteile.setWerte( werte );

        final SimpleInternalFrame vorteileFrame = uebersetzer.erzeugeFrame( 
                VORTEILE,
                vorteile );

        tabbedPane = new JTabbedPane( JTabbedPane.LEFT );
        resetter.einfuegen( tabbedPane );
        
        uebersetzer.erzeugeTab( tabbedPane, GEWAEHLTE_REITER, gewaehlteFrame );

        uebersetzer.erzeugeTab( tabbedPane, VORTEILE_REITER, vorteileFrame );

        uebersetzer.erzeugeTab( tabbedPane, NACHTEILE_REITER, nachteileFrame );

        uebersetzer.erzeugeTab( 
                tabbedPane, 
                VERBINDUNGEN_REITER, 
                verbindungenFrame );

        setLayout( new FormLayout( 
                "2dlu, F:P:G, F:P:G, 2dlu", 
                "2dlu, P, 2dlu, F:0px:G" ) );

        final CellConstraints cc = new CellConstraints();

        add( gpNachteile, cc.xy( 2, 2 ) );

        add( gpSchlechteEigenschaften, cc.xy( 3, 2 ) );

        add( tabbedPane, cc.xyw( 1, 4, 4 ) );
    }

    /**
     * Erzeugt zu einer List an Vor-/Nachteilen die entsprechenden Werte.
     *
     * @param vorNachteile
     *            zu denen die Werte erzeugt werden sollen.
     * @return eine Liste von VorNachteilWerten.
     */
    private final List< VorNachteilWert > erzeugeWerte(
            final List< VorNachteil > vorNachteile ) {

        final List< VorNachteilWert > werte =
            new ArrayList< VorNachteilWert >( vorNachteile.size() );

        for ( final VorNachteil vorNachteil : vorNachteile ) {

            if ( vorNachteil.hatVarianten() ) {

                final List< Variante > varianten = 
                    vorNachteil.getVarianten( null );

                final Variante variante;
                
                if ( 0 < varianten.size() ) {

                    variante = varianten.get( 0 );

                } else {

                    variante = new Variante();

                }

                werte.add( new VorNachteilWert( vorNachteil, variante ) );

            } else {

                werte.add( new VorNachteilWert( vorNachteil ) );

            }
        }

        return werte;
    }
    
    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
        
        final GpNachteile gp = held.getGpNachteile();

        gpNachteile.setText( String.format( 
                I18n.i18n( GP_NACHTEILE ), 
                gp.nachteile ) );

        if ( MAX_GP_NACHTEILE < gp.nachteile ) {
            gpNachteile.setToolTipText( 
                    "<html>" + I18n.i18n( GP_NACHTEILE_HINWEIS ) );
        }

        gpSchlechteEigenschaften.setText( String.format(
                I18n.i18n( GP_SCHLECHTE_EIGENSCHAFTEN ), 
                gp.schlechteEigenschaften ) );

        if ( MAX_GP_SCHLECHTE_EIGENSCHAFTEN < gp.schlechteEigenschaften ) {

            gpSchlechteEigenschaften.setToolTipText( 
                    "<html>"+ I18n.i18n( GP_SCHLECHTE_EIGENSCHAFTEN_HINWEIS ) );
        }
    }
    
    /**
     * @param manager fuer die aktuelle Erschaffung.
     */
    public final void setManager( final ErschaffungManager manager ) {
        
        held = manager.getHeld();

        held.addObserver( this );

        gewaehlte.setHeld( held );

        nachteile.setHeld( held );

        if ( 0 != manager.getVerbindungen() ) {

            verbindungen.setManager( manager );

            tabbedPane.setEnabledAt( 3, true );

        } else {

            tabbedPane.setEnabledAt( 3, false );

            tabbedPane.setSelectedIndex( 0 );

        }

        vorteile.setHeld( held );


        gpNachteile.setText( String.format( I18n.i18n( GP_NACHTEILE ), "0" ) );
        gpNachteile.setForeground( getForeground() );
        gpNachteile.setToolTipText( null );

        gpSchlechteEigenschaften.setText( String.format(
                I18n.i18n( GP_SCHLECHTE_EIGENSCHAFTEN ), 
                "0" ) );
        gpSchlechteEigenschaften.setForeground( getForeground() );
        gpSchlechteEigenschaften.setToolTipText( null );
    }

    public void update(final Observable o, final Object arg) {

        if ( ( arg instanceof VorNachteil )
            && ! ((VorNachteil) arg).istVorteil() ) {

            final GpNachteile gp = held.getGpNachteile();

            gpNachteile.setText( String.format(
                    I18n.i18n( GP_NACHTEILE ), 
                    gp.nachteile ) );

            if ( MAX_GP_NACHTEILE < gp.nachteile ) {

                gpNachteile.setForeground( Util.FEHLERFARBE );
                gpNachteile.setToolTipText( 
                        "<html>" + I18n.i18n( GP_NACHTEILE_HINWEIS ) );

            } else {

                gpNachteile.setForeground( getForeground() );
                gpNachteile.setToolTipText( null );
            }

            gpSchlechteEigenschaften.setText( String.format(
                    I18n.i18n( GP_SCHLECHTE_EIGENSCHAFTEN ), 
                    gp.schlechteEigenschaften ) );

            if ( MAX_GP_SCHLECHTE_EIGENSCHAFTEN < gp.schlechteEigenschaften ) {

                gpSchlechteEigenschaften.setForeground( Util.FEHLERFARBE );
                gpSchlechteEigenschaften.setToolTipText( 
                        "<html>" + 
                        I18n.i18n( GP_SCHLECHTE_EIGENSCHAFTEN_HINWEIS ) );

            } else {

                gpSchlechteEigenschaften.setForeground( getForeground() );
                gpSchlechteEigenschaften.setToolTipText( null );
            }
        }
    }
}
