/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui;

import java.awt.Color;
import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JTabbedPane;
import javax.swing.SwingConstants;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif_lite.panel.SimpleInternalFrame;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;
import de.bernhardjung.dsaprogramm.zauber.Zauber;
import de.bernhardjung.dsaprogramm.zauber.ZauberWert;
import de.bernhardjung.dsaprogramm.zauber.Zaubergruppe;

public class PanelZauber 
extends ResetbaresPanel 
implements Observer, Uebersetzbar {

    private static final String GEWAEHLTE_REITER = "PanelZauber.Reiter.Gewaehlte";

    private static final String GEWAEHLTE = "PanelZauber.Gewaehlte";

    /**
     * Ein Text der dem Benutzer mitteilt, dass er genau einen Zauber aktiviert
     * hat.
     */
    private static final String EIN_ZAUBER_AKTIVIERT =
        "PanelZauber.EinZauberAktiviert";

    /** Der Text neben der Combobox mit der man die Lernmethode waehlen kann. */
    public static final String LERNMETHODE =
        "PanelZauber.Lernmethode";

    /** Die maximale Anzahl an Zaubern die aktiviert werden darf. */
    public static final int MAX_AKTIVIEREN = 7;

    /**
     * Erklaert dem Benutzer, dass nicht mehr als 7 Zauber aktivert werden
     * duerfen.
     */
    public static final String MAXIMAL_AKTIVIERBAR =
        "PanelZauber.MaximalAktivierbar";

    /**
     * Ein Text der dem Benutzer mitteilt, dass mehrere Zauber aktiviert wurden.
     */
    private static final String MEHRERE_ZAUBER_AKTIVIERT =
        "PanelZauber.MehrereZauberAktiviert";

    /**
     * Das Label mit dem angezeigt wird wie viele Zauber der Benutzer schon
     * aktiviert hat.
     */
    private final JLabel aktivierteZauber;
    
    /** Die Liste der Zauber, die der Held besitzt. */
    private final PanelZauberListe gewaehlte;
    
    /**
     * Der Held dessen Zauber bearbeitet werden.
     */
    private Held held;

    /**
     * Die Combobox mit der man die Lernmethode waehlt.
     */
    private final JComboBox lernmethode;
    
    /**
     * Gibt die in der Combobox {@link #lernmethode} gewaehlte Lernmethode an
     * die Subpanel auf denen man die Zauber waehlt weiter.
     */
    private final Lernmethoden lernmethoden;

    
    
    /** Uebersetzt die Benutzeroberflaeche. */
    private final Uebersetzer uebersetzer;
    
    /**
     * Die Panel mit den Zaubern der einzelnen Zaubergruppen.
     */
    private final List< PanelZauberListe > zauber;

    /**
     * Erzeugt ein neues Panel.
     *
     * @param zumSteigern
     *            {@code true}, wenn das Panel beim Steigern des Helden
     *            verwendet wird, {@code false}, wenn das Panel bei der
     *            Heldenerschaffung verwendet wird.
     */
    public PanelZauber( final boolean zumSteigern ) {

        uebersetzer = new Uebersetzer();

        
        
        aktivierteZauber = new JLabel( I18n.i18n( EIN_ZAUBER_AKTIVIERT ) );

        aktivierteZauber.setHorizontalAlignment( SwingConstants.CENTER );
        
        lernmethode = uebersetzer.erzeugeComboBox( Lernmethode.values() );
        resetter.einfuegen( lernmethode );
        
        lernmethoden = new Lernmethoden( lernmethode );
        
        gewaehlte = new PanelZauberListe( true, zumSteigern, lernmethoden );
        resetter.einfuegen( gewaehlte );
        
        zauber = new ArrayList< PanelZauberListe >();

        aufbauenBenutzeroberflaeche( zumSteigern );
    }

    /** Baut die Benutzeroberflaeche auf. */
    private final void aufbauenBenutzeroberflaeche( final boolean zumSteigern ) {

        final SimpleInternalFrame gewaehlteFrame = uebersetzer.erzeugeFrame(
                GEWAEHLTE,
                gewaehlte );
        
        final JLabel lernmethodeLabel = uebersetzer.erzeugeLabel( LERNMETHODE );
        
        final JTabbedPane tabbedPane = new JTabbedPane( JTabbedPane.LEFT );
        resetter.einfuegen( tabbedPane );
        
        uebersetzer.erzeugeTab( tabbedPane, GEWAEHLTE_REITER, gewaehlteFrame );

        for ( final Zaubergruppe gruppe : Zaubergruppe.getZaubergruppen() ) {

            final PanelZauberListe panelZauber = new PanelZauberListe( 
                    false, 
                    zumSteigern, 
                    lernmethoden );
            resetter.einfuegen( panelZauber );
            
            panelZauber.setWerte( erzeugeWerte( gruppe ) );

            zauber.add( panelZauber );

            final SimpleInternalFrame frame = uebersetzer.erzeugeFrame(
                    gruppe.getLokalisierungsschluesselName(),
                    panelZauber );
            
            uebersetzer.erzeugeTab(
                    tabbedPane,
                    gruppe.getLokalisierungsschluesselKuerzel(), 
                    frame );
        }

        if ( zumSteigern ) {

            setLayout( new FormLayout(
                    "0px:G, P, 2dlu, P, 0px:G",
                    "2dlu, P, 2dlu, F:0px:G" ) );

            final CellConstraints cc = new CellConstraints();

            add( lernmethodeLabel, cc.xy( 2, 2 ) );
            add( lernmethode, cc.xy( 4, 2 ) );

            add( tabbedPane, cc.xyw( 1, 4, 5 ) );

        } else {

            setLayout( new FormLayout( "F:0px:G", "2dlu, P, 2dlu, F:0px:G" ) );

            final CellConstraints cc = new CellConstraints();

            add( aktivierteZauber, cc.xy( 1, 2 ) );

            add( tabbedPane, cc.xy( 1, 4 ) );
        }
    }

    private final List< ZauberWert > erzeugeWerte( final List< Zauber > zauber){

        final List< ZauberWert > werte =
            new ArrayList< ZauberWert >( zauber.size() );

        for ( final Zauber z : zauber ) {

            if ( z.hatVarianten() ) {
             
                werte.add( new ZauberWert( z, new Variante(), 0 ) );
                
            } else {
                
                werte.add( new ZauberWert( z ) );
            }
        }

        return werte;
    }

    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
    }
    
    /**
     * Bestimmt den Helden dessen Werte angezeigt werden.
     *
     * @param held dessen Werte angezeigt werden sollen.
     */
    public final void setHeld( final Held held ) {

        this.held = held;
        
        held.addObserver( this );

        gewaehlte.setHeld( held );

        for ( final PanelZauberListe panel : zauber ) {

            panel.setHeld( held );

        }

        update( held, null );
    }

    /**
     * Wir aufgerufen wenn am ueberwachten Helden eine Veraenderung
     * durchgefuehrt wird.
     *
     * @param o der Held der ueberwacht wird.
     * @param arg ungenutzt.
     */
    public final void update( final Observable o, final Object arg ) {

        final int anzahlAktivierteZauber = held.getAnzahlAktivierteZauber();

        final String text;

        final String tooltip;
        
        final Color farbe;
        
        if ( 1 == anzahlAktivierteZauber ) {

            text = I18n.i18n( EIN_ZAUBER_AKTIVIERT );

        } else {

            text = String.format( 
                    I18n.i18n( MEHRERE_ZAUBER_AKTIVIERT ),
                    anzahlAktivierteZauber );

        }

        if ( anzahlAktivierteZauber > MAX_AKTIVIEREN ) {

            farbe = Util.FEHLERFARBE;
            
            tooltip = String.format(
                    "<html>" + I18n.i18n( MAXIMAL_AKTIVIERBAR ),
                    MAX_AKTIVIEREN );

        } else {

            farbe = getForeground();
            
            tooltip = null;
        }

        aktivierteZauber.setText( text );
        aktivierteZauber.setToolTipText( tooltip );
        aktivierteZauber.setForeground( farbe );
    }
}
