/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.action;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;

import de.bernhardjung.dsaprogramm.bausteine.Rasse;
import de.bernhardjung.dsaprogramm.daten.Datenkonstanten;
import de.bernhardjung.dsaprogramm.daten.RassenLader;
import de.bernhardjung.dsaprogramm.gui.Hauptpanel;

public class ActionLadenRasse extends ActionLadenHeldenbaustein {

    private RassenLader rassenLader;

    public ActionLadenRasse( final Hauptpanel dasHauptpanel ) {

        super( dasHauptpanel,
                "ActionLadenRasse.Name",
                "ActionLadenRasse.Mnemonic",
                Datenkonstanten.DATEIERWEITERUNG_RASSE,
                Datenkonstanten.VERZEICHNIS_RASSEN );

        try {

            rassenLader = new RassenLader();
            
        } catch ( final Exception e ) {
            // ignorieren
        }
    }

    @Override
    public void laden( final File datei ) throws Exception {

        try {

            final InputStream in = new FileInputStream( datei );

            final Rasse rasse =
                rassenLader.ladeRasse( datei.getName(), in );

            hauptpanel.getPanelMetaRasse().anzeigen( rasse );

        } catch ( final FileNotFoundException ex ) {

            // ignorieren. Das die Datei existiert haben wir ja gerade
            // getestet.

        }
    }

}
