/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.bonus;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.bausteine.bonus.TalentbonusFrei;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.TalentWert;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

public class PanelTalentBonusFrei
extends PanelTalentBonus 
implements ChangeListener, Uebersetzbar {

    /**
     * Die Label die den aktuellen Talentwert des Helden in einem Talent 
     * anzeigen.
     */
    private final List< JLabel > aktuellLabel;

    /**
     * Die Ueberschrift fuer die Spalte in der der aktuelle Talentwert des
     * Helden angezeigt wird.
     */
    private final JLabel aktuellUeberschrift;
    
    /**
     * Der Held fuer den der Bonus gewaehlt wird.
     */
    private Held held;
    
    /**
     * Die Liste mit den Spinnern die zum waehlen der Talentstufe verwendet
     * werden.
     */
    private final List< JSpinner > spinner;

    /** Die Punkte die man auf die Talente verteilen darf. */
    private int stufenBonus;

    /**
     * Die Talente auf die es den Stufen-Bonus zu verteilen gilt.
     */
    private final List< Talent > talente;

    /**
     * Die Label die die Namen der Talente anzeigen.
     */
    private final List< JLabel > talenteLabel;
    
    /** Uebersetzt die Ueberschrift. */
    private final Uebersetzer uebersetzer;
    /**
     * Die Erklaerung die dem Benutzer mitteilt wie viele Talentpunkte er
     * verteilen darf.
     */
    private final JLabel verteilen;

    /** Erzeugt ein neues, leeres Panel. */
    public PanelTalentBonusFrei() {
        
        uebersetzer = new Uebersetzer();
        
        aktuellLabel = new ArrayList< JLabel >();

        aktuellUeberschrift = uebersetzer.erzeugeUeberschrift( 
                "PanelTalentBonusFrei.aktuell" );
        
        spinner = new ArrayList< JSpinner >();
        
        talente = new ArrayList< Talent >();
        
        talenteLabel = new ArrayList< JLabel >();
        
        verteilen = new JLabel( I18n.i18n( "PanelTalentBonusFrei.verteilen" ) );
        
        setLayout( new BorderLayout() );
    }

    /**
     * Baut die Benutzeroberflaeche auf.
     * Ordnet die Label und Spinner an.
     */
    private final void aufbauenBenutzeroberflaeche() {

        removeAll();

        // Pruefen ob genuegend Spinner da sind.
        initialisiereSpinner();

        final JPanel panel = new JPanel();

        // Ein auf die Anzahl der Talent angepasstes Layout setzen.
        panel.setLayout( erzeugeLayout() );

        final CellConstraints cc = new CellConstraints();

        panel.add( verteilen, cc.xyw( 2, 2, 6 ) );

        // Die Ueberschrift fuer die Spalte mit den aktuellen Talent-Stufen.
        panel.add( aktuellUeberschrift, cc.xy( 4, 4 ) );

        // Die Benutzeroberflaeche Zeilenweise aufbauen.
        for ( int i = 0, n = talente.size() ; i < n ; ++i ) {

            final int y = ( 2 * i ) + 6;

            final Talent talent = talente.get( i );

            final JLabel talentLabel = new JLabel( talent.getName() );
            talenteLabel.add( talentLabel );
            
            // Die Spalte mit den Namen der Talente.
            panel.add( talentLabel, cc.xy( 2, y ) );

            // Die Spalte mit den aktuellen Talentwerten.
            final JLabel aktuell = new JLabel();
            aktuellLabel.add( aktuell );
            
            if ( held.hat( talent ) ) {

                aktuell.setText( String.valueOf( held.getStufe( talent ) ) );

            } else {

                aktuell.setText( "-" );

            }
            panel.add( aktuell, cc.xy( 4, y ) );

            // Der Spinner zum einsellen des Wertes.
            panel.add( spinner.get( i ), cc.xy( 6, y ) );
        }

        final JScrollPane scrollPane = Util.erzeugeScrollPane( panel );
        
        add( scrollPane, BorderLayout.CENTER );
    }

    /** Erzeugt das Layout fuer die Anordnung der Label und Spinner. */
    private final FormLayout erzeugeLayout() {

        final StringBuilder zeilenLayout = new StringBuilder();

        zeilenLayout.append( "7dlu, P, 7dlu, P," );
        for ( int i = talente.size(); i > 0 ; --i ) {
            zeilenLayout.append( " 4dlu, P," );
        }
        zeilenLayout.append( " 7dlu" );

        return new FormLayout(
                "7dlu, R:P:G, 4dlu, C:P, 4dlu, P, P:G, 7dlu",
                zeilenLayout.toString() );
    }

    /**
     * Sorgt dafuer das genuegend Spinner vorhanden sind und, dass alle Spinner
     * auf 0 stehen.
     */
    private final void initialisiereSpinner() {

        final int anzahl = talente.size();

        if ( spinner.size() < anzahl ) {
            // Es sind zu wenig Spinner da.
            for ( int i = spinner.size() ; i < anzahl ; ++i ) {

                final JSpinner s = Util.erzeugeSpinner( 0, 0, 99, 1);

                s.addChangeListener( this );

                spinner.add( s );
            }
        }

        // alle Spinner auf 0 setzen.
        for ( final JSpinner s : spinner ) {

            s.setValue( 0 );

            setMaximum( s, stufenBonus );
        }
    }

    /** {@inheritDoc} */
    @Override
    public final List< TalentWert > getTalentBoni() {

        final List< TalentWert > werte = new ArrayList< TalentWert >( 2 );

        for ( int i = 0, n = talente.size() ; i < n ; ++i ) {

            final int stufe = (Integer) spinner.get( i ).getValue();

            if ( 0 < stufe ) {

                final Talent talent = talente.get( i );

                werte.add( new TalentWert( talent, stufe ) );

            }
        }

        return werte;
    }
    
    public void neuUebersetzen() {

        uebersetzer.neuUebersetzen();
        
        verteilen.setText( String.format( 
                I18n.i18n( "PanelTalentBonusFrei.verteilen" ),
                stufenBonus ) );
        
        final Map< Talent, Integer > boni = new HashMap< Talent, Integer >();
        
        for ( final TalentWert bonus : getTalentBoni() ) {
            
            boni.put( bonus.getTalent(), bonus.getStufe( held ) );
        }
        
        Collections.sort( talente, Talent.NAME_COMPARATOR );
        
        for ( int i = 0, n = talente.size(); i < n ; ++i ) {
            
            final Talent talent = talente.get( i );
            
            talenteLabel.get( i ).setText( talent.getName() );
            
            if ( held.hat( talent ) ) {
                
                final int stufe = held.getStufe( talent );
                
                aktuellLabel.get( i ).setText( String.valueOf( stufe  ) );
            
            } else { 
             
                aktuellLabel.get( i ).setText( "-" );
            }
            
            if ( boni.containsKey( talent ) ) {
                
                final Integer verteilt = boni.get( talent );
                
                spinner.get( i ).setValue( verteilt );
            
            } else {
                
                spinner.get( i ).setValue( 0 );
            }
        }
    }

    /**
     * Bestimmt Bonus und Held fuer die man eine Auswahl treffen soll.
     *
     * @param derHeld der den Bonus erhalten soll.
     * @param bonus den es zu verteilen gilt.
     */
    public final void setBonus(
            final Held derHeld,
            final TalentbonusFrei bonus ) {

        held = derHeld;
        
        stufenBonus = bonus.getStufenBonus();

        talente.clear();
        talente.addAll( bonus.getTalente() );
        
        verteilen.setText( String.format( 
                I18n.i18n( "PanelTalentBonusFrei.verteilen" ),
                stufenBonus ) );
        
        aufbauenBenutzeroberflaeche();

        validate();
        repaint();
    }

    /**
     * Bestimmt den Maximal einstellbaren Wert in einem Spinner.
     * @param s dessen Maximum gesetzt werden soll.
     * @param maximum fuer den Spinner.
     */
    private final void setMaximum( final JSpinner s, final int maximum ) {

        final SpinnerNumberModel model;
        model = (SpinnerNumberModel) s.getModel();

        model.setMaximum( maximum );
    }

    /**
     * Aktualisiert den Maximal waehlbaren Wert in den Spinnern, wenn der
     * Wert in einem Spinner geaendert wird.
     */
    public final void stateChanged( final ChangeEvent e ) {

        // Wie viele Punkte sind schon verteilt worden?
        int verteilt = 0;
        for ( final JSpinner s : spinner ) {

            verteilt += (Integer) s.getValue();
        }

        // Wie viele Punkte duerfen noch verteilt werden?
        final int rest = stufenBonus - verteilt;

        // Maximum der Spinner aktualisieren.
        for ( final JSpinner s : spinner ) {

            final int wert = (Integer) s.getValue();

            setMaximum( s, wert + rest );
        }
    }
}
