/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import com.jgoodies.forms.factories.ButtonBarFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Einstellungen;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;

public class DialogDatenquelle extends JDialog implements Uebersetzbar {
    
    private final Einstellungen einstellungen;

    private final JFileChooser fileChooser;
    
    /**
     * Zeigt das Verzeichnis an, das als Datenquelle gewaehlt ist.
     */
    private final JLabel quelle;

    /** Uebersetzt die Texte der Benutzeroberflaeche. */
    private final Uebersetzer uebersetzer;
    
    public DialogDatenquelle( final JFrame hauptfenster ) {

        super( hauptfenster, I18n.i18n( "DialogDatenquelle.Titel") );

        uebersetzer = new Uebersetzer();
        
        einstellungen = Einstellungen.getInstanz();

        fileChooser = new JFileChooser();
        fileChooser.setFileSelectionMode( JFileChooser.DIRECTORIES_ONLY );
        
        quelle = new JLabel( I18n.i18n( "DialogDatenquelle.Basisregeln" ) );
        
        anzeigenDatenquelle( einstellungen.getDatenquelle() );

        aufbauenBenutzeroberflaeche();

        pack();
    }

    private void abbrechen() {

        anzeigenDatenquelle( einstellungen.getDatenquelle() );

        setVisible( false );
    }

    private void aufbauenBenutzeroberflaeche() {

        final JButton basisregeln = uebersetzer.erzeugeButton( 
                "DialogDatenquelle.Basisregeln" );
        basisregeln.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                basisregelnGewaehlt();
            }
        });

        final JButton verzeichnisWaehlen = uebersetzer.erzeugeButton( 
                "DialogDatenquelle.VerzeichnisWaehlen" );
        verzeichnisWaehlen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                verzeichnisWaehlen();
            }
        });

        final JButton ok = uebersetzer.erzeugeButton( "DialogDatenquelle.Ok" );
        ok.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                ok();
            }
        });

        final JButton abbrechen = uebersetzer.erzeugeButton( 
                "DialogDatenquelle.Abbrechen" );
        abbrechen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                abbrechen();
            }
        });

        final JLabel quelleLabel = uebersetzer.erzeugeLabel( 
                "DialogDatenquelle.Quelle" );

        final JPanel panelQuelle =
            new JPanel( new FormLayout( "P, 4dlu, F:200px:G", "T:P" ) );

        final CellConstraints cc = new CellConstraints();

        panelQuelle.add( quelleLabel, cc.xy( 1, 1 ) );

        final JScrollPane scrollPane = new JScrollPane( quelle );
        scrollPane.setHorizontalScrollBarPolicy( 
                JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS );
        
        panelQuelle.add( scrollPane, cc.xy( 3, 1 ) );

        final JPanel panelWaehlen =
            new JPanel( new FormLayout( "F:P:G, 4dlu, F:P:G", "P" ) );

        panelWaehlen.add( basisregeln, cc.xy( 1, 1 ) );

        panelWaehlen.add( verzeichnisWaehlen, cc.xy( 3, 1 ) );

        final JPanel panelSchliessen =
            ButtonBarFactory.buildOKCancelBar( ok, abbrechen );

        final JPanel hauptpanel = new JPanel(
                new FormLayout( "7dlu, F:P:G, 7dlu", "7dlu, P, 4dlu, P, 4dlu, P, 7dlu" ) );

        hauptpanel.add( panelQuelle, cc.xy( 2, 2 ) );

        hauptpanel.add( panelWaehlen, cc.xy( 2, 4 ) );

        hauptpanel.add( panelSchliessen, cc.xy( 2, 6 ) );

        add( hauptpanel );
    }

    private void basisregelnGewaehlt() {
        quelle.setText( I18n.i18n( "DialogDatenquelle.Basisregeln" ) );
    }

    public void neuUebersetzen() {

        uebersetzer.neuUebersetzen();
        
        setTitle( I18n.i18n( "DialogDatenquelle.Titel") );

        if ( quelle.getText().contains( "/" ) 
                && ! quelle.getText().contains( "\\" ) )
        {
            basisregelnGewaehlt();
        }
        
    }
    private void ok() {

        String datenquelle = quelle.getText();

        if ( datenquelle.equals( I18n.i18n( "DialogDatenquelle.Basisregeln" ) ) ) {

            datenquelle = Einstellungen.DATENQUELLE_INTERN;
        }

        final String bisherigeQuelle = einstellungen.getDatenquelle();

        if ( ! bisherigeQuelle.equals( datenquelle ) ) {

            einstellungen.setDatenquelle( datenquelle );

            final Datenspeicher speicher = Datenspeicher.getInstanz();

            if ( einstellungen.istDatenquelleVerzeichnis() ) {

                final File verzeichnis = new File( datenquelle );

                speicher.speichereDaten( verzeichnis );
            }

            try {

                speicher.ladeDaten();

            } catch( final Exception ex ) {

                JOptionPane.showMessageDialog(
                        null,
                        I18n.i18n( "DialogDatenquelle.DatenNichtGeladen.Meldung" ),
                        I18n.i18n( "DialogDatenquelle.DatenNichtGeladen.Titel" ),
                        JOptionPane.ERROR_MESSAGE );
            }
        }

        setVisible( false );
    }

    private void anzeigenDatenquelle( String datenquelle ) {

        if ( Einstellungen.DATENQUELLE_INTERN.equals( datenquelle ) ) {

            datenquelle = I18n.i18n( "DialogDatenquelle.Basisregeln" );
        }

        quelle.setText( datenquelle );
    }

    private void verzeichnisWaehlen() {

        final int ergebnis = fileChooser.showOpenDialog( this );

        if ( JFileChooser.APPROVE_OPTION == ergebnis ) {

            final File verzeichnis = fileChooser.getSelectedFile();

            anzeigenDatenquelle( verzeichnis.getAbsolutePath() );
        }
    }
}
