/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.ElementMitName;
import de.bernhardjung.dsaprogramm.basis.Fernkampfwaffe;
import de.bernhardjung.dsaprogramm.basis.Tp;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.daten.Datenspeicherer;
import de.bernhardjung.dsaprogramm.gui.CharGenListModel;
import de.bernhardjung.dsaprogramm.gui.WechselbaresPanel;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Kampftechnik;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.util.Einstellungen;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

public class PanelMetaFernkampfwaffen 
extends WechselbaresPanel 
implements Uebersetzbar {
    
    /** Die Liste mit den Fernwaffen. */
    private JList fernwaffen;
    
    /** Das Model fuer die Liste mit den Fernwaffen. */
    private final 
    CharGenListModel< ElementMitName, Fernkampfwaffe > fernwaffenListModel;
    
    /**
     * Gibt an ob der Benutzer schon darauf hingewiesen wurde eine Datenquelle
     * zu waehlen.
     */
    private boolean hinweisGezeigt;

    /** Die Id. */
    private final JTextField id;

    /** Der Name. */
    private final JTextField name;
    
    /** Die Spinner zu Einstellen der Reichweiten. */
    private final List< JSpinner > reichweiten;
    
    /** Da sind die Ruestungen gespeichert. */
    private final Datenspeicher speicher;

    /** Das Talent mit dem die Waffe gefuehrt wird. */
    private final JComboBox talent;

    /** Das Textfeld zum eingeben der Trefferpunkte. */
    private final JTextField tp;
    
    /** Die Spinner zum Einstellen der Trefferpunkte aus Reichweite. */
    private final List< JSpinner > tpPlus;
    
    /**
     * Der Knopf mit dem die eingegebene Ruestung in die Liste der Ruestungen
     * uebernommen wird.
     */
    private final JButton uebernehmen;

    /** Uebersetzt die Texte der Benutzeroberflaeche. */
    private final Uebersetzer uebersetzer;
    
    public PanelMetaFernkampfwaffen() {

        uebersetzer = new Uebersetzer();
    
        speicher = Datenspeicher.getInstanz();

        fernwaffenListModel = 
            new CharGenListModel< ElementMitName, Fernkampfwaffe >(
                    speicher.getFernkampfwaffen(),
                    Fernkampfwaffe.NAME_COMPARATOR );

        fernwaffen = uebersetzer.erzeugeListe( 
                fernwaffenListModel );
        fernwaffen.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        fernwaffen.addListSelectionListener( new ListSelectionListener(){
            public void valueChanged( final ListSelectionEvent e ) {

                if ( e.getValueIsAdjusting() ) {
                    return;
                }

                anzeigen();
            }
        });
        
        hinweisGezeigt = false;

        id = new JTextField();
        id.setBackground( Util.FEHLERFARBE );
        id.getDocument().addDocumentListener( new DocumentListener() {
            public void changedUpdate( final DocumentEvent e ) {}

            public void insertUpdate( final DocumentEvent e ) {
                pruefen();
            }
            public void removeUpdate( final DocumentEvent e ) {
                pruefen();
            }
        });
        
        name = new JTextField();
        
        reichweiten = new ArrayList< JSpinner >( 5 );
        for ( int i = 0; i < 5 ; ++i ) {
            reichweiten.add( Util.erzeugeSpinner( 0, 0, 999, 5 ) );
        }
        
        final Vector< Talent > talente = new Vector< Talent >();
        for ( final Talent talent : Talentgruppe.KAMPF ) {

            if ( ((Kampftechnik) talent).istFernkampfTechnik() ) {

                talente.add( talent );
            }
        }
        talent = uebersetzer.erzeugeComboBox( talente, Talent.NAME_COMPARATOR );
        
        tp = new JTextField();
        tp.setBackground( Util.FEHLERFARBE );
        tp.getDocument().addDocumentListener( new DocumentListener() {
            public void changedUpdate( final DocumentEvent e ) {}

            public void insertUpdate( final DocumentEvent e ) {
                pruefen();
            }
            public void removeUpdate( final DocumentEvent e ) {
                pruefen();
            }
        });
        
        tpPlus = new ArrayList< JSpinner >( 5 );
        for ( int i = 0; i < 5 ; ++i ) {
            tpPlus.add( Util.erzeugeSpinner( 0, -9, 9, 1 ) );
        }

        uebernehmen = uebersetzer.erzeugeButton( 
                "PanelMetaFernkampfwaffen.Uebernehmen" );
        uebernehmen.setEnabled( false );
        uebernehmen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                uebernehmen();
            }
        });

        aufbauenBenutzeroberflaeche();
    }

    /**
     * Zeigt die Werte der in der Liste gewaehlte Ruestung an.
     */
    private final void anzeigen() {

        final Fernkampfwaffe waffe = getGewaehlteFernkampfwaffe();

        if ( null != waffe ) {            

            id.setText( waffe.getId().replace( "Waffe.", "" ) );

            name.setText( waffe.getName() );

            final List< Integer > reichweitenWerte = waffe.getReichweiten();
            final List< Integer > tpPlusWerte = waffe.getTpPlus();

            for ( int i = 0 ; i < 5 ; ++i ) {

                reichweiten.get( i ).setValue( reichweitenWerte.get( i ) );

                tpPlus.get( i ).setValue( tpPlusWerte.get( i ) );
            }

            talent.setSelectedItem( waffe.getTalent() );

            tp.setText( waffe.getTp().toString() );
        }
    }

    private final void aufbauenBenutzeroberflaeche() {

        final JButton entfernen = uebersetzer.erzeugeButton( 
                "PanelMetaFernkampfwaffen.Entfernen" );
        entfernen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                entfernen();
            }
        });

        final JScrollPane fernwaffenScrollPane = 
            Util.erzeugeScrollPane( fernwaffen );
        resetter.einfuegen( fernwaffenScrollPane );
        
        final String spalten = "7dlu, 200px:G, 4dlu, P:G, 7dlu";

        final String zeilen  = "7dlu, F:P:G, 4dlu, P, 7dlu";

        final FormLayout layout = new FormLayout( spalten, zeilen );

        setLayout( layout );

        final CellConstraints cc = new CellConstraints();

        add( fernwaffenScrollPane, cc.xy( 2, 2 ) );

        add( entfernen, cc.xy( 2, 4 ) );

        add( erzeugeDatenPanel(), cc.xy ( 4, 2 ) );
        
        add( uebernehmen, cc.xy( 4, 4 ) );
    }

    /**
     * Entfernt die in {@link #fernwaffen} geweahlte Waffe aus dem
     * Datenspeicher.
     */
    private void entfernen() {

        final Fernkampfwaffe fernwaffe = getGewaehlteFernkampfwaffe();

        if ( null != fernwaffe ) {
            
            speicher.entfernen( fernwaffe );

            speichern();
            
            fernwaffenListModel.removeElement( fernwaffe );
        }
    }

    private final JScrollPane erzeugeDatenPanel() {

        final JLabel idLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaFernkampfwaffen.Id" );

        final JLabel nameLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaFernkampfwaffen.Name" );

        final JLabel reichweitenLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaFernkampfwaffen.Reichweiten" );

        final JLabel talentLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaFernkampfwaffen.Talent" );

        final JLabel tpLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaFernkampfwaffen.Tp" );
        
        final JLabel tpPlusLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaFernkampfwaffen.TpPlus" );

        final FormLayout layout = new FormLayout( "P, 4dlu, P" );

        final DefaultFormBuilder builder = new DefaultFormBuilder( layout );
        
        builder.append( nameLabel, name );

        builder.append( idLabel, id );

        builder.append( tpLabel, tp );

        builder.append( talentLabel, talent );

        builder.append( reichweitenLabel, erzeugePanel( reichweiten ) );

        builder.append( tpPlusLabel, erzeugePanel( tpPlus ) );
        
        final JScrollPane scrollPane = 
            Util.erzeugeScrollPane( builder.getPanel() );
        resetter.einfuegen( scrollPane );
        
        return scrollPane;
    }
    
    /**
     * Ordnet die Spinner auf einem Panel in einer horziontalen Linie an.
     */
    private JPanel erzeugePanel( final List< JSpinner > spinner ) {

        final FormLayout layout =
            new FormLayout( "P, 2dlu, P, 2dlu, P, 2dlu, P, 2dlu, P" );

        final DefaultFormBuilder builder = new DefaultFormBuilder( layout );

        for( final JSpinner s : spinner ) {

            builder.append( s );
        }

        return builder.getPanel();
    }

    /** 
     * @return Die Fernkampfwaffe die in der Liste auf der linken Seite 
     *          gewaehlt ist oder {@code null}, wenn keine Waffe gewaehlt ist.   
     */
    private Fernkampfwaffe getGewaehlteFernkampfwaffe() {
        return (Fernkampfwaffe) fernwaffen.getSelectedValue();
    }

    /**
     * Liefert die in den Spinnern eingestellten Zahlen.
     *
     * @param spinner
     *            deren Werte gesucht sind.
     * @return die Werte aus den Spinnern.
     */
    private List< Integer > getWerte( final List< JSpinner > spinner ) {

        final List< Integer > werte = new ArrayList< Integer >( spinner.size());

        for ( final JSpinner s : spinner ) {

            werte.add( (Integer) s.getValue() );
        }

        return werte;
    }
    
    public boolean kannGewechseltWerden() {
        
        return true;
    }
    
    public void neuUebersetzen() {

        uebersetzer.neuUebersetzen();
    }

    /**
     * Prueft die Eingaben im Panel auf Korrektheit.
     *
     * @return {@code true}, wenn alles Ok ist, sonst {@code false}.
     */
    private boolean pruefen() {

        boolean ergebnis = true;

        if ( id.getText().trim().equals( "" ) ) {

            id.setBackground( Util.FEHLERFARBE );

            ergebnis = false;

        } else {

            id.setBackground( name.getBackground() );
        }

        if ( Tp.istTpFormel( tp.getText().trim() ) ) {

            tp.setBackground( name.getBackground() );

        } else {

            tp.setBackground( Util.FEHLERFARBE );

            ergebnis = false;
        }

        uebernehmen.setEnabled( ergebnis );

        return ergebnis;
    }
    
    /**
     * Speichert die Waffen auf die Festplatte.
     */
    private final void speichern() {

        final Einstellungen einstellungen = Einstellungen.getInstanz();

        if ( einstellungen.istDatenquelleVerzeichnis() ) {

            Datenspeicherer.speichereFernkampfwaffen();

        } else if ( ! hinweisGezeigt ){

            JOptionPane.showMessageDialog( 
                    this, 
                    I18n.i18n( "PanelMetaFernkampfwaffen.HinweisDatenquelle" ) );

            hinweisGezeigt = true;
        }

    }

    /**
     * Uebernimmt die im Panel eingegebenen Fernwaffe in die Liste der
     * Fernwaffen.
     */
    private final void uebernehmen() {

        if ( ! pruefen() ) {
            return;
        }

        String idString = id.getText().trim();

        idString = "Waffe." + idString;

        final Fernkampfwaffe waffe = speicher.getFernkampfwaffe( idString );

        waffe.setName( name.getText().trim() );

        waffe.setReichweiten( getWerte( reichweiten ) );

        waffe.setTalent( (Kampftechnik) talent.getSelectedItem() );

        waffe.setTp( new Tp( tp.getText().trim() ) );

        waffe.setTpPlus( getWerte( tpPlus ) );

        speicher.einfuegen( waffe );

        speichern();

        fernwaffenListModel.setElements( speicher.getFernkampfwaffen() );
    }
}
