/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import javax.swing.ButtonGroup;
import javax.swing.JRadioButton;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.ElementMitName;
import de.bernhardjung.dsaprogramm.bausteine.Kultur;
import de.bernhardjung.dsaprogramm.bausteine.Profession;
import de.bernhardjung.dsaprogramm.bausteine.moeglicheprof.MoeglicheProfessionen;
import de.bernhardjung.dsaprogramm.bausteine.moeglicheprof.MoeglicheProfessionenAlleAusser;
import de.bernhardjung.dsaprogramm.bausteine.moeglicheprof.MoeglicheProfessionenAlleVon;
import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

public class PanelMetaMoeglicheProfessionen
extends ResetbaresPanel 
implements Uebersetzbar { 

    /**
     * Ist dieser Radiobutton angewaehlt gelten alle in der Liste gewaehlten
     * Professionen als erlaubte Professionen fuer die Kultur.  
     */
    private final JRadioButton alleVon;

    /**
     * Ist dieser Radionbutton angewaehlt gelten alle gewaehlten Professionen
     * als nicht moeglich fuer die Kultur.
     */
    private final JRadioButton alleAusser;

    /**
     * Zeigt die Liste mit den Professionen.
     */
    private final PanelMetaListe< ElementMitName, Profession > professionen;
    
    
    
    /** Uebersetzt die Radiobuttons. */
    private final Uebersetzer uebersetzer;

    PanelMetaMoeglicheProfessionen() {

        
        
        uebersetzer = new Uebersetzer();
        
        final ButtonGroup group = new ButtonGroup();

        alleVon = uebersetzer.erzeugeRadioButton( 
                "PanelMetaMoeglicheProfessionen.AlleVon" );
        alleVon.setSelected( true );
        group.add( alleVon );

        alleAusser = uebersetzer.erzeugeRadioButton( 
                "PanelMetaMoeglicheProfessionen.AlleAusser" );
        group.add( alleAusser );

        professionen = new PanelMetaListeProfessionen();
        resetter.einfuegen( professionen );
        uebersetzer.einfuegen( professionen );
        
        final FormLayout layout =
            new FormLayout( "F:P:G, 4dlu, F:P:G", "P, 4dlu, F:200px:G" );

        final DefaultFormBuilder builder =
            new DefaultFormBuilder( layout, this );

        builder.setDefaultDialogBorder();

        builder.append( alleVon );
        builder.append( alleAusser );

        builder.nextLine( 2 );

        builder.append( Util.erzeugeScrollPane( professionen ), 3 );
    }

    /**
     * Zeigt die moeglichen Professionen einer Kultur im Panel an.
     *
     * @param kultur
     *            deren moegliche Professionen angezeigt werden sollen.
     */
    public final void anzeigen( final Kultur kultur ) {

        final MoeglicheProfessionen moegliche = kultur.getMoeglicheProfessionen();

        alleVon.setSelected( moegliche instanceof MoeglicheProfessionenAlleVon );

        alleAusser.setSelected( moegliche instanceof MoeglicheProfessionenAlleAusser );

        professionen.setGewaehlteElemente( moegliche.getProfessionen() );
    }

    /** Waehlt alle Professionen ab. */
    public void eingabenLoeschen() {
        alleVon.setSelected( true );

        professionen.eingabenLoeschen();
    }

    /**
     * @return die im Panel gewaehlten moeglichen Professionene.
     */
    public final MoeglicheProfessionen getMoeglicheProfessionen() {

        if ( alleVon.isSelected() ) {

            return new MoeglicheProfessionenAlleVon(
                    professionen.getGewaehlteElemente() );

        } else {

            return new MoeglicheProfessionenAlleAusser(
                    professionen.getGewaehlteElemente() );
        }
    }
    
    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
        
    }
}
