/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.jgoodies.forms.factories.ButtonBarFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.ElementMitName;
import de.bernhardjung.dsaprogramm.basis.Nahkampfwaffe;
import de.bernhardjung.dsaprogramm.basis.Tp;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.daten.Datenspeicherer;
import de.bernhardjung.dsaprogramm.gui.CharGenListModel;
import de.bernhardjung.dsaprogramm.gui.WechselbaresPanel;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Kampftechnik;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.util.Einstellungen;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

public class PanelMetaNahkampfwaffen 
extends WechselbaresPanel 
implements Uebersetzbar {

    /** Der Bruchfaktor. */
    private final JSpinner bf;
        
    /** Distanzklasse Handgemenge */
    private final JCheckBox dkH;

    /** Distanzklasse Nahkampf. */
    private final JCheckBox dkN;

    /** Distanzklasse Stangenwaffen */
    private final JCheckBox dkS;
    
    /**
     * Gibt an ob der Benutzer schon darauf hingewiesen wurde eine Datenquelle
     * zu waehlen.
     */
    private boolean hinweisGezeigt;

    /** Die Id. */
    private final JTextField id;

    /** Der Initiativemodifikator. */
    private final JSpinner ini;
    
    /** Der Name. */
    private final JTextField name;
    
    
    
    /** Da sind die Ruestungen gespeichert. */
    private final Datenspeicher speicher;
    
    /** Das Haupttalent mit dem die Waffe gefuehrt wird. */
    private final JComboBox talent;
    
    /** Die Talente mit denen die Waffe gefuehrt wird. */
    private final PanelMetaListe< ElementMitName, Kampftechnik > talente;

    /** Die TP. */
    private final JTextField tp;

    /** Die TP/KK. */
    private final JTextField tpKk;
    
    /**
     * Eine Regexp, um die Eingaben ins Textfeld {@link #tpKk} zu ueberpruefen.
     */
    private final Pattern tpKkPattern;

    /** Der Knopf mit dem die eingegeben Waffe gespeichert wird. */
    private final JButton uebernehmen;

    /** Uebersetzt die Text der Benutzeroberflaeche. */
    private final Uebersetzer uebersetzer;
    
    /** Eine Liste mit Waffen */
    private final JList waffen;     
    
    /** Das Mode fuer die Liste der Waffen . */
    private final CharGenListModel< ElementMitName, Nahkampfwaffe > waffenListModel;

    /** Die Liste der Waffen . */
    private final JTextField wm;

    /**
     * Eine Regexp, um die Eingaben ins Textfeld {@link #wm} zu ueberpruefen.
     */
    private final Pattern wmPattern;

    public PanelMetaNahkampfwaffen() {

        uebersetzer = new Uebersetzer();
        
        
        
        bf = Util.erzeugeSpinner( 0, 0, 12, 1 );
                
        dkH = uebersetzer.erzeugeCheckBox( "PanelMetaNahkampfwaffe.DkH" );

        dkN = uebersetzer.erzeugeCheckBox( "PanelMetaNahkampfwaffe.DkN" );

        dkS = uebersetzer.erzeugeCheckBox( "PanelMetaNahkampfwaffe.DkS" );

        hinweisGezeigt = false;

        id = new JTextField();
        id.setBackground( Util.FEHLERFARBE );
        id.getDocument().addDocumentListener( new DocumentListener() {
            public void changedUpdate( final DocumentEvent e ) {}

            public void insertUpdate( final DocumentEvent e ) {
                pruefen();
            }

            public void removeUpdate( final DocumentEvent e ) {
                pruefen();
            }
        } );
        
        ini = Util.erzeugeSpinner( 0, -9, 9, 1 );

        name = new JTextField();

        speicher = Datenspeicher.getInstanz();

        final List< Kampftechnik > nahkampftalente = 
            new ArrayList< Kampftechnik >();

        for ( final Talent talent : Talentgruppe.KAMPF ) {

            if ( ( (Kampftechnik)talent).istNahkampfTechnik() ) {

                nahkampftalente.add( (Kampftechnik) talent );
            }
        }
        talent = uebersetzer.erzeugeComboBox( 
                nahkampftalente, 
                Talent.NAME_COMPARATOR );
        
        talente = new PanelMetaListe< ElementMitName, Kampftechnik >( 
                nahkampftalente,
                Talent.NAME_COMPARATOR );

        tp = new JTextField();
        tp.setBackground( Util.FEHLERFARBE );
        tp.getDocument().addDocumentListener( new DocumentListener() {
            public void changedUpdate( final DocumentEvent e ) {}

            public void insertUpdate( final DocumentEvent e ) {
                pruefen();
            }

            public void removeUpdate( final DocumentEvent e ) {
                pruefen();
            }
        } );
        
        tpKk = new JTextField();
        tpKk.setBackground( Util.FEHLERFARBE );
        tpKk.getDocument().addDocumentListener( new DocumentListener() {
            public void changedUpdate( final DocumentEvent e ) {}

            public void insertUpdate( final DocumentEvent e ) {
                pruefen();
            }

            public void removeUpdate( final DocumentEvent e ) {
                pruefen();
            }
        } );
        
        tpKkPattern = Pattern.compile( "[1-9]\\d*/[1-9]\\d*" );

        uebernehmen = uebersetzer.erzeugeButton( 
                "PanelMetaNahkampfwaffen.Uebernehmen");
        uebernehmen.setEnabled( false );
        uebernehmen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                uebernehmen();
            }
        });

        waffenListModel = new CharGenListModel< ElementMitName, Nahkampfwaffe >(
                speicher.getNahkampfwaffen(),
                Nahkampfwaffe.NAME_COMPARATOR );

        waffen = uebersetzer.erzeugeListe( waffenListModel );
        waffen.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        waffen.addListSelectionListener( new ListSelectionListener(){
            public void valueChanged( final ListSelectionEvent e ) {

                if ( e.getValueIsAdjusting() ) {
                    return;
                }

                anzeigen();
            }
        });
        
        wm = new JTextField();
        wm.setBackground( Util.FEHLERFARBE );
        wm.getDocument().addDocumentListener( new DocumentListener() {
            public void changedUpdate( final DocumentEvent e ) {}

            public void insertUpdate( final DocumentEvent e ) {
                pruefen();
            }

            public void removeUpdate( final DocumentEvent e ) {
                pruefen();
            }
        } );
        
        wmPattern = Pattern.compile( "[+-]?\\d+/[+-]?\\d+" );

        aufbauenBenutzeroberflaeche();
    }

    /**
     * Zeigt die Werte der in der Liste gewaehlte Ruestung an.
     */
    private final void anzeigen() {

        final Nahkampfwaffe waffe = getGewaehlteWaffe();

        if ( null != waffe ) {            
            
            bf.setValue( waffe.getBf() );

            dkH.setSelected( waffe.istDkH() );

            dkN.setSelected( waffe.istDkN() );

            dkS.setSelected( waffe.istDkS() );

            id.setText( waffe.getId().replace( "Waffe.", "" ) );

            ini.setValue( waffe.getIni() );

            name.setText( waffe.getName() );

            talent.setSelectedItem( waffe.getTalent() );

            talente.setGewaehlteElemente( waffe.getTalente() );

            tp.setText( waffe.getTp().toString() );

            tpKk.setText( String.format( "%1$s/%2$s",
                    waffe.getSchwellenwert(),
                    waffe.getSchadensschritt() ) );

            wm.setText( String.format( "%1$+d/%2$+d",
                    waffe.getWmAt(),
                    waffe.getWmPa() ) );
        }
    }

    private final void aufbauenBenutzeroberflaeche() {

        final JLabel bfLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaNahkampfwaffen.Bruchfaktor" );

        final JLabel dkLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaNahkampfwaffe.Dk" );

        final JButton entfernen = uebersetzer.erzeugeButton( 
                "PanelMetaNahkampfwaffen.Entfernen" );
        entfernen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                entfernen();
            }
        });

        final JLabel idLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaNahkampfwaffen.Id" );
        
        final JLabel iniLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaNahkampfwaffen.Initiative" );
        
        final JLabel nameLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaNahkampfwaffen.Name" );

        final JTabbedPane tabbedPane = new JTabbedPane();

        final JLabel talentLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaNahkampfwaffen.Talent" );

        final JLabel tpLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaNahkampfwaffen.Trefferpunkte" );

        final JLabel tpKkLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaNahkampfwaffen.TpKk" );
        
        final JLabel wmLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaNahkampfwaffen.Waffenmodifikator" );

        final JPanel buttonpanel =
            ButtonBarFactory.buildLeftAlignedBar( uebernehmen );

        String spalten = "7dlu, P, 4dlu, P, P:G, 7dlu";

        String zeilen = "7dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, " +
                 "4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 7dlu";

        FormLayout layout = new FormLayout( spalten, zeilen );

        final JPanel panelDaten = new JPanel( layout );

        final CellConstraints cc = new CellConstraints();

        int y = 0;

        panelDaten.add( nameLabel, cc.xy ( 2, y+=2 ) );
        panelDaten.add( name,      cc.xyw( 4, y, 2 ) );

        panelDaten.add( idLabel, cc.xy ( 2, y+=2 ) );
        panelDaten.add( id,      cc.xyw( 4, y, 2 ) );

        panelDaten.add( talentLabel, cc.xy ( 2, y+=2 ) );
        panelDaten.add( talent,      cc.xyw( 4, y, 2 ) );

        panelDaten.add( tpLabel, cc.xy ( 2, y+=2 ) );
        panelDaten.add( tp,      cc.xyw( 4, y, 2 ) );

        panelDaten.add( tpKkLabel, cc.xy ( 2, y+=2 ) );
        panelDaten.add( tpKk,      cc.xyw( 4, y, 2 ) );

        panelDaten.add( bfLabel, cc.xy( 2, y+=2 ) );
        panelDaten.add( bf,      cc.xy( 4, y ) );

        panelDaten.add( iniLabel, cc.xy( 2, y+=2 ) );
        panelDaten.add( ini,      cc.xy( 4, y ) );

        panelDaten.add( wmLabel, cc.xy( 2, y+=2 ) );
        panelDaten.add( wm,      cc.xy( 4, y ) );

        spalten = "P, 4dlu, P, 4dlu, P";

        zeilen = "P";

        final FormLayout dkLayout = new FormLayout( spalten, zeilen );

        final JPanel dk = new JPanel( dkLayout );
        dk.add( dkH, cc.xy( 1, 1 ) );
        dk.add( dkN, cc.xy( 3, 1 ) );
        dk.add( dkS, cc.xy( 5, 1 ) );

        panelDaten.add( dkLabel, cc.xy ( 2, y+=2 ) );
        panelDaten.add( dk,               cc.xyw( 4, y , 2) );

        final JScrollPane panelDatenScrollPane =
            Util.erzeugeScrollPane( panelDaten );
        resetter.einfuegen( panelDatenScrollPane );
        
        uebersetzer.erzeugeTab(
                tabbedPane,
                "PanelMetaNahkampfwaffen.Diverses", 
                panelDatenScrollPane );

        uebersetzer.erzeugeTab( 
                tabbedPane, 
                "PanelMetaNahkampfwaffen.Talente", 
                talente );

        spalten = "7dlu, 200px:G(0.2), 4dlu, 200px:G(0.8), 7dlu";

        zeilen = "7dlu, F:200:G, 4dlu, P, 4dlu, P, 7dlu";

        layout = new FormLayout( spalten, zeilen );

        setLayout( layout );

        add( new JScrollPane( waffen ), cc.xywh( 2, 2, 1, 3 ) );

        add( entfernen, cc.xy( 2, 6 ) );

        add( tabbedPane, cc.xy( 4, 2 ) );

        add( buttonpanel, cc.xy( 4, 4 ) );
    }

    private void entfernen() {

        final Nahkampfwaffe waffe = getGewaehlteWaffe();

        if ( null != waffe ) {

            speicher.entfernen( waffe );

            waffenListModel.removeElement( waffe );

            speichern();
        }
    }

    /**
     * @return Die Waffe die in der Liste auf der linken Seite angewaehlt ist,
     *         oder {@code null}, wenn keine Waffe gewaehlt ist.
     */
    private Nahkampfwaffe getGewaehlteWaffe() {
        return (Nahkampfwaffe) waffen.getSelectedValue();
    }

    public boolean kannGewechseltWerden() {
        
        return true;
    }

    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
    }
    
    private boolean pruefen() {

        boolean ergebnis = true;

        if ( id.getText().trim().equals( "" ) ) {

            id.setBackground( Util.FEHLERFARBE );

            ergebnis = false;

        } else {

            id.setBackground( name.getBackground() );
        }

        if ( Tp.istTpFormel( tp.getText().trim() ) ) {

            tp.setBackground( name.getBackground() );

        } else {

            tp.setBackground( Util.FEHLERFARBE );
        }

        Matcher matcher = tpKkPattern.matcher( tpKk.getText().trim() );

        if ( matcher.matches() ) {

            tpKk.setBackground( name.getBackground() );

        } else {

            tpKk.setBackground( Util.FEHLERFARBE );

            ergebnis = false;
        }

        matcher = wmPattern.matcher( wm.getText().trim() );

        if ( matcher.matches() ) {

            wm.setBackground( name.getBackground() );

        } else {

            wm.setBackground( Util.FEHLERFARBE );

            ergebnis = false;
        }

        uebernehmen.setEnabled( ergebnis );

        return ergebnis;
    }

    private final void speichern() {

        final Einstellungen einstellungen = Einstellungen.getInstanz();

        if ( einstellungen.istDatenquelleVerzeichnis() ) {

            Datenspeicherer.speichereNahkampfwaffen();

        } else if ( ! hinweisGezeigt ){

            JOptionPane.showMessageDialog( 
                    this, 
                    I18n.i18n( "PanelMetaNahkampfwaffen.HinweisDatenquelle" ) );

            hinweisGezeigt = true;
        }

    }

    /**
     * Wandelt einen String, der eine ganze Zahl, mit Vorzeichen, darstellt
     * in einen int-Wert um.
     *
     * @param s
     *            Die Zahl als String.
     * @return die Zahl aus dem String.
     */
    private final int toInt( String s ) {

        if ( s.charAt( 0 ) == '+' ) {

            s = s.substring( 1 );
        }

        return Integer.valueOf( s );
    }

    /**
     * Uebernimmt die im Panel eingegebenen Ruestung in die Liste der
     * Ruestungen.
     */
    private final void uebernehmen() {

        if ( ! pruefen() ) {
            return;
        }

        String idString = id.getText().trim();

        idString = "Waffe." + idString;

        final Nahkampfwaffe waffe = speicher.getNahkampfwaffe( idString );

        waffe.setBf( (Integer) bf.getValue() );

        waffe.setDkH( dkH.isSelected() );

        waffe.setDkN( dkN.isSelected() );

        waffe.setDkS( dkS.isSelected() );

        waffe.setIni( (Integer) ini.getValue() );

        waffe.setName( name.getText().trim() );

        waffe.setTalent( (Kampftechnik) talent.getSelectedItem() );

        waffe.setTalente( talente.getGewaehlteElemente() );

        waffe.setTp( new Tp( tp.getText().trim() ) );

        final String[] tpKkArray = tpKk.getText().trim().split( "/" );

        waffe.setSchwellenwert( Integer.valueOf( tpKkArray[ 0 ] ) );

        waffe.setSchadensschritt( Integer.valueOf( tpKkArray[ 1 ] ) );

        final String[] wmArray = wm.getText().trim().split( "/" );

        waffe.setWmAt( toInt( wmArray[ 0 ] ) );

        waffe.setWmPa( toInt( wmArray[ 1 ] ) );

        speicher.einfuegen( waffe );

        speichern();

        waffenListModel.setElements( speicher.getNahkampfwaffen() );
    }
}
