/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collections;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.namen.Namenliste;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.daten.Datenspeicherer;
import de.bernhardjung.dsaprogramm.gui.CharGenListModel;
import de.bernhardjung.dsaprogramm.gui.WechselbaresPanel;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Einstellungen;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

public class PanelMetaNamenlisten
extends WechselbaresPanel 
implements Uebersetzbar {

    private static final String PRAEFIX_ID = "Namenliste.";

    /** Das Textfeld in das man die Bezeichnung der Liste eingeben kann. */
    private final JTextField bezeichnung;
    
    private boolean hinweisGezeigt;
    
    /** Hier muss man die ID der Liste eingeben. */
    private final JTextField id;
    
    /**
     * In diesen Editor kann man alle Namen fuer eine Liste eingeben.
     */
    private final PanelMetaNamenliste nameListe;
    
    /**
     * Die Liste auf der linken Seite mit den Namenlisten. 
     */
    private final JList namenlisten;
    
    /**
     * Das Model fuer die liste mit den Namen.
     */
    private final CharGenListModel< Namenliste, Namenliste > namenlisteListModel;
    
    /** 
     * Der Button mit dem man die eingegebene List von Namen in die Liste der
     * Namenslisten uebernimmt.
     */
    private final JButton uebernehmen;
    
    /**
     * Uebersetzt die Texte der Benutzeroberflaeche.
     */
    private final Uebersetzer uebersetzer;
    
    public PanelMetaNamenlisten() {
        
        uebersetzer = new Uebersetzer();
        
        bezeichnung = new JTextField();
        
        hinweisGezeigt = false;
        
        id = new JTextField();
        id.getDocument().addDocumentListener( new DocumentListener() {
            public void changedUpdate( final DocumentEvent e ) {}

            public void insertUpdate( final DocumentEvent e ) {
                pruefen();
            }
            public void removeUpdate( final DocumentEvent e ) {
                pruefen();
            }
        });
        
        nameListe = new PanelMetaNamenliste();
        
        final Datenspeicher speicher = Datenspeicher.getInstanz();
        
        namenlisteListModel = new CharGenListModel< Namenliste, Namenliste >(
                speicher.getNamenlisten(),
                Namenliste.BEZEICHNUNG_COMPARATOR );
        
        namenlisten = uebersetzer.erzeugeListe( namenlisteListModel );
        namenlisten.addListSelectionListener( new ListSelectionListener() {
            public void valueChanged( ListSelectionEvent e ) {
                anzeigen();
            }
        } );
        
        uebernehmen = uebersetzer.erzeugeButton( 
                "PanelMetaNamenlisten.Uebernehmen" );
        uebernehmen.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                uebernehmen();
            }
        } );
        
        uebersetzer.einfuegen( nameListe );
        
        aufbauenBenutzeroberflaeche();
        
        pruefen();
    }

    @SuppressWarnings("unchecked")
    private void anzeigen() {
        
        final Namenliste namenliste = getGewaehlteNamenliste();
        
        if ( null != namenliste ) {
            
            bezeichnung.setText( namenliste.getBezeichnung() );
            
            id.setText( namenliste.getId().replace( 
                    PRAEFIX_ID, 
                    "" ) );
            
            nameListe.setNamen( namenliste.getNamen() );
        
        } else {
            
            bezeichnung.setText( "" );
            
            id.setText( "" );
            
            nameListe.setNamen( Collections.EMPTY_LIST );
        }
    }
    
    private void aufbauenBenutzeroberflaeche() {
        
        final String zeilen = "7dlu, F:P:G, 7dlu";
        
        final String spalten = "7dlu, F:100px:G(0.2), 4dlu, F:P:G(0.8), 7dlu";

        final FormLayout layout = new FormLayout( spalten, zeilen );
        
        setLayout( layout );
        
        final CellConstraints cc = new CellConstraints();
        
        // Die Liste mit den Namenslisten
        add( erzeugeLinkeSpalte(), cc.xy( 2, 2 ) );
        
        // Hier werden die Namen und die Bezeichnung der Liste eingegeben.
        add( erzeugeRechteSpalte(), cc.xy( 4, 2 ) );
    }

    /** 
     * Entfernt die aktuell in der Liste gewaehlte Factory aus der Liste der
     * Factories. 
     */
    private void entfernen() {
        
        final Namenliste namenliste = getGewaehlteNamenliste();
        
        if ( null == namenliste )
        {
            return;
        }
        
        final Datenspeicher speicher = Datenspeicher.getInstanz();
        
        speicher.entfernen( namenliste );
        
        speichern();
        
        namenlisteListModel.removeElement( namenliste );
    }

    /**
     * Erzeugt den Teil in dem die Daten ueber die Liste und die Liste der Namen
     * eingegeben wird.
     */
    private JComponent erzeugeRechteSpalte() {
        
        final JPanel panel = new JPanel();
        
        panel.setLayout( new BorderLayout() );
        
        // Der obere Teil mit den Textfeldern.
        panel.add( erzeugeRechteSpalteOben(), BorderLayout.PAGE_START );
        
        // Der mittlere Teil in dem die Liste der Namen eingegeben wird.
        panel.add( erzeugeRechteSpalteMitte(), BorderLayout.CENTER );
        
        // Der untere Teil mit dem Button zum Uebernehmen der Aenderungen.
        panel.add( uebernehmen, BorderLayout.PAGE_END );
        
        return panel;
    }

    /**
     * Der mittlere Teil, in dem man die Namen eingeben kann.
     */
    private JComponent erzeugeRechteSpalteMitte() {
        
        final JLabel nameListeUeberschrift = uebersetzer.erzeugeUeberschrift( 
                "PanelMetaNamenlisten.Namen" );
        
        final String spalten = "F:P:G";
        
        final String zeilen = "P, 4dlu, F:P:G";
        
        final FormLayout layout = new FormLayout( spalten, zeilen );
        
        final DefaultFormBuilder builder = new DefaultFormBuilder( layout );
        
        builder.append( nameListeUeberschrift );
        
        builder.nextLine( 2 );
        builder.append( nameListe );
        
        return builder.getPanel();
    }

    /**
     * Die Textfelder fuer Id, Bezeichnung usw.
     */
    private JComponent erzeugeRechteSpalteOben() {

        final JLabel bezeichnungLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaNamenlisten.Bezeichnung" );
        
        final JLabel idLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaNamenlisten.Id" );
        
        final FormLayout layout = new FormLayout( "P, 4dlu, F:P:G" );
        
        final DefaultFormBuilder builder = new DefaultFormBuilder( layout );
        
        builder.append( idLabel, id );
        
        builder.append( bezeichnungLabel, bezeichnung );
        
        return builder.getPanel();
    }

    /**
     * @return Die Namenliste die in der Liste auf der linken Seite gewaehlt
     *         ist oder {@code null}, wenn keine Liste gewaehlt ist.
     */
    private Namenliste getGewaehlteNamenliste() {
        return (Namenliste) namenlisten.getSelectedValue();
    }
    
    public boolean kannGewechseltWerden() {
        
        return true;
    }

    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
    }
    
    /**
     * Die Liste mit den Listen.
     */
    private JComponent erzeugeLinkeSpalte() {
        
        final JButton entfernen = uebersetzer.erzeugeButton( 
                "PanelMetaNamenlisten.Entfernen" );
        entfernen.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                entfernen();
            }
        } );
        
        final JScrollPane scrollPane = new JScrollPane( namenlisten );
        
        final JPanel panel = new JPanel( new BorderLayout() );
        
        panel.add( scrollPane, BorderLayout.CENTER );
        
        panel.add( entfernen, BorderLayout.PAGE_END );
        
        return panel;
    }

    /**
     * Prueft die Eingaben im Panel auf Korrektheit.
     *
     * @return {@code true}, wenn alles Ok ist, sonst {@code false}.
     */
    private boolean pruefen() {

        boolean ergebnis = true;

        if ( id.getText().trim().equals( "" ) ) {
            
            id.setBackground( Util.FEHLERFARBE );
            
            ergebnis = false;
        
        } else {
            
            id.setBackground( bezeichnung.getBackground() );
        }
        
        uebernehmen.setEnabled( ergebnis );

        return ergebnis;
    }

    /**
     * Speichert die Factorys auf die Festplatte.
     */
    private final void speichern() {

        final Einstellungen einstellungen = Einstellungen.getInstanz();

        if ( einstellungen.istDatenquelleVerzeichnis() ) {

            Datenspeicherer.speichereNameFactorys();

        } else if ( ! hinweisGezeigt ) {

            JOptionPane.showMessageDialog( 
                    this, 
                    I18n.i18n( "PanelMetaNamenlisten.HinweisDatenquelle" ) );
            
            hinweisGezeigt = true;
        }
    }

    private void uebernehmen() {
        
        if ( ! pruefen()  ) {
            
            return;
        }
        
        final String id = PRAEFIX_ID + this.id.getText().trim();
        
        Datenspeicher speicher = Datenspeicher.getInstanz();
        
        final Namenliste namenliste = speicher.getNamenliste( id );
        
        namenliste.setBezeichnung( bezeichnung.getText().trim() );
        
        namenliste.setNamen( nameListe.getNamen() );
        
        speicher.einfuegen( namenliste );
        
        speichern();
        
        namenlisteListModel.setElements( speicher.getNamenlisten() );
    }
        
}
