/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collections;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import com.jgoodies.forms.factories.ButtonBarFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif_lite.panel.SimpleInternalFrame;

import de.bernhardjung.dsaprogramm.bausteine.Profession;
import de.bernhardjung.dsaprogramm.gui.Hauptpanel;
import de.bernhardjung.dsaprogramm.gui.WechselbaresPanel;
import de.bernhardjung.dsaprogramm.gui.action.ActionLadenProfession;
import de.bernhardjung.dsaprogramm.gui.action.ActionSpeichernProfession;
import de.bernhardjung.dsaprogramm.util.LookAndFeelAktualisierbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

public class PanelMetaProfession
extends WechselbaresPanel
implements Uebersetzbar, LookAndFeelAktualisierbar {
    
    /**
     * Wird verwendet, um die eingegebenen Profession auf der Festplatte zu
     * speichern.
     */
    private final ActionLadenProfession actionLadenProfession;

    /**
     * Laedt eine Profession von der Festplatte.
     */
    private final ActionSpeichernProfession actionSpeichernProfession;

    private final PanelMetaBesondererBesitz besondererBesitz;

    private final PanelMetaProfessionDiverses diverses;

    private final PanelMetaListe< VorNachteil, VorNachteil > empfohlene;

    private final PanelMetaLeittalente leittalente;

    private final PanelMetaModifikationen modifikationen;

    private final PanelMetaVorNachteile nachteile;

    private final PanelMetaSonderfertigkeiten sonderfertigkeiten;

    private final PanelMetaTalentboni talentboni;

    /** Uebersetzt die Texte der Benutzeroberflaeche. */
    private final Uebersetzer uebersetzer;
    
    private final PanelMetaUngeeigneteVorNachteile ungeeignete;

    private final PanelMetaVerbilligteSonderfertigkeiten verbilligteSf;

    private final PanelMetaVerbilligteVarianten verbilligteVarianten;

    private final PanelMetaVoraussetzungen voraussetzungen;

    private final PanelMetaVorNachteile vorteile;

    private final PanelMetaZauberboni zauberboni;

    @SuppressWarnings( "unchecked" )
    public PanelMetaProfession(
            final JFrame hauptfenster,
            final Hauptpanel hauptpanel ) {

        uebersetzer = new Uebersetzer();
        
        besondererBesitz = new PanelMetaBesondererBesitz();
        resetter.einfuegen( besondererBesitz );

        diverses = new PanelMetaProfessionDiverses( this );
        resetter.einfuegen( diverses );

        empfohlene = new PanelMetaListe< VorNachteil, VorNachteil >(
                VorNachteil.getVorNachteile(), 
                VorNachteil.NAME_COMPARATOR );
        resetter.einfuegen( empfohlene );

        leittalente = new PanelMetaLeittalente();
        resetter.einfuegen( leittalente );

        modifikationen = new PanelMetaModifikationen( Collections.EMPTY_LIST );
        resetter.einfuegen( modifikationen );

        nachteile = new PanelMetaVorNachteile( 
                VorNachteil.getNachteile(),
                "PanelMetaProfession.Nachteile.Reiter" );
        resetter.einfuegen( nachteile );

        sonderfertigkeiten = new PanelMetaSonderfertigkeiten();
        resetter.einfuegen( sonderfertigkeiten );

        talentboni = new PanelMetaTalentboni(
                PanelMetaTalentboni.Modus.PROFESSION );
        resetter.einfuegen( talentboni );

        ungeeignete = new PanelMetaUngeeigneteVorNachteile();
        resetter.einfuegen( ungeeignete );

        verbilligteSf = new PanelMetaVerbilligteSonderfertigkeiten();
        resetter.einfuegen( verbilligteSf );

        verbilligteVarianten = new PanelMetaVerbilligteVarianten();
        resetter.einfuegen( verbilligteVarianten );

        voraussetzungen = new PanelMetaVoraussetzungen();
        resetter.einfuegen( voraussetzungen );

        vorteile = new PanelMetaVorNachteile( 
                VorNachteil.getVorteile(),
                "PanelMetaProfession.Vorteile.Reiter" );
        resetter.einfuegen( vorteile );

        zauberboni = new PanelMetaZauberboni();
        resetter.einfuegen( zauberboni );

        // Frames
        // ******
        final SimpleInternalFrame diversesFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaProfession.Diverses", 
                diverses );

        final SimpleInternalFrame modifikationenFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaProfession.Modifikationen", 
                modifikationen );
            
        final SimpleInternalFrame voraussetzungenFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaProfession.Voraussetzungen",
                voraussetzungen );

        final SimpleInternalFrame besondererBesitzFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaProfession.BesondererBesitz", 
                besondererBesitz );
        
        final SimpleInternalFrame talentboniFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaProfession.Talente", 
                talentboni );

        final SimpleInternalFrame leittalenteFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaProfession.Leittalente", 
                leittalente );
        
        final SimpleInternalFrame vorteileFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaProfession.Vorteile", 
                vorteile );

        final SimpleInternalFrame nachteileFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaProfession.Nachteile", 
                nachteile );

        final SimpleInternalFrame empfohleneFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaProfession.EmpfohleneVorNachteile", 
                empfohlene );
        
        final SimpleInternalFrame ungeeigneteFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaProfession.UngeeigneteVorNachteile", 
                ungeeignete );
        
        final SimpleInternalFrame sonderfertigkeitenFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaProfession.Sonderfertigkeiten",
                sonderfertigkeiten );

        final SimpleInternalFrame verbilligteSfFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaProfession.VerbilligteSonderfertigkeiten", 
                verbilligteSf );

        final SimpleInternalFrame verbilligteVariantenFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaProfession.VerbilligteVarianten",
                verbilligteVarianten );

        final SimpleInternalFrame zauberboniFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaProfession.Zauber", 
                zauberboni );

        // Button Bar
        // **********
        actionLadenProfession = new ActionLadenProfession( hauptpanel );
        uebersetzer.einfuegen( actionLadenProfession );
        
        actionSpeichernProfession = new ActionSpeichernProfession( hauptpanel );
        uebersetzer.einfuegen( actionSpeichernProfession );
        
        final JButton neu = uebersetzer.erzeugeButton( 
                "PanelMetaProfession.Neu" );
        neu.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                neu();
            }
        } );

        final JPanel buttonBar = new JPanel( new FormLayout( "4dlu, P, 4dlu",
                "4dlu, P, 4dlu" ) );

        final CellConstraints cc = new CellConstraints();

        buttonBar.add( ButtonBarFactory.buildLeftAlignedBar( new JButton(
                actionLadenProfession ),
                new JButton( actionSpeichernProfession ), neu ), cc.xy( 2, 2 ) );

        // Tabbed Pane
        // ***********
        final JTabbedPane tabbedPane = new JTabbedPane( JTabbedPane.LEFT );
        resetter.einfuegen( tabbedPane );
        
        uebersetzer.erzeugeTab(
                tabbedPane, 
                "PanelMetaProfession.Diverses.Reiter",
                diversesFrame );

        uebersetzer.erzeugeTab(
                tabbedPane, 
                "PanelMetaProfession.Modifikationen.Reiter", 
                modifikationenFrame );

        uebersetzer.erzeugeTab(
                tabbedPane, 
                "PanelMetaProfession.Voraussetzungen.Reiter", 
                voraussetzungenFrame );

        uebersetzer.erzeugeTab(
                tabbedPane, 
                "PanelMetaProfession.BesondererBesitz.Reiter",
                besondererBesitzFrame );

        uebersetzer.erzeugeTab(
                tabbedPane, 
                "PanelMetaProfession.Talente.Reiter", 
                talentboniFrame );

        uebersetzer.erzeugeTab(
                tabbedPane, 
                "PanelMetaProfession.Leittalente.Reiter", 
                leittalenteFrame );

        uebersetzer.erzeugeTab(
                tabbedPane, 
                "PanelMetaProfession.Vorteile.Reiter", 
                vorteileFrame );

        uebersetzer.erzeugeTab(
                tabbedPane, 
                "PanelMetaProfession.Nachteile.Reiter", 
                nachteileFrame );

        uebersetzer.erzeugeTab(
                tabbedPane, 
                "PanelMetaProfession.EmpfohleneVorNachteile.Reiter", 
                empfohleneFrame );

        uebersetzer.erzeugeTab(
                tabbedPane, 
                "PanelMetaProfession.UngeeigneteVorNachteile.Reiter", 
                ungeeigneteFrame );

        uebersetzer.erzeugeTab(
                tabbedPane, 
                "PanelMetaProfession.Sonderfertigkeiten.Reiter", 
                sonderfertigkeitenFrame );

        uebersetzer.erzeugeTab(
                tabbedPane, 
                "PanelMetaProfession.VerbilligteSonderfertigkeiten.Reiter", 
                verbilligteSfFrame );

        uebersetzer.erzeugeTab(
                tabbedPane, 
                "PanelMetaProfession.VerbilligteVarianten.Reiter",
                verbilligteVariantenFrame );

        uebersetzer.erzeugeTab(
                tabbedPane, 
                "PanelMetaProfession.Zauber.Reiter", 
                zauberboniFrame );

        setLayout( new BorderLayout() );

        add( buttonBar, BorderLayout.PAGE_START );

        add( tabbedPane, BorderLayout.CENTER );
    }

    public void aktualisiereLookAndFeel() {
        
        Util.aktualisiereLookAndFeel(
                actionLadenProfession,
                actionSpeichernProfession );
    }
    
    public final void anzeigen( final Profession profession ) {

        besondererBesitz.anzeigen( profession );

        diverses.anzeigen( profession );

        empfohlene
                .setGewaehlteElemente( profession.getEmpfohleneVorNachteile() );

        leittalente.anzeigen( profession );

        modifikationen.anzeigen( profession );

        nachteile.anzeigen( profession.getNachteile() );

        sonderfertigkeiten.anzeigen( profession.getSonderfertigkeiten() );

        talentboni.anzeigen( profession );

        ungeeignete.anzeigen( profession.getUngeeigneteVorNachteile() );

        verbilligteSf.anzeigen( profession );

        verbilligteVarianten.anzeigen( profession );

        voraussetzungen.anzeigen( profession );

        vorteile.anzeigen( profession.getVorteile() );

        zauberboni.anzeigen( profession );
    }

    /** @return die fuer die Profession gewaehlte Id. */
    public String getProfessionId() {
        return diverses.getProfessionId();
    }

    /**
     * @return den Namen, den der Benutzer fuer die Profession angegeben hat.
     */
    public String getProfessionName() {
        return diverses.getProfessionName();
    }

    public boolean kannGewechseltWerden() {
        
        return true;
    }

    /**
     * Setzt das Panel wieder zurueck, damit man eine neue Profession eingeben
     * kann.
     */
    private void neu() {

        besondererBesitz.eingabenLoeschen();

        diverses.eingabenLoeschen();

        empfohlene.eingabenLoeschen();

        leittalente.eingabenLoeschen();

        modifikationen.eingabenLoeschen();

        nachteile.eingabenLoeschen();

        sonderfertigkeiten.eingabenLoeschen();

        talentboni.eingabenLoeschen();

        ungeeignete.eingabenLoeschen();

        verbilligteSf.eingabenLoeschen();

        verbilligteVarianten.eingabenLoeschen();

        voraussetzungen.eingabenLoeschen();

        vorteile.eingabenLoeschen();

        zauberboni.eingabenLoeschen();
    }
    
    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
    }

    public final void uebertrageDaten( final Profession profession ) {

        profession.setBesondererBesitz( besondererBesitz.getBesondererBesitz() );

        diverses.uebertrageDaten( profession );

        profession.setEmpfohlene( empfohlene.getGewaehlteElemente() );

        leittalente.uebertrageDaten( profession );

        modifikationen.uebertrageDaten( profession );

        profession.setNachteile( nachteile.getBoni() );

        profession.setSonderfertigkeiten( sonderfertigkeiten.getBoni() );

        profession.setTalente( talentboni.getBoni() );

        profession.setUngeeignete( ungeeignete.getGewaehlteVorNachteile() );

        profession.setVerbilligte( verbilligteSf
                .getVerbilligteSonderfertigkeiten() );

        profession.setVerbilligteVariante( verbilligteVarianten
                .getVerbilligteVarianten() );

        profession.setVoraussetzungen( voraussetzungen.getVoraussetzungen() );

        profession.setVorteile( vorteile.getBoni() );

        profession.setZauberboni( zauberboni.getBoni() );
    }

}
