/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Arrays;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import com.jgoodies.forms.factories.ButtonBarFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif_lite.panel.SimpleInternalFrame;

import de.bernhardjung.dsaprogramm.basis.Augenfarbe;
import de.bernhardjung.dsaprogramm.basis.Haarfarbe;
import de.bernhardjung.dsaprogramm.basiswerte.Basiswert;
import de.bernhardjung.dsaprogramm.bausteine.Rasse;
import de.bernhardjung.dsaprogramm.gui.Hauptpanel;
import de.bernhardjung.dsaprogramm.gui.WechselbaresPanel;
import de.bernhardjung.dsaprogramm.gui.action.ActionLadenRasse;
import de.bernhardjung.dsaprogramm.gui.action.ActionSpeichernRasse;
import de.bernhardjung.dsaprogramm.gui.meta.PanelMetaWuerfeltabelle.ElementQuelle;
import de.bernhardjung.dsaprogramm.util.LookAndFeelAktualisierbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

public class PanelMetaRasse 
extends WechselbaresPanel
implements Uebersetzbar, LookAndFeelAktualisierbar {

    private final ActionLadenRasse actionLadenRasse;
    
    private final ActionSpeichernRasse actionSpeichernRasse;
    
    private final PanelMetaWuerfeltabelle augenfarben;
    
    private final PanelMetaRasseDiverses diverses;
    
    private final PanelMetaListe< VorNachteil, VorNachteil > empfohlene;
    
    private final PanelMetaWuerfeltabelle haarfarben;
    
    private final PanelMetaListeKulturen kulturen;
    
    private final PanelMetaLeittalente leittalente;
    
    private final PanelMetaModifikationen modifikationen;
    
    private final PanelMetaVorNachteile nachteile;
    
    private final PanelMetaTalentboni talentboni;
    
    /** Uebersetzt die Texte der Benutzeroberflaeche. */
    private final Uebersetzer uebersetzer;
    
    private final PanelMetaUngeeigneteVorNachteile ungeeignete;
    
    private final PanelMetaVorNachteile vorteile;
    
    public PanelMetaRasse(
            final JFrame hauptfenster,
            final Hauptpanel hauptpanel ) {

        uebersetzer = new Uebersetzer();
    
        augenfarben = new PanelMetaWuerfeltabelle( new ElementQuelle(){
            public Object getElement( final String id ) {
                
                return Augenfarbe.getAugenfarbe( id );
            }
            public Vector< Object > getElemente() {
                
                return new Vector< Object >( Augenfarbe.getAugenfarben() );
            }
            public String getId( final Object augenfarbe ) {
                
                return ((Augenfarbe) augenfarbe).getId();
            }
        });
        resetter.einfuegen( augenfarben );

        diverses = new PanelMetaRasseDiverses( this );
        resetter.einfuegen( diverses );

        empfohlene = new PanelMetaListe< VorNachteil, VorNachteil >(
                VorNachteil.getVorNachteile(),
                VorNachteil.NAME_COMPARATOR );
        resetter.einfuegen( empfohlene );

        haarfarben = new PanelMetaWuerfeltabelle( new ElementQuelle(){
            public Object getElement( final String id ) {
                
                return Haarfarbe.getHaarfarbe( id );
            }
            public Vector< Object > getElemente() {
                
                return new Vector< Object >( Haarfarbe.getHaarfarben() );
            }
            public String getId( final Object haarfarbe ) {
                
                return ((Haarfarbe) haarfarbe ).getId();
            }
        });
        resetter.einfuegen( haarfarben );

        kulturen = new PanelMetaListeKulturen();
        resetter.einfuegen( kulturen );

        leittalente = new PanelMetaLeittalente();
        resetter.einfuegen( leittalente );

        modifikationen = new PanelMetaModifikationen(
                Arrays.asList( Basiswert.LE, Basiswert.AU, Basiswert.MR ) );
        resetter.einfuegen( modifikationen );

        nachteile = new PanelMetaVorNachteile( 
                VorNachteil.getNachteile(),
                "PanelMetaRasse.Nachteile.Reiter" );
        resetter.einfuegen( nachteile );

        talentboni = new PanelMetaTalentboni(
                PanelMetaTalentboni.Modus.RASSE );
        resetter.einfuegen( talentboni );

        ungeeignete = new PanelMetaUngeeigneteVorNachteile();
        resetter.einfuegen( ungeeignete );

        vorteile = new PanelMetaVorNachteile( 
                VorNachteil.getVorteile(),
                "PanelMetaRasse.Vorteile.Reiter" );
        resetter.einfuegen( vorteile );

        // Button Bar
        // **********
        
        actionLadenRasse = new ActionLadenRasse( hauptpanel );
        uebersetzer.einfuegen( actionLadenRasse );
        
        actionSpeichernRasse = new ActionSpeichernRasse( hauptpanel );
        uebersetzer.einfuegen( actionSpeichernRasse );
        
        final JButton neu = uebersetzer.erzeugeButton( 
                "PanelMetaRasse.Neu" );
        neu.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                neu();
            }
        });

        final JPanel buttonBar = new JPanel(
                new FormLayout( "4dlu, P, 4dlu", "4dlu, P, 4dlu" ) );

        final CellConstraints cc = new CellConstraints();

        buttonBar.add( ButtonBarFactory.buildLeftAlignedBar(
                    new JButton( actionLadenRasse ),
                    new JButton( actionSpeichernRasse ),
                    neu ),
                    cc.xy( 2, 2 ) );

        // Frames
        // ******
        final SimpleInternalFrame diversesFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaRasse.Diverses",
                diverses );

        final SimpleInternalFrame modifikationenFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaRasse.Modifikationen",
                modifikationen );
        
        final SimpleInternalFrame augenfarbenFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaRasse.Augenfarben",
                augenfarben );
        
        final SimpleInternalFrame haarfarbenFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaRasse.Haarfarben",
                haarfarben );

        final SimpleInternalFrame kulturenFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaRasse.Kulturen",
                kulturen );

        final SimpleInternalFrame talentboniFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaRasse.Talente",
                talentboni );

        final SimpleInternalFrame leittalenteFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaRasse.Leittalente",
                leittalente );

        final SimpleInternalFrame vorteileFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaRasse.Vorteile",
                vorteile );

        final SimpleInternalFrame nachteileFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaRasse.Nachteile", 
                nachteile );

        final SimpleInternalFrame empfohleneFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaRasse.EmpfohleneVorNachteile",
                empfohlene );

        final SimpleInternalFrame ungeeigneteFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaRasse.UngeeigneteVorNachteile",
                ungeeignete );

        // Tabbed Pane
        // ***********
        final JTabbedPane tabbedPane = new JTabbedPane( JTabbedPane.LEFT );
        resetter.einfuegen( tabbedPane );
        
        uebersetzer.erzeugeTab(
                tabbedPane, 
                "PanelMetaRasse.Diverses.Reiter",
                diversesFrame );

        uebersetzer.erzeugeTab( 
                tabbedPane, 
                "PanelMetaRasse.Modifikationen.Reiter",
                modifikationenFrame );

        uebersetzer.erzeugeTab( 
                tabbedPane, 
                "PanelMetaRasse.Augenfarben.Reiter",
                augenfarbenFrame );

        uebersetzer.erzeugeTab( 
                tabbedPane, 
                "PanelMetaRasse.Haarfarben.Reiter",
                haarfarbenFrame );

        uebersetzer.erzeugeTab( 
                tabbedPane, 
                "PanelMetaRasse.Kulturen.Reiter",
                kulturenFrame );

        uebersetzer.erzeugeTab( 
                tabbedPane, 
                "PanelMetaRasse.Talente.Reiter",
                talentboniFrame );
        
        uebersetzer.erzeugeTab( 
                tabbedPane, 
                "PanelMetaRasse.Leittalente.Reiter",
                leittalenteFrame );
        
        uebersetzer.erzeugeTab( 
                tabbedPane, 
                "PanelMetaRasse.Vorteile.Reiter",
                vorteileFrame );

        uebersetzer.erzeugeTab( 
                tabbedPane, 
                "PanelMetaRasse.Nachteile.Reiter",
                nachteileFrame );
        
        uebersetzer.erzeugeTab( 
                tabbedPane, 
                "PanelMetaRasse.EmpfohleneVorNachteile.Reiter",
                empfohleneFrame );

        uebersetzer.erzeugeTab( 
                tabbedPane, 
                "PanelMetaRasse.UngeeigneteVorNachteile.Reiter",
                ungeeigneteFrame );

        setLayout( new BorderLayout() );

        add( buttonBar, BorderLayout.PAGE_START  );

        add( tabbedPane, BorderLayout.CENTER );
    }

    public void aktualisiereLookAndFeel() {
        
        Util.aktualisiereLookAndFeel( 
                actionLadenRasse,
                actionSpeichernRasse );
    }
    
    public final void anzeigen( final Rasse rasse ) {

        augenfarben.anzeigen( rasse.getAugenfarben() );

        diverses.anzeigen( rasse );

        empfohlene.setGewaehlteElemente( rasse.getEmpfohleneVorNachteile() );

        haarfarben.anzeigen( rasse.getHaarfarben() );

        kulturen.setGewaehlteElemente( rasse.getKulturen() );

        leittalente.anzeigen( rasse );

        modifikationen.anzeigen( rasse );

        nachteile.anzeigen( rasse.getNachteile() );

        talentboni.anzeigen( rasse );

        ungeeignete.anzeigen( rasse.getUngeeigneteVorNachteile() );

        vorteile.anzeigen( rasse.getVorteile() );
    }

    /** @return Die Id, die der Benutzer fuer die Rasse angegeben hat. */
    public String getRassenId() {
        return diverses.getRassenId();
    }

    /** @return den Namen, den der Benutzer fuer die Rasse angegeben hat. */
    public String getRassenName() {
        return diverses.getRassenName();
    }
    
    public boolean kannGewechseltWerden() {
        
        return true;
    }

    /**
     * Setzt das Panel wieder zurueck, damit man eine neue Kultur
     * eingeben kann.
     */
    private void neu() {

        diverses.eingabenLoeschen();

        empfohlene.eingabenLoeschen();

        kulturen.eingabenLoeschen();

        leittalente.eingabenLoeschen();

        modifikationen.eingabenLoeschen();

        nachteile.eingabenLoeschen();

        talentboni.eingabenLoeschen();

        ungeeignete.eingabenLoeschen();

        vorteile.eingabenLoeschen();
    }

    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
    }

    public final void uebertrageDaten( final Rasse rasse ) {

        rasse.setAugenfarben( augenfarben.getWuerfeltabelle() );

        diverses.uebertrageDaten( rasse );

        rasse.setEmpfohlene( empfohlene.getGewaehlteElemente() );

        rasse.setHaarfarben( haarfarben.getWuerfeltabelle() );

        rasse.setKulturen( kulturen.getGewaehlteElemente() );

        leittalente.uebertrageDaten( rasse );

        modifikationen.uebertrageDaten( rasse );

        rasse.setNachteile( nachteile.getBoni() );

        rasse.setTalente( talentboni.getBoni() );

        rasse.setUngeeignete( ungeeignete.getGewaehlteVorNachteile() );

        rasse.setVorteile( vorteile.getBoni() );
    }
}
