/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import de.bernhardjung.dsaprogramm.basis.Wuerfel;
import de.bernhardjung.dsaprogramm.bausteine.Rasse;
import de.bernhardjung.dsaprogramm.bausteine.kosten.KostenFest;
import de.bernhardjung.dsaprogramm.bausteine.util.AlterFormel;
import de.bernhardjung.dsaprogramm.bausteine.util.GewichtFormel;
import de.bernhardjung.dsaprogramm.bausteine.util.GroesseFormel;
import de.bernhardjung.dsaprogramm.bausteine.voraussetzungen.Voraussetzung;
import de.bernhardjung.dsaprogramm.bausteine.voraussetzungen.VoraussetzungZwergin;
import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

class PanelMetaRasseDiverses extends ResetbaresPanel implements Uebersetzbar {

    /**
     * Der Abstand in Pixeln, der zwischen einzelnen Elementen der
     * Benutzeroberflaeche, bestehen soll.
     */
    private static final int ABSTAND_HORIZONTAL = 5;

    /**
     * Der vertikale Abstand in Pixeln, der zwischen einzelnen Elementen der
     * Benutzeroberflaeche, bestehen soll.
     */
    private static final int ABSTAND_VERTIKAL = 10;

    /** Der Text neben der Formel fuer das Alter. */
    private static final String ALTER = "PanelMetaRasseDiverses.Alter";

    /**
     * Der Text fuer den Knopf zum Berechnen der GP-Kosten.
     */
    private static final String BERECHNEN = "PanelMetaKulturDiverses.Berechnen";

    /** Der Text neben der Formel fuer das Gewicht. */
    private static final String GEWICHT = "PanelMetaRasseDiverses.Gewicht";

    /** Der Text neben der Formel fuer die Groesse. */
    private static final String GROESSE = "PanelMetaRasseDiverses.Groesse";

    /** Der Text neben der Textifeld fuer die Id. */
    private static final String ID = "PanelMetaRasseDiverses.Id";

    /** Der Text neben dem Spinner fuer die Kosten. */
    private static final String KOSTEN = "PanelMetaRasseDiverses.Kosten";

    /** Der Text neben dem Textfeld fuer den Namen. */
    private static final String NAME = "PanelMetaRasseDiverses.Name";

    /** Zwerginnen benoetigen MU mindestens 12. */
    private static final String ZWERGIN = "PanelMetaRasseDiverses.Zwergin";

    /** Der Spinner mit dem die Basisjahre fuer das Alter bestimmt werden. */
    private final JSpinner alterBasis;

    /**
     * Legt den Wuerfelwurf fest, der zu den Basisjahren fuer das Alter
     * addiert wird.
     */
    private final JComboBox alterWuerfel;

    /**
     * Die Zahl die von der Koerpergroesse abgezogen wird um das Gewicht zu
     * bestimmen.
     */
    private final JSpinner gewichtAbzug;

    /** Die Basisgroesse in Zentimetern. */
    private final JSpinner groesseBasis;

    /** Die Anzahl W20 die zur Basisgroesse addiert werden. */
    private final JSpinner groesseAnzahlW20;

    /** Die Anzahl W6 die zur Basisgroesse addiert werden. */
    private final JSpinner groesseAnzahlW6;

    /** Das Textfeld fuer die Id. */
    private final JTextField id;

    /** Der Spinner mit dem die Kosten der Rasse angegeben werden. */
    private final JSpinner kosten;

    /** Das Textfeld, in das der Name der Rasse eingegeben wird. */
    private final JTextField name;

    private final PanelMetaRasse panelRasse;

    
    
    /** Uebersetzt die Texte der Benutzeroberflaeche. */
    private final Uebersetzer uebersetzer;

    private final JLabel w6Label;

    private final JLabel w20Label;
    
    /**
     * Zwerginnen, die ihre Sippe verlassen muessen MU mindestens 12 besitzen.
     */
    private final JCheckBox zwergin;

    PanelMetaRasseDiverses( final PanelMetaRasse dasPanelRasse ) {

        uebersetzer = new Uebersetzer();
    
        
        
        alterBasis = Util.erzeugeSpinner( 16, 0, 999, 1 );

        alterWuerfel = uebersetzer.erzeugeComboBox( Wuerfel.getWuerfel() );

        gewichtAbzug = Util.erzeugeSpinner( 100, 0, 999, 1 );

        groesseBasis = Util.erzeugeSpinner( 160, 0, 999, 1 );

        groesseAnzahlW20 = Util.erzeugeSpinner( 2, 0, 99, 1 );

        groesseAnzahlW6 = Util.erzeugeSpinner( 0, 0, 99, 1 );

        id = new JTextField();

        kosten = Util.erzeugeSpinner( 0, 0, 999, 1 );

        name = new JTextField();

        panelRasse = dasPanelRasse;

        w6Label = new JLabel( Wuerfel.W6.toString() );

        w20Label = new JLabel( Wuerfel.W20.toString() );

        zwergin = uebersetzer.erzeugeCheckBox( ZWERGIN );

        aufbauenBenutzeroberflaeche();
    }

    /**
     * Uebernimmt die Werte einer Rasse in die Benutzeroberflaeche.
     *
     * @param rasse
     *            deren Werte angezeigt werden sollen.
     */
    public final void anzeigen( final Rasse rasse ) {

        // Name
        // ****
        name.setText( rasse.getName() );

        // Id
        // **
        final String idString = rasse.getId();

        final int index = idString.indexOf( '.' );

        final String suffix = idString.substring( 1 + index );

        id.setText( suffix );

        // Kosten
        // ******
        kosten.setValue( ( (KostenFest)rasse.getKostenObjekt() ).getGpKosten());

        // Alter
        // *****
        final AlterFormel alter = rasse.getAlter();

        alterBasis.setValue( alter.getBasis() );

        alterWuerfel.setSelectedItem( alter.getWuerfel() );

        // Groesse
        // *******
        final GroesseFormel groesse = rasse.getGroesse();

        groesseBasis.setValue( groesse.getBasis() );

        groesseAnzahlW20.setValue( groesse.getAnzahlW20() );

        groesseAnzahlW6.setValue( groesse.getAnzahlW6() );

        // Gewicht
        // *******
        final GewichtFormel gewicht = rasse.getGewicht();

        gewichtAbzug.setValue( gewicht.getAbzug() );

        // Voraussetzung Zwerginnen
        // ************************
        final List< Voraussetzung > voraussetzungen = rasse.getVoraussetzungen();

        zwergin.setSelected( voraussetzungen.size() > 0 );
    }

    private void aufbauenBenutzeroberflaeche() {

        final JPanel inhalt = new JPanel();
        
        final GroupLayout layout = new GroupLayout( inhalt );

        inhalt.setLayout( layout );

        layout.setAutoCreateContainerGaps( true );

        final GroupLayout.ParallelGroup hGroup = layout.createParallelGroup();

        layout.setHorizontalGroup(hGroup);

        final GroupLayout.SequentialGroup vGroup = layout.createSequentialGroup();

        layout.setVerticalGroup( vGroup );

        // Zeile 1
        // *******
        final JLabel lName = uebersetzer.erzeugeLabel( NAME );

        hGroup.addGroup( layout.createSequentialGroup()
                .addComponent( lName )
                .addGap( ABSTAND_HORIZONTAL )
                .addComponent( name ) );

        vGroup.addGroup( layout.createParallelGroup(
                GroupLayout.Alignment.CENTER,
                false )
                .addComponent( lName )
                .addComponent( name ) );

        // Zeilenabstand
        // *************
        einfuegenZeilenAbstand( hGroup, vGroup );

        // Zeile 1.1
        // **********
        final JLabel lId = uebersetzer.erzeugeLabel( ID );

        hGroup.addGroup( layout.createSequentialGroup()
                .addComponent( lId )
                .addGap( ABSTAND_HORIZONTAL )
                .addComponent( id ) );

        vGroup.addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE, false )
                .addComponent( lId )
                .addComponent( id ) );

        // Zeilenabstand
        // *************
        einfuegenZeilenAbstand( hGroup, vGroup );

        // Zeile 2
        // *******
        final JLabel lKosten = uebersetzer.erzeugeLabel( KOSTEN );

        final JLabel kostenLabel = new JLabel();

        final JButton berechnen = uebersetzer.erzeugeButton( BERECHNEN );
        berechnen.addActionListener( new ActionListener(){
            public void actionPerformed( final ActionEvent e ) {

                final Rasse rasse = new Rasse( "Rasse" );

                panelRasse.uebertrageDaten( rasse );

                final double kosten = rasse.berechneGpKosten();

                kostenLabel.setText( String.format( "%1$.1f", kosten ) );
            }
        });

        hGroup.addGroup( layout.createSequentialGroup()
                .addComponent( lKosten )
                .addGap( ABSTAND_HORIZONTAL )
                .addComponent( kosten,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE )
                .addGap( ABSTAND_HORIZONTAL )
                .addComponent( berechnen )
                .addGap( ABSTAND_HORIZONTAL )
                .addComponent( kostenLabel ) );

        vGroup.addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE, false )
                .addComponent( lKosten )
                .addComponent( kosten )
                .addComponent( berechnen )
                .addComponent( kostenLabel ) );

        // Zeilenabstand
        // *************
        einfuegenZeilenAbstand( hGroup, vGroup );

        // Zeile 3
        // *******
        final JLabel lAlter = uebersetzer.erzeugeLabel( ALTER );

        JLabel plus1 = new JLabel( "+" );

        hGroup.addGroup( layout.createSequentialGroup()
                .addComponent( lAlter )
                .addGap( ABSTAND_HORIZONTAL )
                .addComponent( alterBasis,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE )
                .addGap( ABSTAND_HORIZONTAL )
                .addComponent( plus1 )
                .addGap( ABSTAND_HORIZONTAL )
                .addComponent( alterWuerfel,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE  ) );

        vGroup.addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE, false )
                .addComponent( lAlter )
                .addComponent( alterBasis )
                .addComponent( plus1 )
                .addComponent( alterWuerfel ) );

        // Zeilenabstand
        // *************
        einfuegenZeilenAbstand( hGroup, vGroup );

        // Zeile 4
        // *******
        final JLabel lGroesse = uebersetzer.erzeugeLabel( GROESSE );

        plus1 = new JLabel( "+" );

        final JLabel plus2 = new JLabel( "+" );

        hGroup.addGroup( layout.createSequentialGroup()
                .addComponent( lGroesse )
                .addGap( ABSTAND_HORIZONTAL )
                .addComponent( groesseBasis,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE )
                .addGap( ABSTAND_HORIZONTAL )
                .addComponent( plus1 )
                .addGap( ABSTAND_HORIZONTAL )
                .addComponent( groesseAnzahlW20,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE  )
                .addGap( ABSTAND_HORIZONTAL / 2 )
                .addComponent( w20Label )
                .addGap( ABSTAND_HORIZONTAL )
                .addComponent( plus2 )
                .addGap( ABSTAND_HORIZONTAL )
                .addComponent( groesseAnzahlW6,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE  )
                .addGap( ABSTAND_HORIZONTAL / 2 )
                .addComponent( w6Label ) );

        vGroup.addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE, false )
                .addComponent( lGroesse )
                .addComponent( groesseBasis )
                .addComponent( plus1 )
                .addComponent( groesseAnzahlW20 )
                .addComponent( w20Label )
                .addComponent( plus2 )
                .addComponent( groesseAnzahlW6 )
                .addComponent( w6Label ) );

        // Zeilenabstand
        // *************
        einfuegenZeilenAbstand( hGroup, vGroup );

        // Zeile 5
        // *******
        final JLabel lGewicht = uebersetzer.erzeugeLabel( GEWICHT );

        final JLabel lGroesse2 = uebersetzer.erzeugeLabel( GROESSE );
        
        final JLabel lMinus = new JLabel( "-" );

        hGroup.addGroup( layout.createSequentialGroup()
                .addComponent( lGewicht )
                .addGap( ABSTAND_HORIZONTAL )
                .addComponent( lGroesse2 )
                .addGap( ABSTAND_HORIZONTAL / 2 )
                .addComponent( lMinus )
                .addGap( ABSTAND_HORIZONTAL )
                .addComponent( gewichtAbzug,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE ) );

        vGroup.addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE, false )
                .addComponent( lGewicht )
                .addComponent( lGroesse2 )
                .addComponent( lMinus )
                .addComponent( gewichtAbzug ) );

        // Zeilenabstand
        // *************
        einfuegenZeilenAbstand( hGroup, vGroup );

        // Zeile 6
        // *******
        hGroup.addGroup( layout.createSequentialGroup()
                .addComponent( zwergin ) );

        vGroup.addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE, false )
                .addComponent( zwergin ) );

        layout.linkSize( lName,
                lId,
                lKosten,
                lAlter,
                lGroesse,
                lGewicht );
        
        final JScrollPane scrollPane = Util.erzeugeScrollPane( inhalt );
        resetter.einfuegen( scrollPane );
        
        setLayout( new BorderLayout() );
        
        add( scrollPane, BorderLayout.CENTER );
    }

    private void einfuegenZeilenAbstand(
            final GroupLayout.ParallelGroup hGroup,
            final GroupLayout.SequentialGroup vGroup ) {

        final Component zeilenabstand =
            Box.createVerticalStrut( ABSTAND_VERTIKAL );

        hGroup.addComponent( zeilenabstand );

        vGroup.addComponent( zeilenabstand );
    }

    /** @return Die fuer die Rasse gewaehlte Id. */
    public String getRassenId() {
        return "Rasse." + id.getText().trim();
    }

    /**
     * @return den Namen, den der Benutzer fuer die Rasse angegeben hat.
     */
    public String getRassenName() {
        return name.getText();
    }
    
    public void neuUebersetzen() {

        uebersetzer.neuUebersetzen();

        w6Label.setText( Wuerfel.W6.toString() );

        w20Label.setText( Wuerfel.W20.toString() );
    }

    /** Loescht die Eintraege im Panel. */
    public void eingabenLoeschen() {
        id.setText( "" );

        kosten.setValue( 0 );

        name.setText( "" );

        zwergin.setSelected( false );
    }
    
    /**
     * Stellt die Werte einer Rasse, so ein wie sie in der Benutzeroberflaeche
     * gewaehlt sind.
     *
     * @param rasse
     *            die die Werte erhalten soll.
     */
    @SuppressWarnings( "unchecked" )
    public final void uebertrageDaten( final Rasse rasse ) {

        rasse.setName( name.getText() );

        rasse.setKosten( new KostenFest( (Integer) kosten.getValue() ) );

        final AlterFormel alter =
            new AlterFormel( (Integer) alterBasis.getValue(),
                             (Wuerfel )alterWuerfel.getSelectedItem() );

        rasse.setAlter( alter );

        final GroesseFormel groesse =
            new GroesseFormel( (Integer) groesseBasis.getValue(),
                               (Integer) groesseAnzahlW20.getValue(),
                               (Integer) groesseAnzahlW6.getValue() );

        rasse.setGroesse( groesse );

        rasse.setGewicht(
                new GewichtFormel( (Integer) gewichtAbzug.getValue() ) );

        if ( zwergin.isSelected() ) {

            rasse.setVoraussetzungen( Arrays.asList(
                    (Voraussetzung) new VoraussetzungZwergin() ) );

        } else {

            rasse.setVoraussetzungen( Collections.EMPTY_LIST );
        }
    }
}
