/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.jgoodies.forms.factories.ButtonBarFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.ElementMitName;
import de.bernhardjung.dsaprogramm.basis.Ruestung;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.daten.Datenspeicherer;
import de.bernhardjung.dsaprogramm.gui.CharGenListModel;
import de.bernhardjung.dsaprogramm.gui.WechselbaresPanel;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Einstellungen;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

public class PanelMetaRuestungen 
extends WechselbaresPanel
implements Uebersetzbar {

    private static final String BE =
        "PanelMetaRuestungen.Behinderung";

    private static final String ENTFERNEN =
        "PanelMetaRuestungen.Entfernen";

    private static final String HINWEIS_DATENQUELLE =
        "PanelMetaRuestungen.HinweisDatenquelle";

    private static final String ID =
        "PanelMetaRuestungen.Id";

    private static final String MINDEST_TORSO_RS =
        "PanelMetaRuestungen.MindestTorsoRs";

    private static final String NAME =
        "PanelMetaRuestungen.Name";

    private static final String RS =
        "PanelMetaRuestungen.Ruestungsschutz";

    private static final String UEBERNEHMEN =
        "PanelMetaRuestungen.Uebernehmen";

    private static final String ZUSATZRUESTUNG =
        "PanelMetaRuestungen.Zusatzruestung";

    /** Die Behinderung. */
    private final JSpinner be;

    /**
     * Gibt an ob der Benutzer schon darauf hingewiesen wurde eine Datenquelle
     * zu waehlen.
     */
    private boolean hinweisGezeigt;

    /** Die Id. */
    private final JTextField id;

    /** Der mindest RS auf den Torso fuer eine Zusatzruestung. */
    private final JSpinner minTorsoRs;

    private final JLabel minTorsoRsLabel;

    /** Der Name. */
    private final JTextField name;

    /** Der Ruestungsschutz. */
    private final JSpinner rs;

    /** Die Liste mit den Ruestungen auf der linken Seite. */
    private final JList ruestungen;
    
    /** Das Model der Liste mit den Ruestungen. */
    private final CharGenListModel< ElementMitName, Ruestung > ruestungListModel;

    /** Da sind die Ruestungen gespeichert. */
    private final Datenspeicher speicher;

    /**
     * Der Knopf mit dem die eingegebene Ruestung in die Liste der Ruestungen
     * uebernommen wird.
     */
    private final JButton uebernehmen;

    /** Uebersetzt die Texte der Benutzeroberflaeche. */
    private final Uebersetzer uebersetzer;
    
    /** Gib an ob es such um eine Zusatzruestung handelt.*/
    private final JCheckBox zusatzruestung;

    public PanelMetaRuestungen() {

        uebersetzer = new Uebersetzer();
        
        be = Util.erzeugeSpinner( 0, 0, 99, 1 );

        hinweisGezeigt = false;

        uebernehmen = uebersetzer.erzeugeButton( UEBERNEHMEN );
        uebernehmen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                uebernehmen();
            }
        });

        id = new JTextField();
        id.setBackground( Util.FEHLERFARBE );
        id.getDocument().addDocumentListener( new DocumentListener() {
            public void changedUpdate( final DocumentEvent e ) {}

            public void insertUpdate( final DocumentEvent e ) {
                pruefen();
            }
            public void removeUpdate( final DocumentEvent e ) {
                pruefen();
            }
        });

        minTorsoRs = Util.erzeugeSpinner( 0, 0, 99, 1 );
        minTorsoRs.setEnabled( false );

        minTorsoRsLabel = uebersetzer.erzeugeLabel( MINDEST_TORSO_RS );
        minTorsoRsLabel.setEnabled( false );

        name = new JTextField();

        rs = Util.erzeugeSpinner( 0, 0, 99, 1 );

        speicher = Datenspeicher.getInstanz();

        ruestungListModel = new CharGenListModel< ElementMitName, Ruestung >(
                speicher.getRuestungen(),
                Ruestung.NAME_COMPARATOR );
        
        ruestungen = uebersetzer.erzeugeListe( ruestungListModel );
        ruestungen.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        ruestungen.addListSelectionListener( new ListSelectionListener(){
            public void valueChanged( final ListSelectionEvent e ) {

                if ( e.getValueIsAdjusting() ) {
                    return;
                }

                anzeigen();
            }
        });
        
        zusatzruestung = uebersetzer.erzeugeCheckBox( ZUSATZRUESTUNG );
        zusatzruestung.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                if ( zusatzruestung.isSelected() ) {
                    minTorsoRsLabel.setEnabled( true );
                    minTorsoRs.setEnabled( true );
                } else {
                    minTorsoRsLabel.setEnabled( false );
                    minTorsoRs.setEnabled( false );
                    minTorsoRs.setValue( 0 );
                }
            }
        });

        aufbauenBenutzeroberflaeche();
    }

    /**
     * Zeigt die Werte der in der Liste gewaehlte Ruestung an.
     */
    private final void anzeigen() {

        final Ruestung ruestung = getGewaehlteRuestung();

        if ( null != ruestung ) {

            be.setValue( ruestung.getBe() );

            id.setText( ruestung.getId().replace( "Ruestung.", "" ) );

            minTorsoRs.setValue( ruestung.getMindestTorsoRs() );

            name.setText( ruestung.getName() );

            rs.setValue( ruestung.getRs() );

            if ( ruestung.istZusatzruestung() ) {

                minTorsoRsLabel.setEnabled( true );
                minTorsoRs.setEnabled( true );
                zusatzruestung.setSelected( true );

            } else {

                minTorsoRsLabel.setEnabled( false );
                minTorsoRs.setEnabled( false );
                zusatzruestung.setSelected( false );
            }
        }
    }

    private final void aufbauenBenutzeroberflaeche() {

        final JButton entfernen = uebersetzer.erzeugeButton( ENTFERNEN );
        entfernen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                entfernen();
            }
        });

        final JPanel buttonpanel =
            ButtonBarFactory.buildLeftAlignedBar( uebernehmen );

        final String spalten = "7dlu, 200px:G, 4dlu, P, 4dlu, P, P:G, 7dlu";

        final String zeilen =
            "7dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, " +
            "4dlu, P, 4dlu, P, P:G, 4dlu, P, 7dlu";

        final FormLayout layout = new FormLayout( spalten, zeilen );

        setLayout( layout );

        final CellConstraints cc = new CellConstraints();

        add( new JScrollPane( ruestungen ), cc.xywh( 2, 2, 1, 14 ) );

        add( entfernen, cc.xy( 2, 17 ) );

        int y = 0;

        add( uebersetzer.erzeugeLabel( NAME ), cc.xy ( 4, y+=2 ) );
        add( name,                             cc.xyw( 6, y, 2 ) );

        add( uebersetzer.erzeugeLabel( ID ), cc.xy ( 4, y+=2 ) );
        add( id,                             cc.xyw( 6, y, 2 ) );

        add( uebersetzer.erzeugeLabel( RS ), cc.xy( 4, y+=2 ) );
        add( rs,                             cc.xy( 6, y ) );

        add( uebersetzer.erzeugeLabel( BE ), cc.xy( 4, y+=2 ) );
        add( be,                             cc.xy( 6, y ) );

        add( zusatzruestung, cc.xyw( 4, y+=2, 4 ) );

        add( minTorsoRsLabel, cc.xy( 4, y+=2 ) );
        add( minTorsoRs,      cc.xy( 6, y ) );

        add( buttonpanel, cc.xyw( 4, y+=2, 4 ) );
    }

    /**
     * Enternt die in der Liste {@link #ruestungen} gewaehlte Ruestung aus dem
     * Datenspeicher.
     */
    private void entfernen() {

        final Ruestung ruestung = getGewaehlteRuestung();

        if ( null != ruestung ) {

            speicher.entfernen( ruestung );

            speichern();

            ruestungListModel.removeElement( ruestung );
        }
    }

    /**
     * @return Die Ruestung, die in der Liste auf der linken Seite angewaehlt
     *         ist, oder {@code null}, wenn keine Ruestung gewaehlt ist. 
     */
    private Ruestung getGewaehlteRuestung() {
        return (Ruestung) ruestungen.getSelectedValue();
    }

    public boolean kannGewechseltWerden() {
        
        return true;
    }

    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
    }

    /**
     * Prueft die Eingabe im Panel auf Korrektheit und setzt den
     * Uebernehmen-Knopf auf aktiv oder deaktiv.
     *
     * @return  {@code true}, wenn alles Ok ist, sonst {@code false}.
     */
    private boolean pruefen() {

        if ( id.getText().trim().equals( "" ) ) {

            id.setBackground( Util.FEHLERFARBE );

            uebernehmen.setEnabled( false );

            return false;

        } else {

            id.setBackground( name.getBackground() );

            uebernehmen.setEnabled( true );

            return true;
        }
    }

    /**
     * Speichert die Liste der Ruestungen auf die Festplatte.
     */
    private final void speichern() {

        final Einstellungen einstellungen = Einstellungen.getInstanz();

        if ( einstellungen.istDatenquelleVerzeichnis() ) {

            Datenspeicherer.speichereRuestungen();

        } else if ( ! hinweisGezeigt ){

            JOptionPane.showMessageDialog( this, I18n.i18n( HINWEIS_DATENQUELLE ) );

            hinweisGezeigt = true;
        }

    }

    /**
     * Uebernimmt die im Panel eingegebenen Ruestung in die Liste der
     * Ruestungen.
     */
    private final void uebernehmen() {

        String idString = id.getText().trim();

        idString = "Ruestung." + idString;

        final Ruestung ruestung = speicher.getRuestung( idString );

        ruestung.setBe( (Integer) be.getValue() );

        ruestung.setMindestTorsoRs( (Integer) minTorsoRs.getValue() );

        ruestung.setName( name.getText().trim() );

        ruestung.setRs( (Integer) rs.getValue() );

        ruestung.setZusatzruestung( zusatzruestung.isSelected() );

        speicher.einfuegen( ruestung );

        speichern();

        ruestungListModel.setElements( speicher.getRuestungen() );
    }
}
