/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.jgoodies.forms.factories.ButtonBarFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.ElementMitName;
import de.bernhardjung.dsaprogramm.basis.Schild;
import de.bernhardjung.dsaprogramm.basis.SchildTyp;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.daten.Datenspeicherer;
import de.bernhardjung.dsaprogramm.gui.CharGenListModel;
import de.bernhardjung.dsaprogramm.gui.WechselbaresPanel;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Einstellungen;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

public class PanelMetaSchilde 
extends WechselbaresPanel 
implements Uebersetzbar {

    private static final String BF =
        "PanelMetaSchilde.Bf";

    private static final String ENTFERNEN =
        "PanelMetaSchilde.Entfernen";

    private static final String HINWEIS_DATENQUELLE =
        "PanelMetaSchilde.HinweisDatenquelle";

    private static final String ID =
        "PanelMetaSchilde.Id";

    private static final String INI =
        "PanelMetaSchilde.Ini";

    private static final String NAME =
        "PanelMetaSchilde.Name";

    private static final String TYP =
        "PanelMetaSchilde.Typ";

    private static final String UEBERNEHMEN =
        "PanelMetaSchilde.Uebernehmen";

    private static final String WM =
        "PanelMetaSchilde.Wm";

    /** Der Bruchfaktor. */
    private final JSpinner bf;

    /**
     * Gibt an ob der Benutzer schon darauf hingewiesen wurde eine Datenquelle
     * zu waehlen.
     */
    private boolean hinweisGezeigt;

    /** Die Id. */
    private final JTextField id;

    /** Initiativemodifikator. */
    private final JSpinner ini;

    /** Der Name. */
    private final JTextField name;

    /** Die Liste mit den Schilden. */
    private final JList schilde;
    
    /** Die Liste der Schilde. */
    private final CharGenListModel< ElementMitName, Schild > schildListModel;

    /** Da werden die Schilde gespeichert. */
    private final Datenspeicher speicher;

    /** 
     * Der Typ des Schildes. (Deckung im Fernkampf). klein, gross oder sehr
     * gross. 
     */
    private final JComboBox typ;

    /**
     * Der Knopf mit dem der eingegebene Schild in die Liste der Schilde
     * uebernommen wird.
     */
    private final JButton uebernehmen;

    /**
     * Uebersetzt die Texte der Benutzeroberflaeche.
     */
    private final Uebersetzer uebersetzer;
    
    /** Hier kann der Waffenmodifikator eingegeben werden. */
    private final JTextField wm;

    /** Zum ueberpruefen der Eingabe ins Textfeld {@link #wm}. */
    private final Pattern wmPattern;

    public PanelMetaSchilde() {

        uebersetzer = new Uebersetzer();
        
        bf = Util.erzeugeSpinner( 0, -9, 99, 1 );

        hinweisGezeigt = false;

        uebernehmen = uebersetzer.erzeugeButton( UEBERNEHMEN );
        uebernehmen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                uebernehmen();
            }
        });

        id = new JTextField();
        id.setBackground( Util.FEHLERFARBE );
        id.getDocument().addDocumentListener( new DocumentListener() {
            public void changedUpdate( final DocumentEvent e ) {}

            public void insertUpdate( final DocumentEvent e ) {
                pruefen();
            }
            public void removeUpdate( final DocumentEvent e ) {
                pruefen();
            }
        });

        ini = Util.erzeugeSpinner( 0, -9, 9, 1 );

        name = new JTextField();

        speicher = Datenspeicher.getInstanz();

        schildListModel = new CharGenListModel< ElementMitName, Schild >(
                speicher.getSchilde(),
                Schild.NAME_COMPARATOR );

        schilde = uebersetzer.erzeugeListe( schildListModel );
        schilde.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        schilde.addListSelectionListener( new ListSelectionListener(){
            public void valueChanged( final ListSelectionEvent e ) {

                if ( e.getValueIsAdjusting() ) {
                    return;
                }

                anzeigen();
            }
        });

        typ = uebersetzer.erzeugeComboBox( SchildTyp.getTypen() );
        
        wm = new JTextField();
        wm.setBackground( Util.FEHLERFARBE );
        wm.getDocument().addDocumentListener( new DocumentListener() {
            public void changedUpdate( final DocumentEvent e ) {}

            public void insertUpdate( final DocumentEvent e ) {
                pruefen();
            }
            public void removeUpdate( final DocumentEvent e ) {
                pruefen();
            }
        });

        wmPattern = Pattern.compile( "[+-]?\\d+/[+-]?\\d+" );

        aufbauenBenutzeroberflaeche();
    }

    /**
     * Zeigt die Werte der in der Liste gewaehlte Ruestung an.
     */
    private final void anzeigen() {

        final Schild schild = getGewaehltenSchild();

        if ( null != schild ) {

            bf.setValue( schild.getBf() );

            id.setText( schild.getId().replace( "Schild.", "" ) );

            ini.setValue( schild.getIni() );

            name.setText( schild.getName() );

            typ.setSelectedItem( schild.getTyp() );

            wm.setText( String.format(
                    "%1$s/%2$s",
                    schild.getWmAt(),
                    schild.getWmPa() ) );
        }
    }

    private final void aufbauenBenutzeroberflaeche() {

        final JButton entfernen = uebersetzer.erzeugeButton( ENTFERNEN );
        entfernen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                entfernen();
            }
        });

        final JPanel buttonpanel =
            ButtonBarFactory.buildLeftAlignedBar( uebernehmen );

        final String spalten = "7dlu, 200px:G(0.2), 4dlu, P, 4dlu, P, P:G(0.8), 7dlu";

        final String zeilen =
            "7dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, " +
            "4dlu, P, 4dlu, P, P:G, 4dlu, P, 7dlu";

        final FormLayout layout = new FormLayout( spalten, zeilen );

        setLayout( layout );

        final CellConstraints cc = new CellConstraints();

        add( new JScrollPane( schilde ), cc.xywh( 2, 2, 1, 14 ) );

        add( entfernen, cc.xy( 2, 17 ) );

        int y = 0;

        add( uebersetzer.erzeugeLabel( NAME ), cc.xy ( 4, y+=2 ) );
        add( name,                             cc.xyw( 6, y, 2 ) );

        add( uebersetzer.erzeugeLabel( ID ), cc.xy ( 4, y+=2 ) );
        add( id,                             cc.xyw( 6, y, 2 ) );

        add( uebersetzer.erzeugeLabel( WM ), cc.xy ( 4, y+=2 ) );
        add( wm,                             cc.xyw( 6, y, 2 ) );

        add( uebersetzer.erzeugeLabel( INI ), cc.xy( 4, y+=2 ) );
        add( ini,                             cc.xy( 6, y ) );

        add( uebersetzer.erzeugeLabel( BF ), cc.xy( 4, y+=2 ) );
        add( bf,                             cc.xy( 6, y ) );

        add( uebersetzer.erzeugeLabel( TYP ), cc.xy( 4, y+=2 ) );
        add( typ,                             cc.xy( 6, y ) );

        add( buttonpanel, cc.xyw( 4, y+=2, 4 ) );
    }

    /**
     * Enternt die in der Liste {@link #ruestungen} gewaehlte Ruestung aus dem
     * Datenspeicher.
     */
    private void entfernen() {

        final Schild schild = getGewaehltenSchild();

        if ( null != schild ) {

            speicher.entfernen( schild );

            schildListModel.removeElement( schild );

            speichern();
        }
    }

    /**
     * @return Den Schild, der in der Liste auf der linken Seite gewaehlt ist,
     *         oder {@code null}, wenn kein Schild gewaehlt ist.
     */
    private Schild getGewaehltenSchild() {
        return (Schild) schilde.getSelectedValue();
    }

    public boolean kannGewechseltWerden() {
        
        return true;
    }

    /**
     * Uebersetzt die Texte der Benutzeroberflaeche neu.
     */
    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
    }
    
    /**
     * Prueft die Eingabe im Panel auf Korrektheit und setzt den
     * Uebernehmen-Knopf auf aktiv oder deaktiv.
     *
     * @return  {@code true}, wenn alles Ok ist, sonst {@code false}.
     */
    private boolean pruefen() {

        boolean ergebnis = true;

        if ( id.getText().trim().equals( "" ) ) {

            id.setBackground( Util.FEHLERFARBE );

            ergebnis = false;

        } else {

            id.setBackground( name.getBackground() );
        }

        final Matcher matcher = wmPattern.matcher( wm.getText().trim() );

        if ( matcher.matches() ) {

            wm.setBackground( name.getBackground() );

        } else {

            wm.setBackground( Util.FEHLERFARBE );

            ergebnis = false;
        }

        uebernehmen.setEnabled( ergebnis );

        return ergebnis;
    }

    /**
     * Speichert die Liste der Schilde auf die Festplatte.
     */
    private final void speichern() {

        final Einstellungen einstellungen = Einstellungen.getInstanz();

        if ( einstellungen.istDatenquelleVerzeichnis() ) {

            Datenspeicherer.speichereSchilde();

        } else if ( ! hinweisGezeigt ){

            JOptionPane.showMessageDialog( this, I18n.i18n( HINWEIS_DATENQUELLE ) );

            hinweisGezeigt = true;
        }

    }

    /**
     * Wandelt einen String, der eine Zahl mit Vorzeichen enthaelt in einen Int
     * Wert um.
     *
     * @param s
     *            die Zahl als String.
     */
    private final int toInt( String s ) {

        if ( s.charAt( 0 ) == '+' ) {

            s = s.substring( 1 );
        }

        return Integer.valueOf( s );
    }

    /**
     * Uebernimmt die im Panel eingegebenen Ruestung in die Liste der
     * Ruestungen.
     */
    private final void uebernehmen() {

        String idString = id.getText().trim();

        idString = "Schild." + idString;

        final Schild schild = speicher.getSchild( idString );

        schild.setBf( (Integer) bf.getValue() );

        schild.setIni( (Integer) ini.getValue() );

        schild.setName( name.getText().trim() );

        schild.setTyp( (SchildTyp) typ.getSelectedItem() );

        final String[] wmArray = wm.getText().trim().split( "/" );

        schild.setWmAt( toInt( wmArray[ 0 ] ) );

        schild.setWmPa( toInt( wmArray[ 1 ] ) );

        speicher.einfuegen( schild );

        speichern();

        schildListModel.setElements( speicher.getSchilde() );
    }
}
