/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JCheckBox;
import javax.swing.JTabbedPane;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.ElementMitName;
import de.bernhardjung.dsaprogramm.bausteine.Heldenbaustein;
import de.bernhardjung.dsaprogramm.bausteine.bonus.VerbilligteSonderfertigkeit;
import de.bernhardjung.dsaprogramm.bausteine.bonus.VerbilligteSonderfertigkeitGelaendekunde;
import de.bernhardjung.dsaprogramm.bausteine.bonus.VerbilligteSonderfertigkeitStandard;
import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeitgruppe;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;

class PanelMetaVerbilligteSonderfertigkeiten
extends ResetbaresPanel 
implements Uebersetzbar {

    /**
     * Erlaubt es den Bonus "Eine Gelaendekunde nach Wahl" zu waehlen.
     */
    private final JCheckBox gelaendekunde;

    private final List< PanelMetaListe< ElementMitName, Sonderfertigkeit > > panel;

    
    
    /** Uebersetzt die Texte des Panels. */
    private final Uebersetzer uebersetzer;
    
    PanelMetaVerbilligteSonderfertigkeiten() {

        uebersetzer = new Uebersetzer();
    
        
        
        gelaendekunde = uebersetzer.erzeugeCheckBox( 
                VerbilligteSonderfertigkeitGelaendekunde.BESCHREIBUNG );

        panel = new ArrayList< PanelMetaListe< ElementMitName, Sonderfertigkeit > >();

        final JTabbedPane tabbedPane = new JTabbedPane( JTabbedPane.LEFT );
        resetter.einfuegen( tabbedPane );

        for ( final Sonderfertigkeitgruppe gruppe:
                        Sonderfertigkeitgruppe.getGruppen() ) {

            final PanelMetaListe< ElementMitName, Sonderfertigkeit > p =
                new PanelMetaListe< ElementMitName, Sonderfertigkeit >( 
                        gruppe,
                        Sonderfertigkeit.NAME_COMPARATOR );
            resetter.einfuegen( p );
            
            panel.add( p );

            uebersetzer.erzeugeTab( 
                    tabbedPane,
                    gruppe.getLokalisierungsschluesselKuerzel(),
                    p );
        }

        final DefaultFormBuilder builder =
            new DefaultFormBuilder( new FormLayout( "P" ) );

        builder.setDefaultDialogBorder();

        builder.append( gelaendekunde );

        setLayout( new BorderLayout() );

        add( builder.getPanel(), BorderLayout.PAGE_START );

        add( tabbedPane, BorderLayout.CENTER );
    }

    public final void anzeigen( final Heldenbaustein baustein ) {

        gelaendekunde.setSelected( false );

        final List< VerbilligteSonderfertigkeit > verbilligte =
            baustein.getVerbilligteSonderfertigkeiten();

        final List< Sonderfertigkeit > sonderfertigkeiten =
            new ArrayList< Sonderfertigkeit >( verbilligte.size() );

        for ( final VerbilligteSonderfertigkeit v : verbilligte ) {

            if ( v.hatAuswahl() ) {

                gelaendekunde.setSelected( true );

            } else {

                sonderfertigkeiten.add( v.getSonderfertigkeit() );
            }
        }

        for ( final PanelMetaListe< ElementMitName, Sonderfertigkeit > p : panel ) {

            p.setGewaehlteElemente( sonderfertigkeiten );
        }
    }

    public final
    List< VerbilligteSonderfertigkeit > getVerbilligteSonderfertigkeiten() {

        final List< VerbilligteSonderfertigkeit > verbilligte =
            new ArrayList< VerbilligteSonderfertigkeit >();

        for ( final PanelMetaListe< ElementMitName, Sonderfertigkeit > p : panel ) {

            for ( final Sonderfertigkeit sonderfertigkeit :
                                        p.getGewaehlteElemente() ) {

                verbilligte.add( new VerbilligteSonderfertigkeitStandard(
                        sonderfertigkeit ) );
            }
        }

        if ( gelaendekunde.isSelected() ) {

            verbilligte.add( VerbilligteSonderfertigkeit.GELAENDEKUNDE );
        }

        return verbilligte;
    }

    /** Uebersetzt die Texte auf dem Panel neu. */
    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
    }
    
    /** Waehlt alle Eintraege im Panel ab. */
    public void eingabenLoeschen() {

        for ( final PanelMetaListe< ElementMitName, Sonderfertigkeit > p : panel ) {

            p.eingabenLoeschen();
        }

        gelaendekunde.setSelected( false );
    }
}
