/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.ListSelectionModel;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.bausteine.Profession;
import de.bernhardjung.dsaprogramm.bausteine.voraussetzungen.Voraussetzung;
import de.bernhardjung.dsaprogramm.bausteine.voraussetzungen.VoraussetzungMindeststufe;
import de.bernhardjung.dsaprogramm.bausteine.voraussetzungen.VoraussetzungSo;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.gui.CharGenListModel;
import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

class PanelMetaVoraussetzungen
extends ResetbaresPanel
implements ActionListener, Uebersetzbar {

    private static final String ENTFERNEN =
        "PanelMetaVoraussetzungenEigenschaften.Entfernen";

    private static final String UEBERNEHMEN =
        "PanelMetaVoraussetzungenEigenschaften.Uebernehmen";

    private static final String VORAUSSETZUNG =
        "PanelMetaVoraussetzungenEigenschaften.Voraussetzung";

    private final Eigenschaft[] eigenschaften;

    private final List< JCheckBox > checkBoxen;

    private final JSpinner mindeststufe;

    
    
    /** Der Spinner mit dem die SO Mindeststufe eingestellt wird. */
    private final JSpinner soMindeststufe;

    /** Der Spinner mit dem die SO Hoechststufe eingestellt wird. */
    private final JSpinner soHoechststufe;

    private final JButton uebernehmen;
    
    /** Uebersetzt die Texte auf dem Panel. */
    private final Uebersetzer uebersetzer;

    /** Die Liste mit den gewaehlten Voraussetzungen.*/
    private final 
    CharGenListModel< Voraussetzung, Voraussetzung > voraussetzungenModel;

    PanelMetaVoraussetzungen() {

        uebersetzer = new Uebersetzer();
    
        
        
        eigenschaften = Eigenschaft.getEigenschaften();

        checkBoxen =
            new ArrayList< JCheckBox >( eigenschaften.length );

        for ( final Eigenschaft eigenschaft : eigenschaften ) {

            final JCheckBox checkBox = uebersetzer.erzeugeCheckBox( 
                    eigenschaft.getLokalisierungsschluesselName() );

            checkBox.addActionListener( this );

            checkBoxen.add( checkBox );
        }

        mindeststufe = Util.erzeugeSpinner( 11, 0, 99, 1 );

        soMindeststufe = Util.erzeugeSpinner( 1, 1, 99, 1 );

        soHoechststufe = Util.erzeugeSpinner( 13, 1, 99, 1 );

        uebernehmen = uebersetzer.erzeugeButton( UEBERNEHMEN );
        uebernehmen.setEnabled( false );
        uebernehmen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                uebernehmen();
            }
        });

        voraussetzungenModel = 
            new CharGenListModel< Voraussetzung, Voraussetzung >();
        
        final JList voraussetzungenListe = 
            uebersetzer.erzeugeListe( voraussetzungenModel );
        voraussetzungenListe.setSelectionMode(
                ListSelectionModel.SINGLE_SELECTION );

        final JButton entfernen = uebersetzer.erzeugeButton( ENTFERNEN );
        entfernen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {

                final int index = voraussetzungenListe.getSelectedIndex();

                if ( index >= 0 ) {

                    voraussetzungenModel.removeElementAt( index );
                }
            }
        });

        final FormLayout layout = new FormLayout(
                "7dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 0px:G, 4dlu, P, 7dlu",
                "7dlu, P, 4dlu, F:P:G, 4dlu, P, 7dlu");

        setLayout( layout );

        final CellConstraints cc = new CellConstraints();

        add( uebersetzer.erzeugeLabel( 
                Eigenschaft.SO.getLokalisierungsschluesselName() ),
                cc.xy( 2, 2 ) );
        add( soMindeststufe, cc.xy( 4, 2 ) );
        add( new JLabel( "-" ), cc.xy( 6, 2 ) );
        add( soHoechststufe, cc.xy( 8, 2 ) );

        add( new JScrollPane( voraussetzungenListe ), cc.xyw( 2, 4, 8 ) );

        add( entfernen, cc.xyw( 2, 6, 8 ) );
        
        
        add( erzeugeDatenPanel(), cc.xy( 11, 4 ) );
    }

    /**
     * Wird aufgerufen, wenn eine Checkbox angeklickt wird.
     */
    public void actionPerformed( final ActionEvent e ) {

        uebernehmen.setEnabled( false );

        for ( final JCheckBox checkBox : checkBoxen ) {

            if ( checkBox.isSelected() ) {

                uebernehmen.setEnabled( true );
                
                break;
            }
        }        
    }

    /**
     * Zeigt die Eigenschaftsvoraussetzungen der Profession an.
     *
     * @param profession
     *            deren Voraussetzungen angezeigt werden sollen.
     */
    public final void anzeigen( final Profession profession ) {

        voraussetzungenModel.clear();

        final List< Voraussetzung > voraussetzungen =
            profession.getVoraussetzungen();

        for ( final Voraussetzung voraussetzung : voraussetzungen ) {

            if ( voraussetzung instanceof VoraussetzungMindeststufe ) {

                voraussetzungenModel.addElement( voraussetzung );

            } else if ( voraussetzung instanceof VoraussetzungSo ) {

                anzeigen( (VoraussetzungSo) voraussetzung );
            }
        }
    }

    /**
     * Stellt die Spinner entsprechend der Voraussetzung ein.
     *
     * @param voraussetzung
     *            die angezeigt werden soll.
     */
    private void anzeigen( final VoraussetzungSo voraussetzung ) {

        soMindeststufe.setValue( voraussetzung.getMindeststufe() );

        soHoechststufe.setValue( voraussetzung.getHoechststufe() );
    }


    /**
     * @return die gewaehlten Voraussetzungen.
     */
    public final List< Voraussetzung > getVoraussetzungen() {

        final List< Voraussetzung > voraussetzungen =
            voraussetzungenModel.getElements();

        voraussetzungen.add( new VoraussetzungSo(
                (Integer) soMindeststufe.getValue(),
                (Integer) soHoechststufe.getValue() ) );

        return voraussetzungen;
    }
    
    /** Uebersetzt alle Texte des Panels neu. */
    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
    }

    /** Loescht alle Eintraege im Panel. */
    public void eingabenLoeschen() {

        for ( final JCheckBox checkBox : checkBoxen ) {

            checkBox.setSelected( false );
        }

        uebernehmen.setEnabled( false );

        voraussetzungenModel.clear();
    }
    
    /**
     * @return Eine Liste mit einer Checkbox pro Eigenschaft.
     */
    public JPanel erzeugeDatenPanel() {

        final DefaultFormBuilder builder = new DefaultFormBuilder(
                new FormLayout( "P, 4dlu, P" ) );

        final DefaultFormBuilder builderEigenschaften = new DefaultFormBuilder(
                new FormLayout( "P" ) );
        
        for ( int i = 0, n = eigenschaften.length ; i < n ; ++i ) {

            builderEigenschaften.append( checkBoxen.get( i ) );
        }

        final JScrollPane scrollPane = 
            Util.erzeugeScrollPane( builderEigenschaften.getPanel() );
        resetter.einfuegen( scrollPane );
        
        builder.append( scrollPane, 3 );
        
        builder.append( 
                uebersetzer.erzeugeLabel( VORAUSSETZUNG ), 
                mindeststufe );
        
        builder.append( uebernehmen, 3 );
        
        return builder.getPanel();
    }
    
    /**
     * Uebernimmt die eingestellte Voraussetzung in die Liste der
     * Voraussetzungen.
     */
    public final void uebernehmen() {

        final List< Eigenschaft > liste = new ArrayList< Eigenschaft >();

        for ( int i = 0, n = eigenschaften.length ; i < n ; ++i ) {

            final JCheckBox checkBox = checkBoxen.get( i );

            if ( checkBox.isSelected() ) {

                checkBox.setSelected( false );

                liste.add( eigenschaften[ i ] );
            }
        }

        uebernehmen.setEnabled( false );

        voraussetzungenModel.addElement( new VoraussetzungMindeststufe(
                (Integer) mindeststufe.getValue(),
                liste.toArray( new Eigenschaft[ liste.size() ] ) ) );
    }
}
