/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.steigern;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.eigenschaften.EigenschaftWert;
import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

public class PanelEigenschaftenSteigern
extends ResetbaresPanel 
implements Observer, Uebersetzbar {

    /** Die Ueberschrift fuer die Spalte mit den AP-Kosten. */
    private static final String AP =
        "PanelEigenschaftenSteigern.AP";

    /** Die Ueberschrift fuer die Spalte mit den Eigenschaften. */
    private static final String EIGENSCHAFT =
        "PanelEigenschaftenSteigern.Eigenschaft";

    /** Die Ueberschrift fuer die Spalte mit der Gesamtstufe. */
    private static final String GESAMTSTUFE =
        "PanelEigenschaftenSteigern.Gesamtstufe";

    /** Der Text neben der Combobox fuer die Lernmethode. */
    private static final String LERNMETHODE =
        "PanelEigenschaftenSteigern.Lernmethode";

    /**
     * Die Ueberschrift fuer die Spalte in der eine Spezielle Erfahrung
     * angegeben werden kann.
     */
    private static final String SPEZIELLE_ERFAHRUNG =
        "PanelEigenschaftenSteigern.SpezielleErfahrung";

    /** Die Ueberschrift fuer die Spalte mit der Startstufe. */
    private static final String STARTSTUFE =
        "PanelEigenschaftenSteigern.Startstufe";

    /** Die Ueberschrift fuer die Spalte mit den zugekauften Punkten. */
    private static final String ZUKAUF =
        "PanelEigenschaftenSteigern.Zukauf";

    /** Wird aufgerufen, wenn eine Lernmethode geaendert wird. */
    private final ActionListener actionLernmethode;

    /**
     * Wird aufgerufen wenn eine Checkbox fuer eine Spezielle Erfahrung
     * angeklickt wird.
     */
    private final ActionListener actionSpezielleErfahrung;

    /**
     * Wird aufgerufen, wenn die Anzahl der zugekauften Punkte veraendert wird.
     */
    private final ChangeListener actionZukauf;

    /** Die Label, die die AP-Kosten der Steigerung anzeigen. */
    private final List< JLabel > ap;

    /** Die Eigenschaften die angezeigt werden. */
    private final Eigenschaft[] eigenschaften;

    /** Die Label mit den Namen der Eigenschaften. */
    private final List< JLabel > eigenschaftenLabel;

    /** Die Label, die die Gesamtstufen der Eigenschaften anzeigen. */
    private final List< JLabel > gesamtstufen;

    /** Dient dazu die Gesamtstufe des Sozialstatus einzustellen. */
    private final JSpinner gesamtstufeSo;

    /** Der Held, dessen Werte vom Panel angezeigt und editiert werden.*/
    private Held held;

    /** Die Combobox mit der die Lernmethode ausgewaehlt werden kann. */
    private final JComboBox lernmethode;

    
    
    /**
     * Die Checkboxen mit denen eine Spezielle Erfahrung angegeben werden kann.
     */
    private final List< JCheckBox > spezielleErfahrungen;

    /** Die Label, welche die Startstufen der Eigenschaften anzeigen. */
    private final List< JLabel > startstufen;

    /** Uebersetzt die Texte der Benutzeroberflaeche. */
    private final Uebersetzer uebersetzer;
    
    /** Die Spinner die die zugekauften Punkte anzeigen. */
    private final List< JSpinner > zukauf;

    /**
     * Erzeugt die Benutzerschnittstelle.
     */
    public PanelEigenschaftenSteigern() {

        uebersetzer = new Uebersetzer();
        
        
        
        actionLernmethode = new ActionListener(){
            public void actionPerformed( final ActionEvent e ) {
                update( null, null );
            }
        };

        actionSpezielleErfahrung = new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {

                final JCheckBox checkBox = (JCheckBox) e.getSource();

                final int index = Integer.valueOf( checkBox.getName() );

                final EigenschaftWert wert =
                    held.getWert( eigenschaften[ index ] );

                wert.setSpezielleErfahrung( checkBox.isSelected() );
            }
        };

        actionZukauf = new ChangeListener() {
            public void stateChanged( final ChangeEvent e ) {
                final JSpinner spinner = (JSpinner) e.getSource();

                final int index = Integer.valueOf( spinner.getName() );

                final int zukauf = (Integer) spinner.getValue();

                final Lernmethode l =
                    (Lernmethode) lernmethode.getSelectedItem();

                held.getWert( eigenschaften[ index ]).setZukauf( zukauf, l );
            }
        };

        ap = new ArrayList< JLabel >( 8 );

        eigenschaften = Eigenschaft.getEigenschaften();

        eigenschaftenLabel = new ArrayList< JLabel >( 8 );

        gesamtstufen = new ArrayList< JLabel >( 8 );

        gesamtstufeSo = Util.erzeugeSpinner( 1, 1, 21, 1 );
        gesamtstufeSo.addChangeListener( new ChangeListener(){
            public void stateChanged( final ChangeEvent e ) {
                final int stufe = (Integer) gesamtstufeSo.getValue();
                held.getWert( Eigenschaft.SO ).setStufe( stufe );
            }
        });

        lernmethode = uebersetzer.erzeugeComboBox( 
                Lernmethode.KEINE,
                Lernmethode.GEGENSEITIG,
                Lernmethode.LEHRMEISTER,
                Lernmethode.SPEZIELLE_ERFAHRUNG );
        resetter.einfuegen( lernmethode );
        lernmethode.addActionListener( actionLernmethode );

        spezielleErfahrungen = new ArrayList< JCheckBox >( 8 );

        startstufen = new ArrayList< JLabel >( 8 );

        zukauf = new ArrayList< JSpinner >( 8 );

        aufbauenBenutzeroberflaeche();
    }

    /**
     * Erzeugt alle Elemente der Benutzerschnittstelle und ordnet sie auf dem
     * Panel an.
     */
    private void aufbauenBenutzeroberflaeche() {

        for ( final Eigenschaft eigenschaft : Eigenschaft.getEigenschaften() ) {

            final String index = String.valueOf( eigenschaft.getIndex() );

            ap.add( Util.erzeugeLabelRechtsbuendig() );
            eigenschaftenLabel.add( uebersetzer.erzeugeLabel( 
                    eigenschaft.getLokalisierungsschluesselName() ) );
            gesamtstufen.add( Util.erzeugeLabelRechtsbuendig() );

            final JCheckBox checkBox = uebersetzer.erzeugeCheckBox( 
                    null,
                    Lernmethode.SPEZIELLE_ERFAHRUNG.getL10nschluesselName(),
                    null );
            checkBox.addActionListener( actionSpezielleErfahrung );
            checkBox.setName( index );
            checkBox.setToolTipText( Lernmethode.SPEZIELLE_ERFAHRUNG.getName());
            spezielleErfahrungen.add( checkBox );

            startstufen.add( Util.erzeugeLabelRechtsbuendig() );

            final JSpinner spinner = Util.erzeugeSpinner( 0, 0, 99, 1 );
            spinner.addChangeListener( actionZukauf );
            spinner.setName( index );
            zukauf.add( spinner );
        }

        final FormLayout layout = new FormLayout(
            // Name    Startstufe  +      zukauf     AP       =      gesamt     SE
                "P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P" );

        final DefaultFormBuilder builder = new DefaultFormBuilder( layout );
        builder.setDefaultDialogBorder();

        // erste Zeile
        final DefaultFormBuilder builder2 = new DefaultFormBuilder(
                new FormLayout( "P, 2dlu, P" ) );
        builder2.append( uebersetzer.erzeugeLabel( LERNMETHODE ) );
        builder2.append( lernmethode );
        builder.append( builder2.getPanel(), 15 );

        // zweite Zeile
        builder.append( uebersetzer.erzeugeUeberschrift( EIGENSCHAFT ) );
        builder.append( uebersetzer.erzeugeUeberschrift( STARTSTUFE ) );
        builder.append( "" );
        builder.append( uebersetzer.erzeugeUeberschrift( ZUKAUF ) );
        builder.append( uebersetzer.erzeugeUeberschrift( AP ) );
        builder.append( "" );
        builder.append( uebersetzer.erzeugeUeberschrift( GESAMTSTUFE ) );
        builder.append( uebersetzer.erzeugeUeberschrift( SPEZIELLE_ERFAHRUNG ) );

        // folgende Zeilen
        for ( int i = 0 , n = ap.size() ; i < n ; ++i ) {

            builder.append( eigenschaftenLabel.get( i ) );
            builder.append( startstufen.get( i ) );
            builder.append( "+" );
            builder.append( zukauf.get( i ) );
            builder.append( ap.get( i ) );
            builder.append( "=" );
            builder.append( gesamtstufen.get( i ) );
            builder.append( spezielleErfahrungen.get( i ) );
        }

        // letzte Zeile
        builder.append( uebersetzer.erzeugeLabel( 
                Eigenschaft.SO.getLokalisierungsschluesselName() ) );

        final CellConstraints cc = new CellConstraints();

        builder.add( gesamtstufeSo, cc.xy( 13, builder.getRow() ) );
        
        final JScrollPane scrollPane;
        
        scrollPane = Util.erzeugeScrollPane( builder.getPanel() );
        resetter.einfuegen( scrollPane );
        
        setLayout( new BorderLayout() );
        
        add( scrollPane, BorderLayout.CENTER );
    }
    
    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
    }
    
    /**
     * @param derHeld dessen Werte editiert werden sollen.
     */
    public final void setHeld( final Held derHeld ) {
        
        held = derHeld;

        held.addObserver( this );

        lernmethode.setSelectedItem( Lernmethode.KEINE );

        update( null, null );
    }

    /**
     * Aktualisiert eine Zeile der Benutzerschnittstelle.
     *
     * @param i
     *            Der Index einer Eigenschaft in {@link #eigenschaften}. Die
     *            Zeile dieser Eigenschaft wird aktualisiert.
     */
    private void update( final int i ) {

        final EigenschaftWert wert = held.getWert( eigenschaften[ i ] );

        // Den Namen der Eigenschaft aktualisieren.
        // ****************************************
        final KonfliktBeschreibung beschreibung = wert.pruefen();

        final JLabel label = this.eigenschaftenLabel.get( i );

        if ( beschreibung.enthaeltKonflikt() ) {

            label.setForeground( Util.FEHLERFARBE );
            label.setToolTipText( beschreibung.toHtml() );

        } else {

            label.setForeground( getForeground() );
            label.setToolTipText( null );

        }

        // Die Startstufe aktualisieren.
        // *****************************
        startstufen.get( i ).setText(
                String.valueOf( wert.getStartstufe() ) );

        // Die zugekauften Punkte einstellen.
        // **********************************
        final int z = wert.getZukauf();
        zukauf.get( i ).setValue( z );

        // Die AP-Kosten fuer das Steigern einstellen.
        // *******************************************
        final Lernmethode lern;
        lern = (Lernmethode) lernmethode.getSelectedItem();

        final int apKosten = wert.getSteigerungskosten( lern );

        ap.get( i ).setText( "(" + apKosten + ")" );

        // Die Gesamtstufe
        // ***************
        gesamtstufen.get( i ).setText( String.valueOf( wert.getStufe() ) );

        // Spezielle Erfahrung.
        // ********************
        final boolean se = wert.hatSpezielleErfahrung();

        spezielleErfahrungen.get( i ).setSelected( se );
    }

    /**
     * Wird aufgerufen, wenn eine Aenderung am Helden durchgefuehrt wurde und
     * aktualisiert die gesamte Benutzerschnittstelle.
     *
     * @param o ungenutzt.
     * @param arg ungenutzt.
     */
    public final void update( final Observable o, final Object arg ) {

        for ( int i = 0 ; i < 8 ; ++i ) {

            update( i );
        }

        gesamtstufeSo.setValue( held.getWert( Eigenschaft.SO ).getStufe() );
    }
}
