/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.steigern;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.ElementMitName;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Nahkampfwaffe;
import de.bernhardjung.dsaprogramm.basis.NahkampfwaffeWert;
import de.bernhardjung.dsaprogramm.basis.Tp;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.gui.CharGenListModel;
import de.bernhardjung.dsaprogramm.talente.Kampftechnik;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

class PanelNahkampfwaffe 
extends JPanel 
implements Observer, Uebersetzbar {

    private static final String AT = "PanelNahkampfwaffe.At";

    private static final String BF = "PanelNahkampfwaffe.Bf";

    private static final String DK = "PanelNahkampfwaffe.Dk";

    private static final String INI = "PanelNahkampfwaffe.Ini";

    private static final String NAME = "PanelNahkampfwaffe.Name";

    private static final String PA = "PanelNahkampfwaffe.Pa";

    private static final String TALENT = "PanelNahkampfwaffe.Talent";

    private static final String TP = "PanelNahkampfwaffe.Tp";

    private static final String TP_KK = "PanelNahkampfwaffe.TpKk";

    private static final String WAFFE = "PanelNahkampfwaffe.Waffe";

    private static final String WM = "PanelNahkampfwaffe.Wm";

    /** Zeigt den Attackewert des Helden an. */
    private final JLabel at;

    /** Spinner fuer den Bruchfaktor. */
    private final JSpinner bf;

    /** Distanzklasse */
    private final JTextField dk;

    /** Checkbox mit der die Waffe fuer den Held (ab)gewaehlt werden kann. */
    private final JCheckBox gewaehlt;

    /** Der Held fuer den die Waffe gewaehlt wird. */
    private Held held;

    /** Spinner fuer den Initiativemodifikator. */
    private final JSpinner ini;

    /** Textfeld fuer den Namen der Waffe. */
    private final JTextField name;

    /** Zeigt den Paradewert des Helden an. */
    private final JLabel pa;
    
    /**
     * Aus dieser Combobox kann das Talent, mit dem die Waffe gefuehrt wird,
     * gewaehlt werden.
     */
    private final JComboBox talent;

    /** Das Model fuer die Combobox {@link #talent}. */
    private final CharGenListModel< ElementMitName, Kampftechnik > talentModel;

    /** Textfeld zum Eingeben der Trefferpunkte. */
    private final JTextField tp;

    /**
     * Textfeld zum Eingeben von Schwellenwert und Schadenschritt fuer die
     * Trefferpunkte aus hoher Koerperkraft.
     */
    private final JTextField tpKk;

    /** Zum Ueberpruefen der Eingabe ins Textfeld {@link #tpKk}. */
    private final Pattern tpKkPattern;

    /** Uebersetzt die Texte der Benutzeroberflaeche. */
    private final Uebersetzer uebersetzer;
    
    /** Die Combobox aus der man die Waffe waehlt. */
    private final JComboBox waffe;
    
    /** Das Model fuer die Combobox aus der man die Waffe waehlen kann. */
    private final CharGenListModel< ElementMitName, Nahkampfwaffe > waffeListModel;

    /** Der ActionListener fuer die Combobox {@link #waffe}. */
    private final ActionListener waffeListener;

    /** Die Waffe die fuer den Helden gewaehlt wird. */
    private NahkampfwaffeWert waffeWert;

    /** Textfeld zum Eingeben der Waffenmodifikatoren. */
    private final JTextField wm;

    /** Zum Ueberpruefen der Eingabe ins Textfeld {@link #wm}. */
    private final Pattern wmPattern;
    public PanelNahkampfwaffe() {
        
        uebersetzer = new Uebersetzer();
        
        final Datenspeicher speicher = Datenspeicher.getInstanz();

        speicher.addObserver( this );
        
        at = new JLabel();

        bf = Util.erzeugeSpinner( 0, 0, 12, 1 );
        bf.addChangeListener( new ChangeListener(){
            public void stateChanged( final ChangeEvent e ) {
                waffeWert.setBf( (Integer) bf.getValue() );
            }
        } );

        dk = new JTextField();
        dk.getDocument().addDocumentListener( new DocumentListener() {
            public void changedUpdate( final DocumentEvent e ) {}

            public void insertUpdate( final DocumentEvent e ) {
                pruefeDk();
            }

            public void removeUpdate( final DocumentEvent e ) {
                pruefeDk();
            }

        });

        gewaehlt = new JCheckBox();
        gewaehlt.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                anAbgewaehlt();
            }
        } );

        ini = Util.erzeugeSpinner( 0, -9, 9, 1 );
        ini.addChangeListener( new ChangeListener() {
            public void stateChanged( final ChangeEvent e ) {
                waffeWert.setIni( (Integer) ini.getValue() );
            }
        } );

        name = new JTextField();
        name.getDocument().addDocumentListener( new DocumentListener(){
            public void changedUpdate( final DocumentEvent e ) {}

            public void insertUpdate( final DocumentEvent e ) {
                update();
            }

            public void removeUpdate( final DocumentEvent e ) {
                update();
            }

            private void update() {
                waffeWert.setName( name.getText() );
            }
        } );

        pa = new JLabel();

        talentModel = new CharGenListModel< ElementMitName, Kampftechnik >();

        talent = new JComboBox( talentModel );
        talent.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                
                final Kampftechnik talent = talentModel.getGewaehltesElement();
                
                if ( null == talent ) {
                    
                    return;
                }
                
                waffeWert.setTalent( talent );
                
                if ( null !=  held ) {
                
                    at.setText( String.valueOf( waffeWert.getAttacke( held ) ) );
                    
                    pa.setText( String.valueOf( waffeWert.getParade ( held ) ) );
                }
            }
        } );

        tp = new JTextField();
        tp.getDocument().addDocumentListener( new DocumentListener() {
            public void changedUpdate( final DocumentEvent e ) {}

            public void insertUpdate( final DocumentEvent e ) {
                pruefeTp();
            }

            public void removeUpdate( final DocumentEvent e ) {
                pruefeTp();
            }
        } );

        tpKk = new JTextField();
        tpKk.getDocument().addDocumentListener( new DocumentListener() {
            public void changedUpdate( final DocumentEvent e ) {}

            public void insertUpdate( final DocumentEvent e ) {
                pruefeTpKk();
            }

            public void removeUpdate( final DocumentEvent e ) {
                pruefeTpKk();
            }
        } );

        tpKkPattern = Pattern.compile( "[1-9]\\d*/[1-9]\\d*" );

        waffeListener = new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                typGewaehlt();
            }
        };

        final List< Nahkampfwaffe > waffenListe = speicher.getNahkampfwaffen();
        
        waffeListModel = new CharGenListModel< ElementMitName, Nahkampfwaffe >(
                waffenListe,
                Nahkampfwaffe.NAME_COMPARATOR );
        
        waffe = uebersetzer.erzeugeComboBox( waffeListModel );
        waffe.addActionListener( waffeListener );

        waffeWert = new NahkampfwaffeWert();
        if ( 0 < waffenListe.size() ) {
            
            waffeWert.setWaffe( waffenListe.get( 0 ) );
        }

        wm = new JTextField();
        wm.getDocument().addDocumentListener( new DocumentListener() {
            public void changedUpdate( final DocumentEvent e ) {}

            public void insertUpdate( final DocumentEvent e ) {
                pruefeWm();
            }

            public void removeUpdate( final DocumentEvent e ) {
                pruefeWm();
            }
        } );

        wmPattern = Pattern.compile( "[+-]?\\d+/[+-]?\\d+" );

        aufbauenBenutzeroberflaeche();

        aktualisiereAnzeige();
    }

    /**
     * Wird aufgerufen wenn auf die Checkbox geklickt wird und fuegt dem Held
     * die Waffe hinzu oder entfernt sie wieder von ihm.
     */
    private void anAbgewaehlt() {

        if ( gewaehlt.isSelected() ) {

            held.einfuegen( waffeWert );

        } else {

            held.entfernen( waffeWert );
        }
    }

    /**
     * Uebernimmt die Werte der {@link #waffeWert} in die Benutzeroberflaeche.
     */
    private void aktualisiereAnzeige() {

        if ( null != held ) {

            at.setText( String.valueOf( waffeWert.getAttacke( held ) ) );

            pa.setText( String.valueOf( waffeWert.getParade( held ) ) );

        } else {

            at.setText( "-" );

            pa.setText( "-" );
        }

        bf.setValue( waffeWert.getBf() );

        dk.setText( waffeWert.getDkString() );

        ini.setValue( waffeWert.getIni() );

        name.setText( waffeWert.getName() );

        final Set< Kampftechnik > talente = 
            new TreeSet< Kampftechnik >( Kampftechnik.NAME_COMPARATOR );
        final List< Kampftechnik > haupttalente = waffeWert.getWaffe().getTalente();
        talente.addAll( haupttalente );
        for ( final Kampftechnik technik : haupttalente ) {

            talente.addAll( technik.getVerwandteFertigkeiten() );
        }
        talentModel.setElements( talente, waffeWert.getTalent() );

        tp.setText( waffeWert.getTp().toString() );

        tpKk.setText( String.format( "%1$s/%2$s",
                waffeWert.getSchwellenwert(),
                waffeWert.getSchadensschritt() ) );

        waffe.removeActionListener( waffeListener );
        waffe.setSelectedItem( waffeWert.getWaffe() );
        waffe.addActionListener( waffeListener );

        wm.setText( String.format( "%1$s/%2$s",
                waffeWert.getWmAt(),
                waffeWert.getWmPa() ) );
    }

    private void aufbauenBenutzeroberflaeche() {

        final DefaultFormBuilder zeile1 =
            new DefaultFormBuilder( new FormLayout( "100px:G" ) );

        zeile1.append( name );

        final FormLayout layout2 = new FormLayout(
                "150px:G, 4dlu, P, 2dlu, 200px:G" );

        final DefaultFormBuilder zeile2 =
            new DefaultFormBuilder( layout2 );

        zeile2.append( waffe          );
        zeile2.append( uebersetzer.erzeugeLabel( TALENT ), talent );

        final FormLayout layout3 = new FormLayout(
                "50px, 4dlu, P, 2dlu, 50px, 4dlu, P, 2dlu, 50px, 4dlu, P, 2dlu, " +
                "50px, 4dlu, P, 2dlu, 50px, 4dlu, P, 2dlu, 50px, 4dlu, P, 2dlu, " +
                "P, 4dlu, P, 2dlu, P");

        final DefaultFormBuilder zeile3 = new DefaultFormBuilder( layout3 );

        zeile3.append( tp          );
        zeile3.append( uebersetzer.erzeugeLabel( TP_KK ), tpKk );
        zeile3.append( uebersetzer.erzeugeLabel( INI ),   ini  );
        zeile3.append( uebersetzer.erzeugeLabel( WM ),    wm   );
        zeile3.append( uebersetzer.erzeugeLabel( BF ),    bf   );
        zeile3.append( uebersetzer.erzeugeLabel( DK ),    dk   );
        zeile3.append( uebersetzer.erzeugeLabel( AT ),    at   );
        zeile3.append( uebersetzer.erzeugeLabel( PA ),    pa   );

        setLayout( new FormLayout(
                "P, 4dlu, P, 2dlu, P:G",
                "P, 2dlu, P, 2dlu, P" ) );

        final CellConstraints cc = new CellConstraints();

        add( gewaehlt,           cc.xy( 1, 1 ) );

        add( uebersetzer.erzeugeLabel( NAME ), cc.xy( 3, 1 ) );
        add( zeile1.getPanel(),  cc.xy( 5, 1 ) );

        add( uebersetzer.erzeugeLabel( WAFFE ),  cc.xy( 3, 3 ) );
        add( zeile2.getPanel(),  cc.xy( 5, 3 ) );

        add( uebersetzer.erzeugeLabel( TP ),   cc.xy( 3, 5 ) );
        add( zeile3.getPanel(),  cc.xy( 5, 5 ) );
    }

    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
    }
    
    /**
     * Wird aufgerufen, wenn in das Textfeld fuer die Distanzklasse etwas
     * eingegeben wird. Ueberprueft die Eingabe und uebernimmt sie ggf.
     */
    private void pruefeDk() {

        final String dkString = dk.getText().trim();

        final StringBuilder builder = new StringBuilder();
        builder.append( "(" );
        builder.append( NahkampfwaffeWert.DK_H );
        builder.append( "|" );
        builder.append( NahkampfwaffeWert.DK_N );
        builder.append( "|" );
        builder.append( NahkampfwaffeWert.DK_S );
        builder.append( ")+" );
        final Pattern dkPattern = Pattern.compile( builder.toString() );

        final Matcher matcher = dkPattern.matcher( dkString );

        if ( matcher.matches() ) {

            dk.setBackground( name.getBackground() );

            waffeWert.setDkH( dkString.contains( NahkampfwaffeWert.DK_H ) );

            waffeWert.setDkN( dkString.contains( NahkampfwaffeWert.DK_N ) );

            waffeWert.setDkS( dkString.contains( NahkampfwaffeWert.DK_S ) );

        } else {

            dk.setBackground( Util.FEHLERFARBE );
        }
    }

    /**
     * Wird aufgerufen, wenn in das Textfeld fuer die Trefferpunkte etwas
     * eingegeben wird.
     */
    private void pruefeTp() {

        final String tpString = tp.getText().trim();

        if ( Tp.istTpFormel( tpString ) ) {

            tp.setBackground( name.getBackground() );

            waffeWert.setTp( new Tp( tpString ) );

        } else {

            tp.setBackground( Util.FEHLERFARBE );
        }
    }

    /**
     * Wird aufgerufen wenn in das Texfeld fuer die TP/KK
     * etwas eingegebenn wird.
     */
    private void pruefeTpKk() {

        final String tpKkString = tpKk.getText().trim();

        final Matcher matcher = tpKkPattern.matcher( tpKkString );

        if ( matcher.matches() ) {

            tpKk.setBackground( name.getBackground() );

            final String[] array = tpKkString.split( "/" );

            waffeWert.setSchadensschritt(
                    Integer.valueOf( array[ 1 ] ) );

            waffeWert.setSchwellenwert(
                    Integer.valueOf( array[ 0 ] ) );

        } else {

            tpKk.setBackground( Util.FEHLERFARBE );
        }

    }

    /**
     * Wird aufgerufen wenn in das Texfeld fuer den Waffenmodifikator
     * etwas eingegebenn wird.
     */
    private void pruefeWm() {

        final String wmString = wm.getText().trim();

        final Matcher matcher = wmPattern.matcher( wmString );

        if ( matcher.matches() ) {

            wm.setBackground( name.getBackground() );

            final String[] array = wmString.split( "/" );

            waffeWert.setWmAt( toInt( array[ 0 ] ) );

            waffeWert.setWmPa( toInt( array[ 1 ] ) );

        } else {

            wm.setBackground( Util.FEHLERFARBE );
        }

    }
    
    /**
     * Legt den Held fest, fuer den die Waffe gewaehlt wird.
     *
     * @param derHeld
     *            fuer den die Waffe gewaehlt wird.
     * @param eineWaffe
     *            die der Held schon besitzt. Die Waffe kann dann ueber dieses
     *            Panel editiert werden. Es kann auch {@code null} uebergeben
     *            werden.
     */
    public final void setHeld(
            final Held derHeld,
            final NahkampfwaffeWert eineWaffe ) {

        held = derHeld;

        held.addObserver( this );

        if ( null != eineWaffe ) {

            gewaehlt.setSelected( true );

            waffeWert = eineWaffe;

            aktualisiereAnzeige();

        } else {

            gewaehlt.setSelected( false );

            at.setText( String.valueOf( waffeWert.getAttacke( held ) ) );

            pa.setText( String.valueOf( waffeWert.getParade( held ) ) );
        }
    }

    /**
     * Wandelt einen String, der eine ganze Zahl, mit Vorzeichen, darstellt
     * in einen int-Wert um.
     *
     * @param s
     *            Die Zahl als String.
     * @return die Zahl aus dem String.
     */
    private final int toInt( String s ) {

        if ( s.charAt( 0 ) == '+' ) {

            s = s.substring( 1 );
        }

        return Integer.valueOf( s );
    }

    /**
     * Wird aufgerufen, wenn mit der Combobox {@link #waffe} eine Waffe gewaehlt
     * wird.
     */
    private void typGewaehlt() {

        final Nahkampfwaffe nahkampfwaffe =
            waffeListModel.getGewaehltesElement();

        if ( null != nahkampfwaffe ) {
            
            waffeWert.setWaffe( nahkampfwaffe );
    
            aktualisiereAnzeige();
        }
    }

    public final void update( final Observable o, final Object arg ) {
        
        if ( Datenspeicher.NAHKAMPFWAFFEN.equals( arg ) ) {
            
            final Datenspeicher speicher = Datenspeicher.getInstanz();
            
            waffeListModel.setElements( speicher.getNahkampfwaffen() );
            
        } else if ( o == held ) {
            
            at.setText( String.valueOf( waffeWert.getAttacke( held ) ) );
            
            pa.setText( String.valueOf( waffeWert.getParade( held ) ) );
        }
    }
}
