/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.steigern;

import java.util.ArrayList;
import java.util.List;

import javax.swing.JComboBox;
import javax.swing.JTabbedPane;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.gui.Lernmethoden;
import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

class PanelVorNachteileSteigern 
extends ResetbaresPanel 
implements Uebersetzbar {

    /**
     * Der Titel fuer den Frame der das Panel mit den Vor- und Nachteilen
     * enthaelt die der Held besitzt. 
     */
    private static final String GEWAEHLTE = 
        "PanelVorNachteileSteigern.Gewaehlte";

    /**
     * Der Titel der Registerkarte mit den gewaehlten Vor- und Nachteilen.
     */
    private static final String GEWAEHLTE_REITER = 
        "PanelVorNachteileSteigern.Reiter.Gewaehlte";

    /**
     * Der Text bei der Combobox mit der man die Lernmethode waehlt.
     */
    private static final String LERNMETHODE =
        "PanelVorNachteileSteigern.Lernmethode";

    /**
     * Der Titel des Frames, der das Panel mit den Nachteilen enthaelt.
     */
    private static final String NACHTEILE =
        "PanelVorNachteileSteigern.Nachteile";

    /**
     * Der Titel fuer die Registerkarte die die Nachteile enthaelt.
     */
    private static final String NACHTEILE_REITER = 
        "PanelVorNachteileSteigern.Reiter.Nachteile";

    /**
     * Die Ueberschrift des Panel, das die Vor und Nachteile des Helden 
     * enthaelt.
     */
    private static final String VOR_NACHTEILE =
        "PanelVorNachteileSteigern.VorNachteile";

    /**
     * Der Titel des Frames, der das Panel mit den Vorteilen enthaelt.
     */
    private static final String VORTEILE =
        "PanelVorNachteileSteigern.Vorteile";

    /**
     * Der Titel der Registerkarte die die Vorteil anzeigt.
     */
    private static final String VORTEILE_REITER = 
        "PanelVorNachteileSteigern.Reiter.Vorteile";

    /** Die Liste der Vor-/Nachteile die der Held besitzt. */
    private final PanelVorNachteileListeSteigern gewaehlte;

    /** Die Liste der Nachteile. */
    private final PanelVorNachteileListeSteigern nachteile;

    
    
    /** Uebersetzt die Texte der Benutzeroberflaeche. */
    private final Uebersetzer uebersetzer;
    
    /** Die Liste der Vorteile. */
    private final PanelVorNachteileListeSteigern vorteile;

    PanelVorNachteileSteigern() {

        uebersetzer = new Uebersetzer();
    
        
        
        final JComboBox lernmethode = uebersetzer.erzeugeComboBox(
                Lernmethode.LEHRMEISTER,
                Lernmethode.SPEZIELLE_ERFAHRUNG,
                Lernmethode.SELBSTSTUDIUM );
        resetter.einfuegen( lernmethode );
        
        final Lernmethoden lernmethoden =
            new Lernmethoden( lernmethode );

        gewaehlte = new PanelVorNachteileListeSteigern(
                true,
                VOR_NACHTEILE,
                lernmethoden );
        resetter.einfuegen( gewaehlte );
        
        nachteile = new PanelVorNachteileListeSteigern(
                false,
                NACHTEILE,
                lernmethoden );
        resetter.einfuegen( nachteile );

        List< VorNachteilWert > werte =
            erzeugeWerte( VorNachteil.getNachteile() );

        nachteile.setWerte( werte );

        vorteile = new PanelVorNachteileListeSteigern(
                false,
                VORTEILE,
                lernmethoden );
        resetter.einfuegen( vorteile );

        werte = erzeugeWerte( VorNachteil.getVorteile() );

        vorteile.setWerte( werte );

        final JTabbedPane tabbedPane = new JTabbedPane( JTabbedPane.LEFT );
        resetter.einfuegen( tabbedPane );
        
        uebersetzer.erzeugeTab( 
                tabbedPane,
                GEWAEHLTE_REITER,
                uebersetzer.erzeugeFrame( GEWAEHLTE, gewaehlte ) );

        uebersetzer.erzeugeTab( 
                tabbedPane,
                VORTEILE_REITER,
                uebersetzer.erzeugeFrame( VORTEILE, vorteile ) );

        uebersetzer.erzeugeTab( 
                tabbedPane,
                NACHTEILE_REITER,
                uebersetzer.erzeugeFrame( NACHTEILE, nachteile ) );

        setLayout( new FormLayout( 
                "P:G, R:P, 2dlu, L:P, P:G", 
                "2dlu, P, 2dlu, F:D:G" ) );

        final CellConstraints cc = new CellConstraints();

        add( uebersetzer.erzeugeLabel( LERNMETHODE ), cc.xy( 2, 2 ) );

        add( lernmethode, cc.xy( 4, 2 ) );

        add( tabbedPane, cc.xyw( 1, 4, 5 ) );
    }

    /**
     * Erzeugt zu einer List an Vor-/Nachteilen die entsprechenden Werte.
     *
     * @param vorNachteile
     *            zu denen die Werte erzeugt werden sollen.
     * @return eine Liste von VorNachteilWerten.
     */
    private final List< VorNachteilWert > erzeugeWerte(
            final List< VorNachteil > vorNachteile ) {

        final List< VorNachteilWert > werte =
            new ArrayList< VorNachteilWert >( vorNachteile.size() );

        for ( final VorNachteil vorNachteil : vorNachteile ) {

            if ( vorNachteil.hatVarianten() ) {

                final List< Variante > varianten = vorNachteil.getVarianten(null);

                final Variante variante;
                if ( 0 < varianten.size() ) {

                    variante = varianten.get( 0 );

                } else {

                    variante = new Variante();

                }

                werte.add( new VorNachteilWert( vorNachteil, variante ) );

            } else {

                werte.add( new VorNachteilWert( vorNachteil ) );

            }
        }

        return werte;
    }

    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
    }
    
    /**
     * @param held dessen Werte angezeigt und manipuliert werden.
     */
    public final void setHeld( final Held held ) {

        gewaehlte.setHeld( held );

        nachteile.setHeld( held );

        vorteile.setHeld( held );

    }

}
