/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.sonderfertigkeiten;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

import de.bernhardjung.dsaprogramm.basis.ElementMitNameImpl;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.basis.varianten.Varianten;
import de.bernhardjung.dsaprogramm.basis.varianten.VariantenFrei;
import de.bernhardjung.dsaprogramm.basis.varianten.VariantenTalentspezialisierung;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.kosten.Kosten;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.voraussetzungen.Voraussetzung;
import de.bernhardjung.dsaprogramm.talente.Kampftechnik;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;

public class Sonderfertigkeit
extends ElementMitNameImpl{

    /** Die Id der Sonderfertigkeit Scharfschuetze */
    public static final String ID_SCHARFSCHUETZE =
        "Sonderfertigkeit.Scharfschuetze";

    /** Die Id der Sonderfertigkeit Talentspezialisierung. */
    public static final String ID_TALENTSPEZIALISIERUNG =
        "Sonderfertigkeit.Talentspezialisierung";

    /** Die Id der Sonderfertigkeit Waffenspezialisierung. */
    public static final String ID_WAFFENSPEZIALISIERUNG =
        "Sonderfertigkeit.Waffenspezialisierung";

    /**
     * Eine Liste aller Gelaendekunden.
     */
    public static final List< Sonderfertigkeit > GELAENDEKUNDEN =
        new ArrayList< Sonderfertigkeit >();

    /**
     * Eine Map mit allen Auspraegungen der Sonderfertigkeit Scharfschuetze.
     * Die Schluessel in der Map sind Id von Fernkampftechniken.
     */
    private static final
    Map< String, Sonderfertigkeit > SCHARFSCHUETZE_MAP =
        new HashMap< String, Sonderfertigkeit >();

    /**
     * Eine Map in der jede Sonderfertigkeit unter ihrer Id gespeichert ist.
     */
    private static final
    Map< String, Sonderfertigkeit > SONDERFERTIGKEITEN_MAP =
        new HashMap< String, Sonderfertigkeit >();

    /**
     * Eine Map mit allen Talentspezialisierungen.
     * Fuer jedes Talent ist die entsprechende Spezialisierung gespeichert.
     * Die Schluessel der Map sind die Ids von Talenten.
     */
    private static final
    Map< String, Sonderfertigkeit > TALENTSPEZIALISIERUNGEN =
        new HashMap< String, Sonderfertigkeit >();

    /**
     * Eine Map mit allen Waffenspezialisierungen.
     * Unter jeder Kampftechnik ist die entsprechende Spezialisierung
     * gespeichert.
     * Die Schluessel der Map sind die IDs von Kampftechniken.
     */
    private static final
    Map< String, Sonderfertigkeit > WAFFENSPEZIALISIERUNGEN =
        new HashMap< String, Sonderfertigkeit >();

    public static final Sonderfertigkeit AKKLIMATISIERUNG_KAELTE =
        new Sonderfertigkeit(
            "Sonderfertigkeit.AkklimatisierungKaelte",
            Sonderfertigkeitgruppe.ALLGEMEIN );

    public static final Sonderfertigkeit AKKLIMATISIERUNG_HITZE =
        new Sonderfertigkeit(
            "Sonderfertigkeit.AkklimatisierungHitze",
            Sonderfertigkeitgruppe.ALLGEMEIN );

    public static final Sonderfertigkeit AUFMERKSAMKEIT =
        new Sonderfertigkeit(
            "Sonderfertigkeit.Aufmerksamkeit",
            Sonderfertigkeitgruppe.KAMPF );

    public static final Sonderfertigkeit AUSWEICHEN_I =
        new Sonderfertigkeit(
            "Sonderfertigkeit.AusweichenI",
            Sonderfertigkeitgruppe.KAMPF );

    public static final Sonderfertigkeit AUSWEICHEN_II =
        new Sonderfertigkeit(
            "Sonderfertigkeit.AusweichenII",
            Sonderfertigkeitgruppe.KAMPF );

    public static final Sonderfertigkeit AUSWEICHEN_III =
        new Sonderfertigkeit(
            "Sonderfertigkeit.AusweichenIII",
            Sonderfertigkeitgruppe.KAMPF );

    public static final Sonderfertigkeit BETAEUBUNGSSCHLAG =
        new Sonderfertigkeit(
            "Sonderfertigkeit.Betaeubungsschlag",
            Sonderfertigkeitgruppe.KAMPF );

    public static final Sonderfertigkeit DEFENSIVER_KAMPFSTIL =
        new Sonderfertigkeit(
            "Sonderfertigkeit.DefensiverKampfstil",
            Sonderfertigkeitgruppe.KAMPF );

    public static final Sonderfertigkeit DSCHUNGELKUNDIG =
        new Sonderfertigkeit(
            "Sonderfertigkeit.Dschungelkundig",
            // Sonderfertigkeitgruppe.GELAENDEKUNDE );
            Sonderfertigkeitgruppe.ALLGEMEIN );

    public static final Sonderfertigkeit EISKUNDIG = new Sonderfertigkeit(
            "Sonderfertigkeit.Eiskundig",
            // Sonderfertigkeitgruppe.GELAENDEKUNDE );
            Sonderfertigkeitgruppe.ALLGEMEIN );

    public static final Sonderfertigkeit FINTE =
        new Sonderfertigkeit( "Sonderfertigkeit.Finte",
                Sonderfertigkeitgruppe.KAMPF );

    public static final Sonderfertigkeit GEBIRGSKUNDIG = new Sonderfertigkeit(
            "Sonderfertigkeit.Gebirgskundig",
            // Sonderfertigkeitgruppe.GELAENDEKUNDE );
            Sonderfertigkeitgruppe.ALLGEMEIN );

    public static final Sonderfertigkeit GROSSE_MEDITATION =
        new Sonderfertigkeit(
            "Sonderfertigkeit.GrosseMeditation",
            Sonderfertigkeitgruppe.MAGISCHE );

    public static final Sonderfertigkeit HAMMERFAUST =
        new Sonderfertigkeit(
            "Sonderfertigkeit.Hammerfaust",
            Sonderfertigkeitgruppe.KAMPF );

    public static final Sonderfertigkeit HOEHLENKUNDIG = new Sonderfertigkeit(
            "Sonderfertigkeit.Hoehlenkundig",
            // Sonderfertigkeitgruppe.GELAENDEKUNDE );
            Sonderfertigkeitgruppe.ALLGEMEIN );

    public static final Sonderfertigkeit KAMPF_IM_WASSER =
        new Sonderfertigkeit(
            "Sonderfertigkeit.KampfImWasser",
            Sonderfertigkeitgruppe.KAMPF );

    public static final Sonderfertigkeit KAMPFREFLEXE =
        new Sonderfertigkeit(
            "Sonderfertigkeit.Kampfreflexe",
            Sonderfertigkeitgruppe.KAMPF );

    public static final Sonderfertigkeit KULTURKUNDE =
        new Sonderfertigkeit(
            "Sonderfertigkeit.Kulturkunde",
            // Sonderfertigkeitgruppe.KULTURKUNDE );
            Sonderfertigkeitgruppe.ALLGEMEIN );

    public static final Sonderfertigkeit LINKHAND =
        new Sonderfertigkeit(
            "Sonderfertigkeit.Linkhand",
            Sonderfertigkeitgruppe.KAMPF );

    public static final Sonderfertigkeit MARASKANKUNDIG = new Sonderfertigkeit(
            "Sonderfertigkeit.Maraskankundig",
            // Sonderfertigkeitgruppe.GELAENDEKUNDE );
            Sonderfertigkeitgruppe.ALLGEMEIN );

    public static final Sonderfertigkeit MEERESKUNDIG = new Sonderfertigkeit(
            "Sonderfertigkeit.Meereskundig",
            // Sonderfertigkeitgruppe.GELAENDEKUNDE );
            Sonderfertigkeitgruppe.ALLGEMEIN );

    public static final Sonderfertigkeit MEISTER_DER_IMPROVISATION =
        new Sonderfertigkeit(
                "Sonderfertigkeit.MeisterDerImprovisation",
                Sonderfertigkeitgruppe.ALLGEMEIN );

    public static final Sonderfertigkeit MEISTERPARADE =
        new Sonderfertigkeit(
            "Sonderfertigkeit.Meisterparade",
            Sonderfertigkeitgruppe.KAMPF );

    public static final Sonderfertigkeit NANDUSGEFAELLIGES_WISSEN =
        new Sonderfertigkeit(
                "Sonderfertigkeit.NandusgefaelligesWissen",
                Sonderfertigkeitgruppe.ALLGEMEIN );

    public static final Sonderfertigkeit ORTSKENNTNIS = new Sonderfertigkeit(
            "Sonderfertigkeit.Ortskenntnis",
            Sonderfertigkeitgruppe.ALLGEMEIN );

    public static final Sonderfertigkeit REGENERATION_I =
        new Sonderfertigkeit(
            "Sonderfertigkeit.RegenerationI",
            Sonderfertigkeitgruppe.MAGISCHE );

    public static final Sonderfertigkeit REITERKAMPF_PFERD =
        new Sonderfertigkeit(
            "Sonderfertigkeit.ReiterkampfPferd",
            Sonderfertigkeitgruppe.KAMPF );

    public static final Sonderfertigkeit REITERKAMPF_STREITWAGEN =
        new Sonderfertigkeit(
            "Sonderfertigkeit.ReiterkampfStreitwagen",
            Sonderfertigkeitgruppe.KAMPF );

    public static final Sonderfertigkeit REPRAESENTATION =
        new Sonderfertigkeit(
                "Sonderfertigkeit.Repraesentation",
                Sonderfertigkeitgruppe.MAGISCHE );

    public static final Sonderfertigkeit RUESTUNGSGEWOEHNUNG_I =
        new Sonderfertigkeit(
            "Sonderfertigkeit.RuestungsgewoehnungI",
            Sonderfertigkeitgruppe.KAMPF );

    public static final Sonderfertigkeit RUESTUNGSGEWOEHUNG_II =
        new Sonderfertigkeit(
            "Sonderfertigkeit.RuestungsgewoehnungII",
            Sonderfertigkeitgruppe.KAMPF );

    public static final Sonderfertigkeit SCHILDKAMPF_I =
        new Sonderfertigkeit(
            "Sonderfertigkeit.SchildkampfI",
            Sonderfertigkeitgruppe.KAMPF );

    public static final Sonderfertigkeit SCHILDKAMPF_II =
        new Sonderfertigkeit(
            "Sonderfertigkeit.SchildkampfII",
            Sonderfertigkeitgruppe.KAMPF );

    public static final Sonderfertigkeit SCHNELLLADEN_ARMBRUST =
        new Sonderfertigkeit(
            "Sonderfertigkeit.Schnellladen",
            Talent.ARMBRUST,
            Sonderfertigkeitgruppe.KAMPF );

    public static final Sonderfertigkeit SCHNELLLADEN_BOGEN =
        new Sonderfertigkeit(
            "Sonderfertigkeit.Schnellladen",
            Talent.BOGEN,
            Sonderfertigkeitgruppe.KAMPF );

    public static final Sonderfertigkeit SCHNELLZIEHEN =
        new Sonderfertigkeit(
            "Sonderfertigkeit.Schnellziehen",
            Sonderfertigkeitgruppe.KAMPF );

    public static final Sonderfertigkeit STANDFEST = new Sonderfertigkeit(
            "Sonderfertigkeit.Standfest",
            Sonderfertigkeitgruppe.ALLGEMEIN );

    public static final Sonderfertigkeit STEPPENKUNDIG = new Sonderfertigkeit(
            "Sonderfertigkeit.Steppenkundig",
            // Sonderfertigkeitgruppe.GELAENDEKUNDE );
            Sonderfertigkeitgruppe.ALLGEMEIN );

    public static final Sonderfertigkeit SUMPFKUNDIG = new Sonderfertigkeit(
            "Sonderfertigkeit.Sumpfkundig",
            // Sonderfertigkeitgruppe.GELAENDEKUNDE );
            Sonderfertigkeitgruppe.ALLGEMEIN );

    public static final Sonderfertigkeit UNAUER_SCHULE =
        new Sonderfertigkeit(
            "Sonderfertigkeit.UnauerSchule",
            Sonderfertigkeitgruppe.KAMPF );

    public static final Sonderfertigkeit WALDKUNDIG = new Sonderfertigkeit(
            "Sonderfertigkeit.Waldkundig",
            // Sonderfertigkeitgruppe.GELAENDEKUNDE );
            Sonderfertigkeitgruppe.ALLGEMEIN );

    public static final Sonderfertigkeit WUCHTSCHLAG =
        new Sonderfertigkeit(
            "Sonderfertigkeit.Wuchtschlag",
            Sonderfertigkeitgruppe.KAMPF );

    public static final Sonderfertigkeit WUESTENKUNDIG = new Sonderfertigkeit(
            "Sonderfertigkeit.Wuestenkundig",
            // Sonderfertigkeitgruppe.GELAENDEKUNDE );
            Sonderfertigkeitgruppe.ALLGEMEIN );

    /**
     * @return eine Liste aller Gelaendekunden.
     */
    public static final List< Sonderfertigkeit > getGelaendekunden() {

        return new ArrayList< Sonderfertigkeit >( GELAENDEKUNDEN );
    }

    /**
     * @return die zur uebergebenen Fernkampf-Technik passende Sonderfertigkeit
     *      Scharfschuetze.
     * @throws IllegalArgumentException wenn eine Nahkampf-Technik uebergeben
     *      wird.
     */
    public static final Sonderfertigkeit getScharfschuetze(
            final Kampftechnik kampftechnik ) {

        if ( kampftechnik.istNahkampfTechnik() ) {
            throw new IllegalArgumentException( String.format(
                    "Es wurde die Nahkampf-Technik %1$s uebergeben. Die " +
                    "Sonderfertigkeit Scharfschuetze existiert aber nur " +
                    "fuer Fernkampf-Techniken.",
                    kampftechnik ) );
        }

        if ( ! SCHARFSCHUETZE_MAP.containsKey( kampftechnik ) ) {

            final Sonderfertigkeit scharfschuetze;
            scharfschuetze = new Sonderfertigkeit( ID_SCHARFSCHUETZE,
                    // Sonderfertigkeitgruppe.SCHARFSCHUETZE );
                    kampftechnik,
                    Sonderfertigkeitgruppe.KAMPF );

            initialisieren(
                    scharfschuetze,
                    Kosten.AP_300,
                    Voraussetzung.SPEZIALISIERUNG );
            //scharfschuetze.setTalent( kampftechnik );
            scharfschuetze.einfuegen( Voraussetzung.STANDARD );

            SCHARFSCHUETZE_MAP.put( kampftechnik.getId(), scharfschuetze );
        }

        return SCHARFSCHUETZE_MAP.get( kampftechnik.getId() );
    }

    /**
     * @param id der gesuchten Sonderfertigkeit.
     * @return die Sonderfertigkeit mit der uebergebenen Id.
     * @throws IllegalArgumentException wenn es keinen Sonderfertigkeit mit
     *      der uebergebenen Id gibt.
     */
    public static final Sonderfertigkeit getSonderfertigkeit(
            final String id ) {

        if ( SONDERFERTIGKEITEN_MAP.containsKey( id ) ) {

            return SONDERFERTIGKEITEN_MAP.get( id );

        } else {

            throw new IllegalArgumentException(
                    "Es existiert keine Sonderfertigkeit mit Id " + id );
        }
    }
    /**
     * @see #getWaffenspezialisierung(Kampftechnik)
     * @param talent dessen Talentspezialisierung gesucht ist.
     * @return die Talentspezialisierung zum uebergebenen Talent. Die Methode
     *      liefert fuer alle Talente eine Talentspezialisierung auch fuer
     *      die Kampftechniken fuer die es eigentlich keine
     *      Talentspezialisierung gibt, sondern spezielle
     *      Waffenspezialisierungen.
     */
    public static final Sonderfertigkeit getTalentspezialisierung(
            final Talent talent ) {

        if ( ! TALENTSPEZIALISIERUNGEN.containsKey( talent ) ) {

            final Sonderfertigkeit spezialisierung;
            spezialisierung = new Sonderfertigkeit( ID_TALENTSPEZIALISIERUNG,
                    talent,
                    Sonderfertigkeitgruppe.TALENTSPEZIALISIERUNG );

            // spezialisierung.setTalent( talent );
            spezialisierung.setKosten( Kosten.TALENTSPEZIALISIERUNG );
            spezialisierung.setVarianten( new VariantenTalentspezialisierung( 
                            talent.getSpezialisierungen() ) );
            spezialisierung.einfuegen( Voraussetzung.SPEZIALISIERUNG );
            spezialisierung.einfuegen( Voraussetzung.STANDARD );
            spezialisierung.setTalentspezialisierung( true );

            TALENTSPEZIALISIERUNGEN.put( talent.getId(), spezialisierung );
        }

        return TALENTSPEZIALISIERUNGEN.get( talent.getId() );
    }

    /**
     * @param kampftechnik deren Waffenspezialisierung gesucht ist.
     * @return die Waffenspezialisierung der uebergebenen Kampftechnik.
     */
    public static final Sonderfertigkeit getWaffenspezialisierung(
            final Kampftechnik kampftechnik ) {

        if ( ! WAFFENSPEZIALISIERUNGEN.containsKey( kampftechnik.getId() ) ) {

            final Sonderfertigkeit spezialisierung;
            spezialisierung = new Sonderfertigkeit( 
                    ID_WAFFENSPEZIALISIERUNG,
                    kampftechnik,
                    Sonderfertigkeitgruppe.WAFFENSPEZIALISIERUNG );

            spezialisierung.setKosten( Kosten.WAFFENSPEZIALISIERUNG );
            spezialisierung.setVarianten( new VariantenTalentspezialisierung( 
                    kampftechnik.getSpezialisierungen() ) );
            spezialisierung.einfuegen( Voraussetzung.SPEZIALISIERUNG );
            spezialisierung.einfuegen( Voraussetzung.STANDARD );

            WAFFENSPEZIALISIERUNGEN.put(
                    kampftechnik.getId(),
                    spezialisierung );
        }

        return WAFFENSPEZIALISIERUNGEN.get( kampftechnik.getId() );
    }

    /**
     * Initialisiert eine Gelaendekunde.
     *
     * @param gelaendekunde die initialisiert werden soll.
     */
    private static final void initialisiereGelaendekunde(
            final Sonderfertigkeit gelaendekunde ) {

        initialisieren( gelaendekunde, Kosten.GELAENDEKUNDE );

        gelaendekunde.setGelaendekunde( true );

        if ( ! GELAENDEKUNDEN.contains( gelaendekunde ) ) {

            GELAENDEKUNDEN.add( gelaendekunde );
        }

    }

    /** Initialisiert alle Sonderfertigkeiten. */
    public static final void initialisieren() {

        initialisieren(
                AKKLIMATISIERUNG_HITZE,
                Kosten.AP_150,
                Voraussetzung.NICHT_BEGINN );

        initialisieren(
                AKKLIMATISIERUNG_KAELTE,
                Kosten.AP_150,
                Voraussetzung.NICHT_BEGINN );

        initialisieren(
                AUFMERKSAMKEIT,
                Kosten.AP_200_KAMPF,
                Voraussetzung.IN_12 );

        initialisieren(
                AUSWEICHEN_I,
                Kosten.AP_300_SCHLANGEMENSCH,
                Voraussetzung.GE_10 );

        initialisieren(
                AUSWEICHEN_II,
                Kosten.AP_200_SCHLANGEMENSCH,
                Voraussetzung.GE_12,
                Voraussetzung.AUSWEICHEN_I,
                Voraussetzung.AUFMERKSAMKEIT );

        initialisieren(
                AUSWEICHEN_III,
                Kosten.AP_200_SCHLANGEMENSCH,
                Voraussetzung.GE_15,
                Voraussetzung.AUSWEICHEN_II,
                Voraussetzung.KAMPFREFLEXE );

        initialisieren(
                BETAEUBUNGSSCHLAG,
                Kosten.AP_200,
                Voraussetzung.FINTE,
                Voraussetzung.WUCHTSCHLAG );

        initialisieren(
                DEFENSIVER_KAMPFSTIL,
                Kosten.AP_100,
                Voraussetzung.GE_12,
                Voraussetzung.MEISTERPARADE );

        initialisiereGelaendekunde( DSCHUNGELKUNDIG );

        initialisiereGelaendekunde( EISKUNDIG );

        initialisieren(
                FINTE,
                Kosten.AP_200_KAMPF,
                Voraussetzung.GE_12 );

        initialisiereGelaendekunde( GEBIRGSKUNDIG );

        initialisieren(
                HAMMERFAUST,
                Kosten.AP_150_KAMPF,
                Voraussetzung.RAUFEN_7 );

        initialisiereGelaendekunde( HOEHLENKUNDIG );

        initialisieren(
                KAMPF_IM_WASSER,
                Kosten.AP_100,
                Voraussetzung.GE_12,
                Voraussetzung.KOERPERBEHERRSCHUNG_7 );

        initialisieren(
                KAMPFREFLEXE,
                Kosten.AP_300_KAMPF,
                Voraussetzung.INITIATIVE_10 );

        initialisieren(
                KULTURKUNDE,
                Kosten.KULTURKUNDE,
                Voraussetzung.IN_10,
                Voraussetzung.KL_10,
                Voraussetzung.KULTURKUNDE )
                .setVarianten( Varianten.KULTURKUNDE );

        initialisieren(
                LINKHAND,
                Kosten.AP_300_KAMPF,
                Voraussetzung.KK_10,
                Voraussetzung.GE_10,
                Voraussetzung.NICHT_EINARMIG );

        initialisiereGelaendekunde( MARASKANKUNDIG );

        initialisiereGelaendekunde( MEERESKUNDIG );

        initialisieren(
                MEISTER_DER_IMPROVISATION,
                Kosten.AP_200,
                Voraussetzung.FF_12,
                Voraussetzung.IN_12 );

        initialisieren(
                MEISTERPARADE,
                Kosten.AP_200_KAMPF,
                Voraussetzung.PARADE_8 );

        initialisieren(
                NANDUSGEFAELLIGES_WISSEN,
                Kosten.NANDUSGEFAELLIGES_WISSEN,
                Voraussetzung.KL_12,
                Voraussetzung.IN_12 );

        initialisieren( ORTSKENNTNIS, Kosten.ORTSKENNTNIS ).setVarianten();

        initialisieren(
                REITERKAMPF_PFERD,
                Kosten.AP_200_KAMPF,
                Voraussetzung.REITEN_7 );

        initialisieren(
                REITERKAMPF_STREITWAGEN,
                Kosten.AP_200_KAMPF,
                Voraussetzung.FAHRZEUGE_LENKEN_7 );

        initialisieren(
                RUESTUNGSGEWOEHNUNG_I,
                Kosten.AP_150_KAMPF,
                Voraussetzung.KK_10 )
                .setVarianten( Varianten.RUESTUNGEN );

        initialisieren(
                RUESTUNGSGEWOEHUNG_II,
                Kosten.AP_300_KAMPF,
                Voraussetzung.RUESTUNGSGEWOEHNUNG_I,
                Voraussetzung.KK_12 );

        for ( final Talent talent  : Talentgruppe.KAMPF ) {

            final Kampftechnik kampftechnik = (Kampftechnik) talent;

            if ( kampftechnik.istNahkampfTechnik() ) {
                continue;
            }

            getScharfschuetze( kampftechnik );
        }

        initialisieren(
                SCHILDKAMPF_I,
                Kosten.AP_200_KAMPF,
                Voraussetzung.LINKHAND,
                Voraussetzung.KK_12,
                Voraussetzung.NICHT_EINARMIG );

        initialisieren(
                SCHILDKAMPF_II,
                Kosten.AP_200,
                Voraussetzung.SCHILDKAMPF_I,
                Voraussetzung.KK_15,
                Voraussetzung.NICHT_EINARMIG );

        initialisieren(
                SCHNELLLADEN_ARMBRUST,
                Kosten.AP_200,
                Voraussetzung.FF_12,
                Voraussetzung.KK_12,
                Voraussetzung.SPEZIALISIERUNG );
               // .setTalent( Talent.ARMBRUST );

        initialisieren(
                SCHNELLLADEN_BOGEN,
                Kosten.AP_200,
                Voraussetzung.FF_12,
                Voraussetzung.KK_12,
                Voraussetzung.SPEZIALISIERUNG );
               // .setTalent( Talent.BOGEN );

        initialisieren(
                SCHNELLZIEHEN,
                Kosten.AP_200_SCHLANGEMENSCH,
                Voraussetzung.GE_12,
                Voraussetzung.FF_10 );

        initialisieren(
                STANDFEST,
                Kosten.AP_200_MIT_BALANCE,
                Voraussetzung.GE_12 );

        initialisiereGelaendekunde( STEPPENKUNDIG );

        initialisiereGelaendekunde( SUMPFKUNDIG );

        for ( final Talent talent  : Talent.getTalente() ) {

            final Talentgruppe talentgruppe = talent.getTalentgruppe();

            if ( Talentgruppe.KAMPF.equals( talentgruppe )
              || Talentgruppe.SPRACHEN.equals( talentgruppe )
              || Talentgruppe.SCHRIFTEN.equals( talentgruppe )
              || Talentgruppe.RITUALKENNTNIS.equals( talentgruppe ) ) {
                continue;
            }

            getTalentspezialisierung( talent );
        }

        initialisieren(
                UNAUER_SCHULE,
                Kosten.AP_150_KAMPF,
                Voraussetzung.RINGEN_7 );

        for ( final Talent kampftechnik : Talentgruppe.KAMPF ) {

            if ( Talent.RINGEN == kampftechnik ) {
                continue;
            }

            getWaffenspezialisierung( (Kampftechnik) kampftechnik );
        }

        initialisiereGelaendekunde( WALDKUNDIG );

        initialisieren(
                WUCHTSCHLAG,
                Kosten.AP_200_KAMPF,
                Voraussetzung.KK_12 );

        initialisiereGelaendekunde( WUESTENKUNDIG );

        /* ****************************
         *
         * magische Sonderfertigkeiten
         *
         ******************************/
        initialisieren(
                "Sonderfertigkeit.AstraleMeditation",
                Sonderfertigkeitgruppe.MAGISCHE,
                Kosten.AP_100,
                Voraussetzung.IN_12,
                Voraussetzung.Z );

        initialisieren(
                "Sonderfertigkeit.BindungDesStabes",
                //Sonderfertigkeitgruppe.STABZAUBER,
                Sonderfertigkeitgruppe.MAGISCHE,
                Kosten.AP_100,
                Voraussetzung.RITUALKENNTNIS_3 );

        initialisieren(
                "Sonderfertigkeit.Erinnerungsmelodie",
                // Sonderfertigkeitgruppe.ELFENLIEDER,
                Sonderfertigkeitgruppe.MAGISCHE,
                Kosten.AP_150,
                Voraussetzung.ZWEISTIMMIGER_GESANG );

        initialisieren(
                "Sonderfertigkeit.EwigeFlamme",
                // Sonderfertigkeitgruppe.STABZAUBER,
                Sonderfertigkeitgruppe.MAGISCHE,
                Kosten.AP_75,
                Voraussetzung.RITUALKENNTNIS_5 );

        initialisieren(
                "Sonderfertigkeit.Freundschaftslied",
                // Sonderfertigkeitgruppe.ELFENLIEDER,
                Sonderfertigkeitgruppe.MAGISCHE,
                Kosten.AP_50,
                Voraussetzung.ZWEISTIMMIGER_GESANG );

        initialisieren(
                "Sonderfertigkeit.Friedenslied",
                // Sonderfertigkeitgruppe.ELFENLIEDER,
                Sonderfertigkeitgruppe.MAGISCHE,
                Kosten.AP_250,
                Voraussetzung.ZWEISTIMMIGER_GESANG );

        initialisieren(
                GROSSE_MEDITATION,
                Kosten.AP_0,
                Voraussetzung.IN_12,
                Voraussetzung.KL_12,
                Voraussetzung.Z );

        initialisieren(
                "Sonderfertigkeit.Kraftfokus",
                // Sonderfertigkeitgruppe.STABZAUBER,
                Sonderfertigkeitgruppe.MAGISCHE,
                Kosten.AP_150,
                Voraussetzung.RITUALKENNTNIS_11 );

        initialisieren(
                "Sonderfertigkeit.LiedDerLieder",
                // Sonderfertigkeitgruppe.ELFENLIEDER,
                Sonderfertigkeitgruppe.MAGISCHE,
                Kosten.AP_150,
                Voraussetzung.ZWEISTIMMIGER_GESANG );

        initialisieren(
                REGENERATION_I,
                Kosten.AP_100,
                Voraussetzung.VZ,
                Voraussetzung.IN_12 );

        initialisieren(
                "Sonderfertigkeit.RegenerationII",
                Sonderfertigkeitgruppe.MAGISCHE,
                Kosten.AP_100,
                Voraussetzung.Z,
                Voraussetzung.IN_15,
                Voraussetzung.REGENERATION_I );

        initialisieren(
                REPRAESENTATION,
                Kosten.AP_2000,
                Voraussetzung.NICHT_FREI_WAEHLBAR )
                .setVarianten( Varianten.REPRAESENTATION );

        initialisieren(
                "Sonderfertigkeit.Salasandra",
                Sonderfertigkeitgruppe.MAGISCHE,
                Kosten.AP_200,
                Voraussetzung.ZWEISTIMMIGER_GESANG );

        initialisieren(
                "Sonderfertigkeit.SeilDesAdepten",
                // Sonderfertigkeitgruppe.STABZAUBER,
                Sonderfertigkeitgruppe.MAGISCHE,
                Kosten.AP_150,
                Voraussetzung.RITUALKENNTNIS_7 );

        initialisieren(
                "Sonderfertigkeit.Sorgenlied",
                // Sonderfertigkeitgruppe.ELFENLIEDER,
                Sonderfertigkeitgruppe.MAGISCHE,
                Kosten.AP_75,
                Voraussetzung.ZWEISTIMMIGER_GESANG );

        initialisieren(
                "Sonderfertigkeit.Windgefluester",
                // Sonderfertigkeitgruppe.ELFENLIEDER,
                Sonderfertigkeitgruppe.MAGISCHE,
                Kosten.AP_100,
                Voraussetzung.ZWEISTIMMIGER_GESANG );

        initialisieren(
                "Sonderfertigkeit.Zaubermelodie",
                // Sonderfertigkeitgruppe.ELFENLIEDER,
                Sonderfertigkeitgruppe.MAGISCHE,
                Kosten.AP_150,
                Voraussetzung.ZWEISTIMMIGER_GESANG );

    }

    /**
     * Initialisiert eine Sonderfertigkeit.
     * @param sonderfertigkeit die initialisiert werden soll.
     * @param kosten der Sonderfertigkeit.
     * @param voraussetzungen der Sonderfertigkeit.
     * @return die uebergebene Sonderfertigkeit.
     */
    private static final Sonderfertigkeit initialisieren(
            final Sonderfertigkeit sonderfertigkeit,
            final Kosten kosten,
            final Voraussetzung ... voraussetzungen ) {

        sonderfertigkeit.setKosten( kosten );

        for ( final Voraussetzung voraussetzung : voraussetzungen ) {
            sonderfertigkeit.einfuegen( voraussetzung );
        }
        sonderfertigkeit.einfuegen( Voraussetzung.STANDARD );

        SONDERFERTIGKEITEN_MAP.put(
                sonderfertigkeit.getId(),
                sonderfertigkeit );

        return sonderfertigkeit;
    }

    /**
     * Initialisiert eine Sonderfertigkeit.
     * @param id der Sonderfertigkeit die initialisiert werden soll.
     * @param dieGruppe zu der die Sonderfertigkeit gehoert.
     * @param kosten der Sonderfertigkeit.
     * @param voraussetzungen der Sonderfertigkeit.
     * @return die erzeugte Sonderfertigkeit.
     */
    private static final Sonderfertigkeit initialisieren(
            final String dieId,
            final Sonderfertigkeitgruppe dieGruppe,
            final Kosten kosten,
            final Voraussetzung ... voraussetzungen ) {

        final Sonderfertigkeit sonderfertigkeit =
            new Sonderfertigkeit( dieId, dieGruppe );

        return initialisieren( sonderfertigkeit, kosten, voraussetzungen );
    }

    /** Gibt an ob die Sonderfertigkeit eine Gelaendekunde ist. */
    private boolean gelaendekunde;

    /** Die Gruppe zu der die Sonderfertigkeit gehoert. */
    private final Sonderfertigkeitgruppe gruppe;

    /**
     * Das Objekt das sich um die Bestimmung der AP- und GP-Kosten der
     * Sonderfertigkeit kuemmert.
     */
    private Kosten kosten;

    /**
     * Ein Talent das evtl. mit der Sonderfertigkeit verbunden ist. Das ist
     * bei Waffen-/Talentspezialisierungen, Schnelladen und Scharfschuetze der
     * Fall.
     */
    private Talent talent;

    /**
     * Gibt an ob es sich bei der Sonderfertigkeiten um eine
     * Talentspezialisierung handelt.
     */
    private boolean talentspezialisierung;

    /**
     * Die Varianten die mit der Sonderfertigkeit verbunden sind.
     * Bei einer Waffenspezialisierung sind das z.B. die Waffengattungen auf
     * die man sich spezialisieren kann.
     */
    private Varianten varianten;

    /**
     * Die Voraussetzungen die ein Held erfuellen muss wenn er die
     * Sonderfertigkeit waehlen will.
     */
    private final List< Voraussetzung > voraussetzungen;

    /**
     * Erzeugt eine neue Sonderfertigkeit.
     *
     * @param dieId der Sonderfertigkeit.
     * @param dieGruppe zu der die Sonderfertigkeit gehoert.
     */
    private Sonderfertigkeit(
            final String dieId,
            final Sonderfertigkeitgruppe dieGruppe ) {

        super ( dieId );
        
        gelaendekunde = false;

        gruppe = dieGruppe;

        varianten = Varianten.KEINE;

        voraussetzungen = new ArrayList< Voraussetzung >();

        gruppe.einfuegen( this );
    }

    /**
     * Erzeugt eine neue Sonderfertigkeit.
     *
     * @param dieId der Sonderfertigkeit.
     * @param talent fuer die Sonderfertigkeit, z.B. bei Scharfschuetze.
     * @param dieGruppe zu der die Sonderfertigkeit gehoert.
     */
    private Sonderfertigkeit(
            final String dieId,
            final Talent dasTalent,
            final Sonderfertigkeitgruppe dieGruppe ) {

        super( dieId );
        
        gelaendekunde = false;

        gruppe = dieGruppe;

        talent = dasTalent;

        varianten = Varianten.KEINE;

        voraussetzungen = new ArrayList< Voraussetzung >();

        gruppe.einfuegen( this );
    }

    /**
     * Fuegt der Sonderfertigkeit eine weitere Voraussetzung hinzu.
     *
     * @param voraussetzung die fuer die Sonderfertigkeit gilt.
     */
    private final void einfuegen( final Voraussetzung voraussetzung ) {

        assert( null != voraussetzung ) : String.format(
                "%1$s sollten als Voraussetzung null zugeweisen werden.",
                getName() );

        voraussetzungen.add( voraussetzung );
    }

    /**
     * Liefert die Talente, Sonderfertigkeiten, usw. von denen die
     * Sonderfertigkeit in irgendeiner Weise abhängig ist, z.B. weil
     * sie Einfluss auf die Berechnung der Kosten haben oder weil sie
     * Vorausgesetzt werden.
     *
     * @return die Regelelemente von denen die Sonderfertigkeit abhaengt.
     */
    public final Collection< Object > getAbhaengigkeiten() {

        final Collection< Object > abhaengigkeiten = new HashSet< Object >();

        abhaengigkeiten.add( this );

        if ( null != talent ) {
            abhaengigkeiten.add( talent );
        }

        abhaengigkeiten.addAll( kosten.getAbhaengigkeiten() );

        for ( final Voraussetzung voraussetzung: voraussetzungen ) {

            abhaengigkeiten.addAll( voraussetzung.getAbhaengigkeiten() );
        }

        return abhaengigkeiten;
    }

    /**
     * @return die AP-Kosten die fuer die Sonderfertigkeit zu zahlen sind.
     */
    public int getApKosten() {
        return kosten.getApKosten(this);
    }

    /**
     * @param held der die AP-Kosten zahlen soll.
     * @param wert fuer den die AP-Kosten zu zahlen sind.
     * @return die AP die der Held fuer den Wert zu entrichten hat.
     */
    public final int getApKosten(
            final Held held,
            final SonderfertigkeitWert wert ) {

        return kosten.getApKosten( held, wert );
    }

    /**
     * @param held der die AP-Kosten zahlen soll.
     * @param wert fuer den die AP-Kosten zu zahlen sind.
     * @return die AP die der Held fuer den Wert zu entrichten hat.
     */
    public final int getApKosten(
            final Held held,
            final SonderfertigkeitWert wert,
            final Lernmethode lernmethode ) {

        return kosten.getApKosten( held, wert, lernmethode );
    }

    /**
     * @param wert dessen AP-Kosten bestimmt werden sollen.
     * @return die AP-Kosten des Wertes.
     */
    public double getApKosten(
            final SonderfertigkeitWert wert ) {
        return kosten.getApKosten( wert );
    }

    /**
     * @see #getGpKosten(Held, SonderfertigkeitWert)
     * @return die Standard GP-Kosten fuer die Sonderfertigkeit. Koennen die
     *         Kosten ohne den Helden nicht genau bestimmt werden, wird
     *         irgendein Wert geliefert.
     */
    public final int getGpKosten( final SonderfertigkeitWert wert ) {
        return kosten.getGpKosten(wert);
    }

    /**
     * @see #hatGpKosten()
     * @param held der die GP-Kosten zahlen soll.
     * @param wert fuer den die GP-Kosten zu zahlen sind.
     * @return die GP die der Held fuer den Wert zu entrichten hat. Es wird
     *      auch fuer Sonderfertigkeiten die eigentlich nicht mit GP bezahlt
     *      werden koennen ein Wert geliefert.
     */
    public final int getGpKosten(
            final Held held,
            final SonderfertigkeitWert wert ) {

        return kosten.getGpKosten( held, wert );
    }

    /**
     * @return die Gruppe zu der die Sonderfertigkeit gehoert.
     */
    public final Sonderfertigkeitgruppe getGruppe() {
        return gruppe;
    }

    /**
     * @return den Namen der Sonderfertigkeit.
     */
    public final String getName() {
        
        final StringBuilder name = new StringBuilder( super.getName() );
        
        if ( null != talent ) {
            
            name.append( " " ).append( talent.getName() );
        
        } 

        return name.toString();
    }

    /**
     * Liefert die Variante, die einem Objekt entspricht. Das Objekt kann selbst
     * vom Typ Variante sein, woraufhin es selbst zurueckt geliefert wird oder
     * es kann eine freie Variante sein, die der Benutzer eingegeben hat. Ist
     * es eine freie Variante wird die zum Text passende Variante erzeugt,
     * falls sie existiert und zurueckgeliefert.
     * 
     * @param objekt vom Typ Variante oder String.
     * @return Die Variante.
     */
    public final Variante getVariante( final Object objekt ) {
        
        return varianten.getVariante( objekt );
    }
    
    /**
     * @param held fuer den die Varianten sein sollen.
     * @see #hatVarianten()
     * @return die Liste der Varianten die mit dieser Sf verbunden ist.
     *      Die Liste ist leer, wenn die Sf keine Varianten besitzt.
     *      Die Liste kann auch dann leer sein wenn alle Varianten vom Benutzer
     *      eingegeben werden muessen.
     */
    public final List< Variante > getVarianten(final Held held) {
        return varianten.getVarianten(held);
    }

    /**
     * @return das Talent das mit der Sonderfertigkeit verbunden ist, oder
     *      {@code null} wenn es kein solches Talent gibt.
     */
    public final Talent getTalent() {
        return talent;
    }

    /**
     * @return {@code true}, wenn die Varianten der Sonderfertigkeit vom
     *      benutzer festegelegt werden darf, sonst {@code false}.
     */
    public final boolean hatFreieVarianten() {
        return varianten.istFreieVariante();
    }

    /**
     * @return {@code true}, wenn die Sonderfertigkeit mit GP bezahlt werden
     *      darf, sonst {@code false}.
     */
    public final boolean hatGpKosten() {
        return kosten.hatGpKosten();
    }

    /**
     * @see #getTalent()
     * @return {@code true}, wenn die Sonderfertigkeit ein Talent besitzt, sonst
     *         {@code false}.
     */
    public boolean hatTalent() {
        return null != talent;
    }

    /**
     * @return {@code true} wenn es fuer diese Sf Varianten gibt, sonst
     *      {@code false}.
     */
    public final boolean hatVarianten() {
        return varianten.hatVarianten();
    }

    /**
     * @return {@code true} wenn es sich bei der Sonderfertigkeit um eine
     *      Gelaendekunde handelt, sonst {@code false}.
     */
    public final boolean istGelaendekunde() {
        return gelaendekunde;
    }

    /**
     * @return {@code true}, wenn es sich um eine Talentspezialisierung
     *         handelt, sonst {@code false}.
     */
    public final boolean istTalentspezialisierung() {
        return talentspezialisierung;
    }

    /**
     * Prueft ob eine Held alle Voraussetzungen fuer eine Sonderfertigkeit
     * erfuellt.
     *
     * @param held der die Voraussetzungen erfuellen soll.
     * @param wert dessen Voraussetzungen erufellt werden sollen.
     * @return eine Beschreibung der Voraussetzungen die vom Helden nicht
     *      erfuellt werden.
     */
    public final KonfliktBeschreibung pruefen(
            final Held held,
            final SonderfertigkeitWert wert ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        for ( final Voraussetzung voraussetzung : voraussetzungen ) {

            beschreibung.einfuegen( voraussetzung.pruefen( held, wert ) );
        }

        return beschreibung;
    }

    /**
     * @param istGelaendekunde {@code true} wenn es sich bei der
     *      Sonderfertigkeit um eine Gelaendekunde handelt, sonst
     *      {@code false}.
     */
    private final void setGelaendekunde( final boolean istGelaendekunde ) {

        assert ( gelaendekunde == false ) : String.format(
                "Die Sonderfertigkeit %1$s sollte zum zweiten mal als " +
                "Gelaendekunde gekennzeichent werden.",
                getName() );

        gelaendekunde = istGelaendekunde;
    }

    /**
     * @param dieKosten Das Objekt das sich um die Bestimmung der AP- und
     *      GP-Kosten fuer die Sonderfertigkeit kuemmert.
     */
    private final void setKosten( final Kosten dieKosten ) {

        assert ( null == kosten ) : String.format(
            "Die Kosten von Sonderfertigkeit %1$s sollten ein zweites mal " +
            "gesetzt werden.",
            getName() );

        kosten = dieKosten;
    }

    /**
     * @param istTalentspezialisierung
     *            {@code true}, wenn es sich um eine Talentspezialisierung
     *            handelt, sonst {@code false}.
     */
    private final void setTalentspezialisierung(
            final boolean istTalentspezialisierung ) {

        talentspezialisierung = istTalentspezialisierung;
    }

    /**
     * @param dieVarianten die die Sonderfertigkeit zur Verfuegung stellen
     *      soll.
     */
    private final void setVarianten( final Varianten dieVarianten ) {

        assert ( Varianten.KEINE == varianten ) : String.format(
                "Die Varianten von %1$s sollten zum zweiten mal " +
                "initialisiert werden.\n" +
                "vorhandene Varianten: %2$s\n" +
                "      neue Varianten: %3$s",
                getName(),
                varianten,
                dieVarianten );

        varianten = dieVarianten;
    }

    /**
     * @param dieVarianten die die Sonderfertigkeit zur Verfuegung stellen
     *      soll.
     */
    private final void setVarianten( final String ... dieVarianten ) {

        assert ( Varianten.KEINE == varianten ) : String.format(
                "Die Varianten von %1$s sollten zum zweiten mal " +
                "initialisiert werden.\n" +
                "vorhandene Varianten: %2$s\n" +
                "      neue Varianten: %3$s",
                getName(),
                varianten,
                Arrays.toString( dieVarianten ) );

        varianten = new VariantenFrei( dieVarianten );
    }

    /** @return {@link #getName()} */
    @Override
    public final String toString() {
        return getName();
    }
}
