/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.sonderfertigkeiten;

import java.util.AbstractList;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.ElementMitName;
import de.bernhardjung.dsaprogramm.basis.ElementMitNameImpl;
import de.bernhardjung.dsaprogramm.util.Util;

public class Sonderfertigkeitgruppe
extends AbstractList< Sonderfertigkeit >
implements Comparable< Sonderfertigkeitgruppe >, ElementMitName {

    /** Eine Liste aller Gruppen */
    private static final List< Sonderfertigkeitgruppe > GRUPPEN =
        new ArrayList< Sonderfertigkeitgruppe >();

    public static final Sonderfertigkeitgruppe ALLGEMEIN =
        new Sonderfertigkeitgruppe( "SonderfertigkeitGruppe.Allgemein" );
/*
    public static final Sonderfertigkeitgruppe ELFENLIEDER =
        new Sonderfertigkeitgruppe( "SonderfertigkeitGruppe.Elfenlieder" );
*//*
    public static final Sonderfertigkeitgruppe GELAENDEKUNDE =
        new Sonderfertigkeitgruppe( "SonderfertigkeitGruppe.Gelaendekunde" );
*/
    public static final Sonderfertigkeitgruppe KAMPF =
        new Sonderfertigkeitgruppe( "SonderfertigkeitGruppe.Kampf" );
/*
    public static final Sonderfertigkeitgruppe KULTURKUNDE =
        new Sonderfertigkeitgruppe( "SonderfertigkeitGruppe.Kulturkunde" );
*/
    public static final Sonderfertigkeitgruppe MAGISCHE =
        new Sonderfertigkeitgruppe( "SonderfertigkeitGruppe.Magische" );
/*
    public static final Sonderfertigkeitgruppe SCHARFSCHUETZE =
        new Sonderfertigkeitgruppe( "SonderfertigkeitGruppe.Scharfschuetze" );

    public static final Sonderfertigkeitgruppe STABZAUBER =
        new Sonderfertigkeitgruppe( "SonderfertigkeitGruppe.Stabzauber" );
*/
    public static final Sonderfertigkeitgruppe TALENTSPEZIALISIERUNG =
        new Sonderfertigkeitgruppe(
                "SonderfertigkeitGruppe.Talentspezialisierung" );

    public static final Sonderfertigkeitgruppe WAFFENSPEZIALISIERUNG =
        new Sonderfertigkeitgruppe( "SonderfertigkeitGruppe.Waffenspezialisierng" );

    /**
     * @return eine Liste aller Gruppen.
     */
    public static final List< Sonderfertigkeitgruppe > getGruppen() {
        return new ArrayList< Sonderfertigkeitgruppe >( GRUPPEN );
    }

    /**
     * Sortiert die Sonderfertigkeiten in allen Gruppen neu, wenn der Benutzer
     * die Sprache des Programm geaendert hat.
     */
    public static final void neuSortieren() {
        
        for ( final Sonderfertigkeitgruppe gruppe : GRUPPEN ) {
            
            gruppe.sortieren();
        }
    }
    
    /** Der volle Name der Sonderfertigkeit, zB "Talentspezialisierung" */
    private final ElementMitName name;

    /** Die Sonderfertigkeiten die zu dieser Liste gehoeren. */
    private final List< Sonderfertigkeit > sonderfertigkeiten;

    private Sonderfertigkeitgruppe( final String dieId ) {

        name = new ElementMitNameImpl( dieId );

        sonderfertigkeiten = new ArrayList< Sonderfertigkeit >();

        int index = Collections.binarySearch( GRUPPEN, this );

        index = -( index + 1);

        GRUPPEN.add( index, this );
    }

    /**
     * Sortiert die Gruppen alphabetisch nach Namen.
     */
    public int compareTo( final Sonderfertigkeitgruppe o ) {
        return Util.compare( getName(), o.getName() );
    }

    /**
     * Fuegt der Gruppe eine neue Sonderfertigkeit hinzu.
     * Macht nichts, wenn die Sonderfertigkeit schon in der Gruppe ist.
     *
     * @param sonderfertigkeit fuer die Gruppe.
     */
    final void einfuegen( final Sonderfertigkeit sonderfertigkeit ) {

        int index = Collections.binarySearch( 
                sonderfertigkeiten,
                sonderfertigkeit,
                Sonderfertigkeit.NAME_COMPARATOR );

        if ( 0 > index ) {

            index = -( index + 1 );

            sonderfertigkeiten.add( index, sonderfertigkeit );
        }
    }

    /**
     * @return die Sonderfertigkeit an Position {@code index}.
     */
    @Override
    public Sonderfertigkeit get( final int index ) {
        return sonderfertigkeiten.get( index );
    }
    
    /**
     * @return die ID der Gruppe.
     */
    public final String getId() {
        
        return name.getId();
    }

    /**
     * @return den Namen der Gruppe.
     */
    public final String getName() {
        return name.getName();
    }

    /**
     * @return eine Abkuerzung fuer den Namen der Gruppe.
     */
    public final String getKuerzel() {
        return name.getKuerzel();
    }

    /**
     * @return Der Schluessel mit dem das Kuerzel der Gruppe aus der 
     *          Lokalisierungsdatei geladen wird.
     */
    public final String getLokalisierungsschluesselKuerzel() {
        
        return name.getLokalisierungsschluesselKuerzel();
    }
    
    /**
     * @return Den Schluessel unter dem der Name der Gruppe in der 
     *          Lokalisierungsdatei zu finden ist. 
     */
    public final String getLokalisierungsschluesselName() {
        
        return name.getLokalisierungsschluesselName();
    }
    
    /**
     * @throws Exception
     */
    public void setName( String neuerName ) {
        
        throw new RuntimeException( "Nicht implementiert" );
    }
    
    /**
     * @return die Anzahl an Sonderfertigkeiten in der Liste.
     */
    @Override
    public final int size() {
        return sonderfertigkeiten.size();
    }
    
    /**
     * Sortiert die Sonderfertigkeiten in der Gruppe neu, wenn der Benutzer
     * die Sprache des Programms geaendert hat.
     */
    private final void sortieren() {
        
        Collections.sort( 
                sonderfertigkeiten, 
                Sonderfertigkeit.NAME_COMPARATOR );
    }
}
