/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.sonderfertigkeiten.kosten;

import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

class KostenAutomatischMitVorNachteil extends KostenFest {

    /** Der Vor-/Nachteile die die Sonderfertigkeit mit sich bringen. */
    private final VorNachteil[] vorNachteile;
    
    KostenAutomatischMitVorNachteil( 
            final int dieApKosten,
            final VorNachteil ... dieVorNachteile ) {
        
        super( dieApKosten );
        
        vorNachteile = dieVorNachteile;
    }
    
    @Override
    public Collection<Object> getAbhaengigkeiten() {
        
        Collection< Object > abhaengigkeiten = super.getAbhaengigkeiten();
        
        for ( final VorNachteil vorNachteil : vorNachteile ) {
            
            abhaengigkeiten.add( vorNachteil );
        }
        
        return abhaengigkeiten;
    }
    
    public int getApKosten(
            final Held held, 
            final SonderfertigkeitWert wert ) {
        
        if ( held.hatVorgegeben( wert ) ) {
            
            return 0;
        }
        
        for ( final VorNachteil vorNachteil : vorNachteile ) {
         
            if ( held.hat( vorNachteil ) ) {
                
                return 0;
                
            } 
        }
            
        return super.getApKosten(wert);
    }

}
