/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.talente.stufen;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

class StufenTalent implements Stufen {

    private VorNachteilWert begabungTalent;

    private VorNachteilWert begabungTalentgruppe;

    public Collection<Object> getAbhaengigkeiten() {

        final List<Object> abhaengigkeiten = new ArrayList< Object >( 2 );

        abhaengigkeiten.add( VorNachteil.BEGABUNG_TALENT );

        abhaengigkeiten.add( VorNachteil.BEGABUNG_TALENTGRUPPE );

        return abhaengigkeiten;
    }

    /** {@inheritDoc} */
    public int getHoechststufe( final Held held, final Talent talent ) {

        int hoechststufe = 0;

        for ( final Eigenschaft eigenschaft :
            talent.getProbeneigenschaften() ) {

            hoechststufe = Math.max( hoechststufe,
                                     held.getStufe( eigenschaft ) );
        }

        setBegabungen( talent );

        if ( held.hat( begabungTalent ) || held.hat( begabungTalentgruppe ) ) {

            return hoechststufe + 5;

        } else {

            return hoechststufe + 3;

        }
    }

    /**
     * @return die Modifikation auf das Talent, die z.B. durch Vor-/Nachteile
     *      entsteht.
     */
    public int getModifikation( final Held held, final Talent talent ) {

        int modifikation = 0;

        setBegabungen( talent );

        if ( held.hat( begabungTalent ) ) {
            ++modifikation;
        }

        return modifikation;
    }

    private void setBegabungen( final Talent talent ) {

        if ( null == begabungTalent ) {

            begabungTalent = new VorNachteilWert( 
                    VorNachteil.BEGABUNG_TALENT,
                    new Variante( talent ) );

            begabungTalentgruppe = new VorNachteilWert( 
                        VorNachteil.BEGABUNG_TALENTGRUPPE,
                        new Variante( talent.getTalentgruppe() ) );

        } else {

            begabungTalent.setVariante( talent );

            begabungTalentgruppe.setVariante( talent.getTalentgruppe() );
        }
    }
}
