/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bernhardjung.dsaprogramm.util;

import javax.swing.AbstractButton;

import de.bernhardjung.dsaprogramm.i18n.I18n;

class UebersetzerButton implements Uebersetzbar {

    /** Der Button dessen Text uebersetzt werden soll. */
    private final AbstractButton button;
    
    /** 
     * Der Schluessel mit dem der Text fuer den Button aus der 
     * Lokalisierungsdatei geladen wird. 
     */
    private final String l10nschluesselText;
    
    /**
     * Der Schluessel um den Tooltip des Knopfs zu lokalisieren.
     */
    private final String l10nschluesselToolTip;
    
    /**
     * Der Schluessel der verwendet wird um das Mnemonic des Knopf aus
     * der Lokalisierungsdatei zu laden.
     */
    private final String l10nschluesselMnemonic;
    
    /**
     * @param derButton Der Knopf der lokalisiert werden soll.
     * @param derL10nschluesselText Der Schluessel, um den Text des Knopfs zu
     *          uebersetzen.
     * @param derL10nschluesselToolTip Der Schluessel um den Tooltip des Knopfs
     *          aus der Lokalisierungsdatei zu laden. Kann {@code null} sein.
     * @param derL10nschluesselMnemonic Der Schluessel, um das Mnemonic des
     *          Knopfs aus der Lokalisierungsdatei zu laden. Kann {@code null}
     *          sein.
     */
    public UebersetzerButton(
            final AbstractButton derButton,
            final String derL10nschluesselText,
            final String derL10nschluesselToolTip,
            final String derL10nschluesselMnemonic ) {
        
        button = derButton;
        
        l10nschluesselText = derL10nschluesselText;
        
        l10nschluesselToolTip = derL10nschluesselToolTip;
        
        l10nschluesselMnemonic = derL10nschluesselMnemonic;
        
        neuUebersetzen();
    }
    
    /**
     * Uebersetzt den Knopf neu.
     */
    public void neuUebersetzen() {

        if ( null != l10nschluesselText ) {
            
            button.setText( I18n.i18n( l10nschluesselText ) );
        }
        
        if ( null != l10nschluesselToolTip ) {
            
            button.setToolTipText( I18n.i18n( l10nschluesselToolTip ) );
        }
        
        if ( null != l10nschluesselMnemonic ) {

            button.setMnemonic(
                    Util.getKeyCode( I18n.i18n( l10nschluesselMnemonic ) ) );
        }
    }
}
