/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bernhardjung.dsaprogramm.util;

import java.awt.Component;

import javax.swing.JScrollPane;

import com.jgoodies.uif_lite.panel.SimpleInternalFrame;

import de.bernhardjung.dsaprogramm.i18n.I18n;

class UebersetzerFrame implements Uebersetzbar {

    /** Der Frame der uebersetzt werden soll. */ 
    private final SimpleInternalFrame frame;
    
    /**
     * Falls der Inhalt des Frames uebersetzbar ist wird er hier gespeichert,
     * sonst ist die Referenz {@code null}
     */
    private final Uebersetzbar inhalt;
    
    /** Der Lokalisierungschluessel fuer den Titel des Frames. */
    private final String l10nschluessel;
    
    public UebersetzerFrame(
            final SimpleInternalFrame derFrame,
            final String derL10nschluessel ) {
        
        frame = derFrame;
        
        l10nschluessel = derL10nschluessel;
        
        Component derInhalt = frame.getContent();
        
        if ( derInhalt instanceof JScrollPane ) {
            
            derInhalt = ((JScrollPane) derInhalt).getViewport().getView();
        }

        if ( derInhalt instanceof Uebersetzbar ) {
            
            inhalt = (Uebersetzbar) derInhalt;
        
        } else {
            
            inhalt = null;
        }
    }
    
    /** 
     * Ueberstezt Titel und ggf. Inhalt des Frames neu. 
     */
    public void neuUebersetzen() {
        
        frame.setTitle( "<html><b>" + I18n.i18n( l10nschluessel ) );
        
        if ( null != inhalt ) {
            
            inhalt.neuUebersetzen();
        }
    }

}
