/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.vornachteile.anpasser;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

class AnpasserHerausragendeEigenschaft implements Anpasser {

    /** Die Eigenschaft auf die sich die herausragende Eigenschaft bezieht. */
    private final Eigenschaft eigenschaft;

    /**
     * @param dieEigenschaft auf die sich Herausragende Eigenschaft bezieht.
     */
    AnpasserHerausragendeEigenschaft( final Eigenschaft dieEigenschaft ) {

        eigenschaft = dieEigenschaft;
    }

    public final void eingefuegt( final Held held, final VorNachteilWert wert ) {
        setEigenschaft( held, wert.getStufe() );
    }

    public final void entfernt( final Held held, final VorNachteilWert wert ) {
        setEigenschaft( held, 0 );
    }

    /**
     * Setzt die Startstufe der Eigenschaft auf {@code 14 + bonus}.
     *
     * @param held dessen Startstufe eingestellt werden soll.
     * @param bonus auf die Startstufe.
     */
    private final void setEigenschaft(
            final Held held,
            final int bonus ) {

        final int startstufe = 14 + bonus;

        held.getWert( eigenschaft ).setBasisstufe( startstufe );
    }

    public final void stufeGeaendert(
            final Held held,
            final VorNachteilWert wert ) {
        setEigenschaft( held, wert.getStufe() );
    }

    public void geaendertVariante(
            final Held held,
            final Object alteVariante,
            final Object neueVariante ) {
        // nichts zu tun.
    }


}
