/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.vornachteile.kosten;

import java.util.Arrays;
import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

class KostenAngerechneteVariante implements Kosten {

    /** Die GP, die normal fuer den Vor-/Nachteil zu zahlen sind. */
    private final int gpKosten;

    /** Der Vor-/Nachteil, der angerechnet wird. */
    private final VorNachteil vorNachteil;

    /**
     * @param dieGpKosten
     *            die normal fuer den Vor-/Nachteil zu zahlen sind.
     * @param derVorNachteil
     *            dessen Kosten evtl. angerechnet werden.
     */
    KostenAngerechneteVariante(
            final int dieGpKosten,
            final VorNachteil derVorNachteil ) {

        gpKosten = dieGpKosten;

        vorNachteil = derVorNachteil;
    }

    /** {@inheritDoc} */
    public int getAbbaukosten(final Held held, final VorNachteilWert wert) {
        return -100 * gpKosten;
    }

    /** @return den Vor-/Nachteil der angerechnet wird. */
    public Collection<Object> getAbhaengigkeiten() {
        return Arrays.asList( (Object) vorNachteil );
    }

    public int getGpKosten() {
        return gpKosten;
    }

    public int getGpKosten(final VorNachteilWert wert) {
        return gpKosten;
    }

    public int getGpKosten(final Held held, final VorNachteilWert wert) {

        if ( held.hatVorgegeben( wert ) ) {
            return 0;
        }

        int verbilligung = 0;

        if ( held.hatVorgegeben( vorNachteil ) ) {

            final Object variante = wert.getVariante();

            for ( final VorNachteilWert resistenz :
                                            held.getWerte( vorNachteil ) ) {

                if ( resistenz.getVariante().equals( variante ) ) {

                    verbilligung = resistenz.getGpKosten();
                }
            }
        }

        return gpKosten - verbilligung;
    }

    /** @return 0 */
    public int getSenkungskosten(final Lernmethode lernmethode) {
        return 0;
    }

    /** @return {@code false} */
    public final boolean istSenkbar() {
        return false;
    }
}
