/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.vornachteile.kosten;

import java.util.Collection;
import java.util.Collections;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

class KostenFest implements Kosten {

    /** Die GP-Kosten. */
    private final int gpKosten;

    KostenFest( final int dieGpKosten ) {
        gpKosten = dieGpKosten;
    }

    /** @return eine leere Liste. */
    @SuppressWarnings( "unchecked" )
    public Collection<Object> getAbhaengigkeiten() {
        return Collections.EMPTY_LIST;
    }

    /** @return die GP-Kosten mal -100. */
    public int getAbbaukosten(final Held held, final VorNachteilWert wert) {
        return -100 * gpKosten;
    }

    /** {@inheritDoc} */
    public int getGpKosten() {
        return gpKosten;
    }

    /** {@inheritDoc} */
    public int getGpKosten(final VorNachteilWert wert) {
        return gpKosten;
    }

    /** {@inheritDoc} */
    public int getGpKosten( final Held held, final VorNachteilWert wert ) {

        if ( held.hatVorgegeben( wert ) ) {

            return 0;
        }

        // Besitzt der Held eine schwaechere Auspraegung des Vor-/Nachteils
        // als vorgegeben, so wird die schwaechere Auspraegung angerechnet.
        // Daher wird jetzt die Liste der schwaecheren Auspraegungen durchlaufen
        // und ueberprueft.
        VorNachteil vorNachteil = wert.getVorNachteil();

        while ( vorNachteil.hatSchwaechereAuspraegung() ) {

            vorNachteil = vorNachteil.getSchwaechereAuspraegung();

            if ( held.hatVorgegeben( vorNachteil ) ) {

                return gpKosten - vorNachteil.getGpKosten();
            }
        }

        return gpKosten;
    }

    /** @return 0 */
    public int getSenkungskosten(final Lernmethode lernmethode) {
        return 0;
    }

    /** @return {@code false} */
    public final boolean istSenkbar() {
        return false;
    }
}
