/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.vornachteile.kosten;

import java.util.Arrays;
import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

class KostenVerbindungen implements Kosten {

    /** @return Soziale Anpassungsfaehigkeit. */
    public Collection<Object> getAbhaengigkeiten() {
        return Arrays.asList(
                (Object) VorNachteil.SOZIALE_ANPASSUNGSFAEHIGKEIT );
    }

    /** @return 0 */
    public int getAbbaukosten( final Held held, final VorNachteilWert wert ) {
        return 0;
    }

    /** @return 0 */
    public int getGpKosten() {
        return 0;
    }

    /** {@inheritDoc} */
    public int getGpKosten( final VorNachteilWert wert ) {

        return Math.max( 1, Math.round( wert.getStufe() / 3.0F ) );
    }

    /** {@inheritDoc} */
    public int getGpKosten(
            final Held held,
            final VorNachteilWert wert ) {

        int vorgegeben = 0;

        if ( held.hatVorgegeben( wert ) ) {

            vorgegeben = held.getVorgegeben( wert ).getStufe();

        }

        final int zuBezahlen = wert.getStufe() - vorgegeben;

        if ( 0 == zuBezahlen ) {

            return 0;
        }

        if ( held.hat( VorNachteil.SOZIALE_ANPASSUNGSFAEHIGKEIT ) ) {

            return Math.max( 1, Math.round( zuBezahlen / 5.0F ) );

        } else {

            return Math.max( 1, Math.round( zuBezahlen / 3.0F ) );

        }
    }

    /** @return 0 */
    public int getSenkungskosten(final Lernmethode lernmethode) {
        return 0;
    }

    /** @return {@code false} */
    public final boolean istSenkbar() {
        return false;
    }
}
