/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.vornachteile.voraussetzungen;

import java.util.Arrays;
import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

class VoraussetzungArkanophobie extends Voraussetzung {

    private VorNachteilWert vorurteile1Magie;
    private VorNachteilWert vorurteile2Magie;
    private VorNachteilWert vorurteile1Zauberer;
    private VorNachteilWert vorurteile2Zauberer;

    /** @return {@link Held#HAARFARBE} und {@link Held#AUGENFARBE}. */
    @Override
    public Collection<Object> getAbhaengigkeiten() {
        return Arrays.asList(
                (Object) VorNachteil.VORURTEILE_1,
                VorNachteil.VORURTEILE_2 );
    }

    /**
     * Prueft ob der Held Vorurteile gegen Zauberer/gegen Magie besitzt.
     */
    @Override
    public KonfliktBeschreibung pruefen(
            final Held held,
            final VorNachteilWert wert ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        initialisieren();

        if ( held.hat( vorurteile1Magie )
          || held.hat( vorurteile2Magie )
          || held.hat( vorurteile1Zauberer )
          || held.hat( vorurteile2Zauberer ) ) {

            beschreibung.einfuegen( I18n.i18n( "VoraussetzungArkanophobie" ) );
        }

        return beschreibung;
    }

    /**
     * Initialisiert die 4 Werte. Sie muessen verzoegert erzeugt werden, da es
     * zu einer Exception kommt, wenn sie im Konstruktor instanziert werden.
     */
    private final void initialisieren() {

        if ( null == vorurteile1Magie ) {

            final Variante magie = 
                new Variante( "VoraussetzungArkanophobie.Magie" );

            vorurteile1Magie = new VorNachteilWert(
                    VorNachteil.VORURTEILE_1, magie );

            vorurteile2Magie = new VorNachteilWert(
                    VorNachteil.VORURTEILE_2, magie );

            final Variante zauberer =
                new Variante( "VoraussetzungArkanophobie.Zauberer" );

            vorurteile1Zauberer = new VorNachteilWert(
                    VorNachteil.VORURTEILE_1, zauberer );

            vorurteile2Zauberer = new VorNachteilWert(
                    VorNachteil.VORURTEILE_2, zauberer );
        }
    }
}
