/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.vornachteile.voraussetzungen;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.varianten.Varianten;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

class VoraussetzungMaximal5Aktivierbar extends Voraussetzung {

    /** So oft darf Meisterhandwerk maximal gewaehlt werden. */
    private static final int MAXIMUM = 5;

    /** Die Varianten des Vor-/Nachteils */
    private final Varianten varianten;


    /**
     * @param dieVarianten
     *            des Vor-/Nachteils. Diese muessen nur angegeben werden wenn es
     *            sich dabei um Regelobjekte (Eigenschaften, Talente, usw.)
     *            handelt. Die Varianten duerfen nicht von einem Held abhaengig
     *            sein, d.h. beim Aufruf von
     *            {@link Varianten#getVarianten(Held)} muss es moeglich sein
     *            {@code null} zu uebergeben.
     */
    public VoraussetzungMaximal5Aktivierbar( final Varianten dieVarianten ) {

        varianten = dieVarianten;
    }

    /** @return alle moeglichen Varianten. */
    @Override
    public Collection< Object > getAbhaengigkeiten() {
        return new ArrayList< Object >( varianten.getVarianten( null ) );
    }

    @Override
    public KonfliktBeschreibung pruefen(
            final Held held,
            final VorNachteilWert wert ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        final VorNachteil vorNachteil = wert.getVorNachteil();

        final List< VorNachteilWert > werte = held.getWerte( vorNachteil );

        final int anzahl = werte.size();

        if ( ( MAXIMUM < anzahl )
          || ( ( MAXIMUM == anzahl ) && ! werte.contains( wert ) ) ) {

            beschreibung.einfuegen( String.format(
                    I18n.i18n( "VoraussetzungMaximal5Aktivierbar.Maximum" ),
                    vorNachteil.getName(),
                    MAXIMUM ) );
        }

        return beschreibung;
    }

}
