/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.vornachteile.voraussetzungen;

import java.util.Arrays;
import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

public class VoraussetzungNichtKombinierbar extends Voraussetzung {

    public static final String getNichtKombinierbar() {
        
        return I18n.i18n( "VoraussetzungNichtKombinierbar" );
    }
    
    private final VorNachteil vorNachteil;

    VoraussetzungNichtKombinierbar( final VorNachteil derVorNachteil ) {

        vorNachteil = derVorNachteil;
    }

    /**
     * @return der nicht kombinierbare Vor-/Nachteil.
     */
    @Override
    public Collection<Object> getAbhaengigkeiten() {
        return Arrays.asList( (Object) vorNachteil );
    }

    @Override
    public KonfliktBeschreibung pruefen(
            final Held held,
            final VorNachteilWert wert ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        if ( held.hat( vorNachteil ) ) {

            beschreibung.einfuegen( String.format( getNichtKombinierbar(),
                                                   vorNachteil.getName() ) );
        }

        return beschreibung;
    }

}
