/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.zauber;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.bernhardjung.dsaprogramm.basis.ElementMitNameImpl;

public class Repraesentation extends ElementMitNameImpl{

    public static final Repraesentation MAGIER =
        new Repraesentation( "Repraesentation.Magier" );
    
    public static final Repraesentation ELFEN =
        new Repraesentation( "Repraesentation.Elfen" );

    private static final Map< String, Repraesentation > REPRAESENTATIONEN_MAP =
        new HashMap< String, Repraesentation >();
        
    static {
        
        REPRAESENTATIONEN_MAP.put( MAGIER.getId(), MAGIER );
        
        REPRAESENTATIONEN_MAP.put( ELFEN.getId(), ELFEN );
    }
    
    /**
     * Prueft ob der uebergebene String die ID einer Repraesentation ist. 
     * 
     * @param string Der String fuer den geprueft werden soll, ob er eine ID 
     *          ist.
     * @return {@code true}, wenn es eine ID ist, sonst {@code false}.
     */
    public static final boolean istRepraesentationId( final String string ) {
       
        return REPRAESENTATIONEN_MAP.containsKey( string );
    }
    
    /**
     * Liefert eine Repraesentation mit einer bestimmten Id.
     *
     * @param id der gesuchten Repraesentation.
     * @return die Repraesentation mit der uebergebenen Id.
     * @throws IllegalArgumentException wenn es keinen Repraesentation mit der
     *         uebergebenen Id gibt.
     */
    public static final Repraesentation getRepraesentation( final String id ) {

        return REPRAESENTATIONEN_MAP.get( id );
    }

    /**
     * @return Eine Liste aller Repraesentationen.
     */
    public static final List< Repraesentation > getRepraesentationen() {
        
        return Arrays.asList( MAGIER, ELFEN );
    }
    
    private Repraesentation( final String dieId ) {
        
        super( dieId );
    }
}
