/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.zauber.voraussetzungen;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.zauber.Repraesentation;
import de.bernhardjung.dsaprogramm.zauber.ZauberWert;

class VoraussetzungRepraesentationBeginn implements Voraussetzung {

    /** Die Sonderfertigkeit Repraesentation. */
    private SonderfertigkeitWert repraesentationWert;

    /** @return Sonderfertigkeit Repraesentation. */
    public Collection< Object > getAbhaengigkeiten() {
        return Arrays.asList( (Object)
                Sonderfertigkeit.getSonderfertigkeit(
                        "Sonderfertigkeit.Repraesentation" ) );
    }

    /**
     * Initialisiert den {@link #repraesentationWert} mit der uebergebenen Repraesentation.
    */
    private final void initWert( final Repraesentation repraesentation ) {

        if ( null == repraesentationWert ) {

            repraesentationWert = new SonderfertigkeitWert(
                    Sonderfertigkeit.getSonderfertigkeit(
                            "Sonderfertigkeit.Repraesentation"),
                    repraesentation );

        } else {

            repraesentationWert.setVariante( repraesentation );

        }
    }

    /** {@inheritDoc} */
    public KonfliktBeschreibung pruefen(
            final Held held,
            final ZauberWert wert ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        if (  held.wirdErschaffen() ) {

            final List< Repraesentation > repraesentationen =
                wert.getZauber().getRepraesentationen();

            if ( 1 == repraesentationen.size() ) {

                final Repraesentation repraesentation = repraesentationen.get( 0 );

                initWert( repraesentation );

                if ( ! held.hat( repraesentationWert ) ) {

                    beschreibung.einfuegen( 
                            I18n.i18n( "VoraussetzungRepraesentationBeginn" ) );
                }
            }

        }

        return beschreibung;
    }

}
