/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.zauber.voraussetzungen;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Strings;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.zauber.ZauberWert;

class VoraussetzungVorteil implements Voraussetzung {

    /** Erklaert dem Benutzer, dass der Vorteil benoetigt wird. */
    static final String getBenoetigt() {
        
        return I18n.i18n( "VoraussetzungVorteil" ) ;
    }

    /** Die Vorteile von denen der Held einen besitzen muss. */
    private final List< VorNachteil > vorteile;

    /**
     * @param dieVorteile von denen einer vorausgesetzt wird.
     */
    VoraussetzungVorteil( final VorNachteil ... dieVorteile ) {

        vorteile = new ArrayList< VorNachteil >( dieVorteile.length );

        for ( final VorNachteil vorteil : dieVorteile ) {

            vorteile.add( vorteil );
        }
    }

    /** @return die vorausgesetzten Vorteile */
    public final Collection< Object > getAbhaengigkeiten() {
        return new ArrayList< Object >( vorteile );
    }

    /**
     * {@inheritDoc}
     */
    public KonfliktBeschreibung pruefen( final Held held, ZauberWert wert ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        boolean erfuelltVoraussetzung = false;

        for ( final VorNachteil vorteil : vorteile ) {

            if ( held.hat( vorteil ) ) {

                erfuelltVoraussetzung = true;

                break;
            }
        }

        if ( ! erfuelltVoraussetzung ) {

            beschreibung.einfuegen( String.format( 
                    getBenoetigt(),
                    Strings.erzeugeAufzaehlungMitOder( vorteile )));
        }

        return beschreibung;
    }

}
